%{
/*
 nec2ascii.lex by Hideki ISOZAKI, June 6, 1988

 This program converts JIS alphabet and numbers to corresponding ASCII
 code.

 Compile:
 % lex nec2ascii.lex
 % cc -o nec2ascii lex.yy.c -ll

 Usage:
 % nec2ascii < foo.txt > foo.tex
 */

#define errmess(x,y)   	{fprintf(stderr,x,y);exit(1);}
#define IT		((int)(yytext[yyleng-1])-32)
#define KIN             printf("\033\$@")
#define KOUT            printf("\033\(J")
#define CIRCLED      	printf("(%d)",IT)
#define ROMAN		makeroman((IT-20))
#define NECERR		{errmess("!! unsupported code at line %d\n",yylineno);}
#define PCNUMERAL	{if(IT<21) CIRCLED;else ROMAN;}
%}
K       [^\033\n\t]
Kin     \033\$.
Kout    \033\(.
PCnum   "-"[^\033\n\t]
%Start Normal JIS PCNUM
%%
<Normal>{Kin}/{PCnum}   {BEGIN PCNUM;}
<Normal>{Kin}           {ECHO; BEGIN JIS;}
<JIS>{Kout}             {ECHO; BEGIN Normal;}
<JIS>\n                 {ECHO;}
<JIS>\t                 {ECHO;}
<JIS>{PCnum}            {KOUT; PCNUMERAL; BEGIN PCNUM;}
<JIS>{K}{K}             {ECHO;}
<PCNUM>{Kout}           {BEGIN Normal;}
<PCNUM>\n               {ECHO;}
<PCNUM>\t               {ECHO;}
<PCNUM>{PCnum}          {PCNUMERAL;}
<PCNUM>{K}{K}           {KIN; ECHO; BEGIN JIS;}
%%
main(argc,argv)
        char **argv;
{
        BEGIN Normal; /* Starts yylex off in the right state */
        if(argc==1) {yyin = stdin; yylex(); exit(0);}
        else errmess("!! Invalid Argument for %s\n",argv[0]);
}

makeroman(x)
     int x;
{
  switch(x)
    {
    case 1: printf(" I "); break;
    case 2: printf(" II "); break;
    case 3: printf(" III "); break;
    case 4: printf(" IV "); break;
    case 5: printf(" V "); break;
    case 6: printf(" VI "); break;
    case 7: printf(" VII "); break;
    case 8: printf(" VIII "); break;
    case 9: printf(" IX "); break;
    case 10: printf(" X "); break;
    default: NECERR;
    }
}

	
