#ifndef lint
/*
   "$Header: dvikanji.c,v 1.5 88/04/26 21:39:13 spock Locked $";
*/
#endif

/*
 *	$Log:	dvikanji.c,v $
 * Revision 1.5  88/04/26  21:39:13  spock
 * *** empty log message ***
 * 
 * Revision 1.4  88/04/20  18:54:52  root
 * kanji font name fix
 * 
 * Revision 1.3  88/04/20  13:33:04  root
 * char range fix for VAX pascal compiler
 * 
 * Revision 1.2  88/04/20  11:28:20  spock
 * NTT JTeX support.
 * 
 * Revision 1.1  88/04/19  22:17:43  spock
 * Initial revision
 * 
 * Revision 1.1  86/04/21  14:44:20  nanako

 */

#define  MAIN

#include <sgtty.h>
#include <stdio.h>
#include <sys/file.h>
#include "commands.h"

#include "defs.h"
#include "global.h"

/**********************************************************************/
/***********************  external definitions  ***********************/
/**********************************************************************/

char	*index();
char	*malloc();
char	*mktemp();
char	*rindex();
char	*sprintf();
char	*strcpy();

#define NAMSIZ 76
#define SYNC 4

struct iorec {
	char		*fileptr;	/* ptr to file window */
	long		lcount;		/* number of lines printed */
	long		llimit;		/* maximum number of text lines */
	FILE		*fbuf;		/* FILE ptr */
	struct iorec	*fchain;	/* chain to next file */
	struct iorec	*flev;		/* ptr to associated file variable */
	char		*pfname;	/* ptr to name of file */
	short		funit;		/* file status flags */
	unsigned short	fblk;		/* index into active file table */
	long		fsize;		/* size of elements in the file */
	char		fname[NAMSIZ];	/* name of associated UNIX file */
	char		buf[BUFSIZ];	/* I/O buffer */
	char		window[1];	/* file window element */
};

/*-->NoSignExtend*/
/**********************************************************************/
/***************************  NoSignExtend  ***************************/
/**********************************************************************/

int
nosignextend(f, n)
register struct iorec *f;
int n;
{
  f->funit = SYNC;
  return NoSignExtend(f->fbuf,n);
}

int
NoSignExtend(fp, n)	/* return n byte quantity from file fd */
register FILE *fp;	/* file pointer    */
register int n;		/* number of bytes */
{
	register int x;	/* number being constructed */

	x = 0;
	while (n--)  {
		x <<= 8;
		x |= getc(fp);
	}
	return(x);
}

/*-->SignExtend*/
/**********************************************************************/
/****************************  SignExtend  ****************************/
/**********************************************************************/

int
signextend(f, n)
register struct iorec *f;
int n;
{
  f->funit = SYNC;
  return SignExtend(f->fbuf,n);
}

int
SignExtend(fp, n)   /* return n byte quantity from file fd */
register FILE *fp;  /* file pointer    */
register int n;     /* number of bytes */
{
	int	n1;	/* number of bytes	    */
	register int x; /* number being constructed */

	x = getc(fp);   /* get first (high-order) byte */
	n1 = n--;
	while (n--)  {
		x <<= 8;
		x |= getc(fp);
	}

/* NOTE: This code assumes that the right-shift is an arithmetic, rather
than logical, shift which will propagate the sign bit right.   According
to Kernighan and Ritchie, this is compiler dependent! */
 
	x <<= 32 - 8 * n1;
	x >>= 32 - 8 * n1;	/* sign extend */


#ifdef DEBUG
	if (Debug) {
		fprintf(stderr,"\tSignExtend(fp,%d)=%X\n",n1,x);
	}
#endif
	return(x);
}


/*-->GetBytes*/
/**********************************************************************/
/*****************************  GetBytes  *****************************/
/**********************************************************************/

void
GetBytes(fp, cp, n)	/* get n bytes from file fp */
register FILE *fp;	/* file pointer	 */
register char *cp;	/* character pointer */
register int n;		/* number of bytes  */

{
    while (n--)
	*cp++ = getc(fp);
}


void
Fatal(s)
char *s;
{
  fprintf(stderr,s);
  exit(1);
}

/*-->GetFontDef*/
/**********************************************************************/
/**************************** GetFontDef  *****************************/
/**********************************************************************/
void
getfontdef(f)
register struct iorec *f;
{
  f->funit = SYNC;
  GetFontDef(f->fbuf);
}

void
GetFontDef(DVIfile)
FILE *DVIfile;

/***********************************************************************
   Read the font  definitions as they  are in the  postamble of the  DVI
   file.  Note that the font directory  is not yet loaded.  In order  to
   adapt ourselves to the existing "verser" the following font paramters
   are  copied	 onto  output	fontno  (4   bytes),  chksum,	fontmag,
   fontnamelength (1 byte), fontname.  At the end, a -1 is put onto  the
   file.
***********************************************************************/

{
    char    *calloc ();
    unsigned char   byte;

    dvifp = DVIfile;
    while (((byte = NoSignExtend(dvifp, 1)) >= FNT_DEF1) &&
	(byte <= FNT_DEF4)) {
	switch (byte) {
	case FNT_DEF1:
	    ReadFontDef(NoSignExtend(dvifp, 1));
	    break;
	case FNT_DEF2:
	    ReadFontDef(NoSignExtend(dvifp, 2));
	    break;
	case FNT_DEF3:
	    ReadFontDef(NoSignExtend(dvifp, 3));
	    break;
	case FNT_DEF4:
	    ReadFontDef(NoSignExtend(dvifp, 4));
	    break;
	default:
	    Fatal("Bad byte value in font defs");
	    break;
	}
    }
    if (byte != POST_POST)
	Fatal("POST_POST missing after fontdefs");
}

/*-->PixRound*/
/**********************************************************************/
/*****************************  PixRound  *****************************/
/**********************************************************************/

int
PixRound(x)		/* return rounded number of pixels */
register int x;		/* in DVI units     */
{
	return((int)((x + (conv >> 1)) / conv));
}

/*-->ReadFontDef*/
/**********************************************************************/
/****************************  ReadFontDef  ***************************/
/**********************************************************************/

int
ReadFontDef(k)
int k;
{
	int	t, i, fdir, lastchar;
	register struct font_entry *tfontptr;	/* font_entry pointer  */
	register struct char_entry *tcharptr;	/* char_entry pointer  */

	if (CheckFontNum(k)) return;

	if ((tfontptr = NEW(struct font_entry)) == NULL)
		Fatal("can't malloc space for font_entry");
	tfontptr->next = hfontptr;
	fontptr = hfontptr = tfontptr;

	tfontptr->gfxSetNumber = 0;
	tfontptr->k = k;
	tfontptr->c = NoSignExtend(dvifp, 4); /* checksum */
	tfontptr->s = NoSignExtend(dvifp, 4); /* space size */
	tfontptr->d = NoSignExtend(dvifp, 4); /* design size */
	tfontptr->a = NoSignExtend(dvifp, 1); /* area length for font name */
	tfontptr->l = NoSignExtend(dvifp, 1); /* device length */
	GetBytes(dvifp, tfontptr->n, tfontptr->a+tfontptr->l);
	tfontptr->n[tfontptr->a+tfontptr->l] = '\0';

	/* check font name is kanji or not */

	if ((!strncmp(tfontptr->n,"jisj",4) 
	     &&(i=subfont_num(tfontptr->n+4,'j'))>0) ||
	     (!strncmp(tfontptr->n,"dmj",3) 
	     &&(i=subfont_num(tfontptr->n+3,'d'))>0) ||
	     (!strncmp(tfontptr->n,"dgj",3) 
	     &&(i=subfont_num(tfontptr->n+3,'d'))>0)) { 
	  tfontptr->gfxSetNumber = i;
	  tfontptr->isDownLoad = 2;
	} else
	  tfontptr->isDownLoad = 0;
}

int
KanjiFont()
{
  return (fontptr->isDownLoad == 2);
}

/*-->SetChar*/
/**********************************************************************/
/*****************************  SetChar  ******************************/
/**********************************************************************/

void
Kanji(k,t)
int *k, *t;
{
	  ku_ten_compute(fontptr->gfxSetNumber,*k,k,t);
	  *k += 0x80;
}


BOOLEAN
CheckFontNum(k)
int	k;
{
	register struct	font_entry	*tfontptr;

	tfontptr = hfontptr;
	while(tfontptr != NULL) {
		if (tfontptr->k == k)
			return(TRUE);
		tfontptr = tfontptr->next;
	}
	return(FALSE);
}


/*-->SetFntNum*/
/**********************************************************************/
/****************************  SetFntNum  *****************************/
/**********************************************************************/

/*
 *  this routine is used to specify the font to be used in printing future
 *  characters
 */

void
SetFntNum(k)
int k;
{
	fontptr = hfontptr;
	while ((fontptr!=NULL) && (fontptr->k!=k))
		fontptr = fontptr->next;
	if (fontptr == NULL)
		Fatal("font %d undefined", k);
}

/* ku-ten code to code and font number ( form NttJTeX) */

compute_f_c(ku,ten)
int ku,ten;
{
  int f,c,n;
  
  if ((ku<=0)||((ku>=9)&&(ku<=15))||(ku>84))
    /* @<Report invalid ku in JIS code and |goto exit|@>; */
    return 256+1;
  if ((ten<1)||(ten>94))
    /* then @<Report invalid ten in JIS code and |goto exit|@>; */
    return 256+1;
  if (ku<=8) {
    switch(ku) {
    case 1: f=1; c=ten; break;
    case 2: f=1; c=ten+100; break;
    case 3: f=2; c=ten+32; break;
    default:f=ku-1; c=ten;
    }
  } else if (ku<=47) { /* then {Daiichi Suijun} */
    n=(ku-16)*94+ten-1; f=(n / 256)+8; c=n-(f-8)*256;
  } else { /* {Daini Suijun} */
    n=(ku-48)*94+ten-1; f=(n / 256)+20; c=n-(f-20)*256;
  }
  return 256*f+c;
}

/* @<Utility functions and procedures for Japanese@>= */
ku_ten_compute(f,c,k,t)
int f,c;
int *k,*t;
{
  int ku, ten, n;
  if (f<=7) 
    switch(f) {
    case 1: 
      if (c>=100) {
	ku=2; ten=c-100;
      }else {
	ku=1; ten=c;
      }
      break;
    case 2: 
      ku=3; ten=c-32;
      break;
    default: 
      ku=f+1; ten=c;
    }
  else if (f<=19) { /* then {Daiichi Suijun} */
    n=(f-8)*256+c; ku=(n / 94)+16; ten=(n % 94)+1;
  }else { /* {Daini Suijun} */
    n=(f-20)*256+c; ku=(n / 94)+48; ten=(n % 94)+1;
  }
  *k = ku+32;
  *t = ten+32;
}

struct sub_name {
  char *name;
  int  no;
} kanji_font_name[] = {
      {"sy",1},
      {"roma",2},
      {"hira",3},
      {"kata",4},
      {"greek",5},
      {"russian",6},
      {"keisen",7},
      {"ka",8},
      {"kb",9},
      {"kc",10},
      {"kd",11},
      {"ke",12},
      {"kf",13},
      {"kg",14},
      {"kh",15},
      {"ki",16},
      {"kj",17},
      {"kk",18},
      {"kl",19},
      {"km",20},
      {"kn",21},
      {"ko",22},
      {"kp",23},
      {"kq",24},
      {"kr",25},
      {"ks",26},
      {"kt",27},
      {"ku",28},
      {"kv",29},
      {"kw",30},
      {"kx",31},
      {"ky",32},
      {"kz",33}
};

int
subfont_num(name,c)
char *name;
int  c;
{
  int i;
  char *n,*m;
  struct sub_name *font;

  font = kanji_font_name;
  for(i=0;i<(sizeof(kanji_font_name)/sizeof(kanji_font_name[0]));i++) {
    n = font-> name;
    m = name;
    while(*n++ == *m++)
      if(*n=='\0') 
        return font->no;
    font++;
  }
  return -1;
}
