#include <stdio.h>
#include "pxl.h"

read_PXL_bitmap(ch, g)
ubyte ch;
register struct glyph *g;
{
	register struct bitmap *bitmap;
	register int i, j, bitwidth;
	register char *bitsp;
	register file_bytes_wide;
	FILE *pxlfp;

	pxlfp = current_font->file;
	fseek(pxlfp, g->addr, 0);
	bitmap = &g->bitmap;

	/* in file, bitmap rows are multiples of 32 bits wide */
	file_bytes_wide = ROUNDUP(bitmap->w, BITS_PER_LONG)*BYTES_PER_LONG;
	/* width must be multiple of 16 bits for raster_op */
	bitwidth = bitmap->bytes_wide = ROUNDUP(bitmap->w, BITS_PER_SHORT)*BYTES_PER_SHORT;
	bitsp = bitmap->bits = malloc((unsigned) bitmap->h * bitmap->bytes_wide);
	if (bitsp == NULL)
		error("xdvi: Can't allocate bitmap for character %d of font %s (%d by %d)",
		    ch, current_font->fontname, bitmap->h, bitmap->w);
	for (i = 0; i < bitmap->h; i += 1)
		for (j = 0; j < file_bytes_wide; j += 1)
			if (j < bitwidth)
				*bitsp++ = reverse_byte[one(pxlfp)];
			else
				one(pxlfp);
}


read_PXL_glyphs(fontp)
register struct font *fontp;
{
	register struct glyph *g;
	FILE *pxlfp;
	long checksum, magnify, design_size;
	long font_dir_ptr, pxl_id_word;

	pxlfp = fontp->file;
	/* seek to trailer info */
	fseek(pxlfp, (long) -(5 * BYTES_PER_LONG), 2);
	checksum = four(pxlfp);
	magnify = four(pxlfp);
	design_size = four(pxlfp);
	font_dir_ptr = sfour(pxlfp) * 4;
	pxl_id_word = four(pxlfp);
#ifdef lint
	magnify = design_size = pxl_id_word = magnify;
#endif
	/* seek to font directory */
	fseek(pxlfp, font_dir_ptr, 0);
	for (g = &fontp->glyph[0]; g < &fontp->glyph[MAXCHARS]; g += 1) {
		g->bitmap.bits = NULL;
		g->bitmap.w = two(pxlfp);	/* leave this for shrink_bitmap */
		g->bitmap.h = two(pxlfp);	/* leave this for shrink_bitmap */
		g->x = stwo(pxlfp) / shrink_factor;
		g->y = stwo(pxlfp) / shrink_factor;
		g->addr = four(pxlfp) * 4;
	/*
	**  The TFM-width word is kind of funny in the units
	**  it is expressed in.  It works this way:
	**
	**  If a glyph has width 'w' in a font with design-size
	**  'd' (both in same units), the TFM-width word is
	**
	**                    (w/d) * 2^20
	**
	**  Therefore, in order to find the glyph width in
	**  DVI units (1 / 2^16 points), we take the design-size
	**  'd' (in DVI's), the magnification 'm' of the PXL file
	**  and the TFM-width word 't' to the width (in DVI's)
	**  as follows:
	**
	**                     dmt
	**                w = -----
	**                    2^20
	**
	**  But the magnification of the PXL file is just the
	**  scaled size 's' over the design size, so the final
	**  expression for the width is
	**
	**                     st
	**                w = ----
	**                    2^20
	**
	*/
		g->dvi_adv = ((double) fontp->scale * four(pxlfp)) / (1 << 20);
		g->pxl_adv = pixel_round(g->dvi_adv);
	}
}
