/*
 * font.h - some definitions for the font manipulation routines
 *           **CHANGED at Cornell for the dvi-imagen program **
 *
 * Note that our terminology is slightly different from that of Knuth
 * and Parc. Since TROFF assumes that fonts are scalable, we must
 * use a single Knuth font (which we call a family, similar to a Parc face),
 * at different magnifications (which is what we call a "font").
 */


/*
 * the layout of a font information block
 * There is one of these for every loaded "font", or
 * magnification thereof in Knuthian terms.
 *
 * Also note the strange units.  The design size is in 1/2^20 point
 * units (also called micro-points), and the individual character widths
 * are in the TFM file in 1/2^20 ems units, i.e. relative to the design size.
 */

/* change added by yaski to remember family and member for each character
   in each font. */

#define one(fp)	    (getc(fp) & 0xff)
#define sone(fp)    snum(fp, 1)
#define two(fp)     num (fp, 2)
#define four(fp)    num (fp, 4)
#define sfour(fp)   snum(fp, 4)

unsigned long num();
long snum();
typedef unsigned char   ubyte;

#define pixel_round(x)      ((long) (conv * (double) (x) + 0.5))
#define dvi_round(x)        ((long) ((double) (x) / conv + 0.5))

extern double conv;

# define TwoToTheTwenty (1024*1024)
# define MAXCHARS 256		/* make 128 for 7 bit characters */

/* Font format style flag */
# define UNKNOWN	0
# define PK_FORMAT	1
# define GF_FORMAT	2
# define PXL_FORMAT	3

enum CharStatus {NotThere, NotReady, Ready, Loaded, Wanted, Kept, InProm};

struct font
 {
    struct font * next;   	/* link to next font info block            */
    char  *filename;		/* pixel file name */
    FILE  *pxlfp;		/* pointer to pixel file */
    struct font *left, *right;	/* double link for LRU management */
    int Inumber;   		/* font number (on Imagen)                 */
    int TeXnumber;              /* font number (in DVI file)               */
    int scale;             	/* scaled size in DVI units                */
    int design;               	/* design size in DVI units                */

    unsigned char bc;       	/* beginning char defined                  */
    unsigned char ec;       	/* ending char defined                     */

    enum CharStatus state[ MAXCHARS ];
    	    			/* loaded, existant, or in Prom??          */

    unsigned char fam[ MAXCHARS ];/* added to store imagen family */
    unsigned char mem[ MAXCHARS ]; /* and member */

				/* following are in pixels                 */
    short H[ MAXCHARS ];    	/* height for each glyph raster            */
    short W[ MAXCHARS ];    	/* width for each glyph raster             */
    short Y[ MAXCHARS ];    	/* Y-offset for each glyph raster          */
    short X[ MAXCHARS ];    	/* X-offset for each glyph raster          */
    short pxl_adv[ MAXCHARS ];	/* pixels to move reference point          */
    long  dvi_adv[ MAXCHARS ];	/* DVI units to move reference point       */

    char  *Glyph[ MAXCHARS ];	/* pointers to actual glyph bitmaps        */
    char  fontname[100];
    short format;		/* font format style */
    long  glyph_ptr[256];	/* Seek locations of each glyph */
 };

extern struct font *fontListHead; /* head of linked list of font structures */
int debug;			        /* true for debugging information */
