/*
**      (C) Hideki ISOZAKI for GF fonts
**       I borrowed GF parser from Beebe's program.
*/

#include <stdio.h>
#include "font.h"
#include "gf.h"

read_GF_font(fontp)
  struct font *fontp;
{
  FILE *pxl_fp;
  ubyte pxlch;
  int i;

  pxl_fp = fontp->pxlfp;
  fseek(pxl_fp, -4L, 2); 
  while((pxlch=one(pxl_fp))==GFFILLER)
    fseek(pxl_fp, -2L, 1);
  if(pxlch != GFID)
    error("GF ID %s = %d not %d\n",fontp->filename,pxlch,GFID);
  
  fseek(pxl_fp, -6L, 1); 
  if((pxlch=one(pxl_fp)) != GFPOSTPOST)
    error("GF PostPost %s = %d not %d\n",fontp->filename,pxlch,GFPOSTPOST);
  
  fseek(pxl_fp,four(pxl_fp),0);
  if((pxlch=one(pxl_fp)) != GFPOST)
    error("GF Post %s = %d not %d\n",fontp->filename,pxlch,GFPOST);

  /* Skip GFPOST args */
  fseek(pxl_fp, 36L, 1);
  
  while(((pxlch=one(pxl_fp)) == GFCHLOC) || (pxlch == GFCHLOC0)){
    i = one(pxl_fp);
    fontp->state[i] = NotReady;
    if(pxlch==GFCHLOC){
#ifdef TEXCOPY
      fontp->pxl_adv[i] = 0;
      fseek(pxl_fp,8L,1);
#else
      fontp->pxl_adv[i] = four(pxl_fp)/65536;
      fseek(pxl_fp,4L,1);
#endif
    }
    else if(pxlch==GFCHLOC0){
#ifdef TEXCOPY
      fontp->pxl_adv[i] = 0;
      fseek(pxl_fp,1L,1);
#else
      fontp->pxl_adv[i] = one(pxl_fp);
#endif
    }
    fontp->dvi_adv[i] = ((double) fontp->scale * four(pxl_fp))/(1<<20);
    fontp->glyph_ptr[i] = four(pxl_fp);
  }
  
  if(pxlch != GFPOSTPOST)
    error("There is %d between Post Charloc and PostPost.\n",pxlch);
}


#define WHITE 0
#define BLACK 1
#define SETBIT(x,y) bits[y+(x>>3)] |= (0x80>>(x & 7))
#define INVERT(x) x = 1-x 

read_GF_glyph(fontp, i)
     register struct font *fontp;
     int i;
{
  register char    *bits;
  register int m,n0,k,runlen;
  int max_m,min_m,del_m,max_n,min_n,del_n;
  int arraywidth;
  FILE *pxl_fp;
  ubyte paint_switch;
  ubyte pxlch;


  pxl_fp = fontp->pxlfp;

  if(fseek(pxl_fp,fontp->glyph_ptr[i],0) != 0)
    error("BitMap of %s char %d not found\n",fontp->filename,i);

  if((pxlch = one(pxl_fp))== GFBOC){
    if(four(pxl_fp) != i)
      error("Postamble %d points glyph %d\n",i,pxlch);
    (void)sfour(pxl_fp);
    min_m = sfour(pxl_fp);
    max_m = sfour(pxl_fp);
    min_n = sfour(pxl_fp);
    max_n = sfour(pxl_fp);
    del_m = max_m - min_m;
    del_n = max_n - min_n;
  } 
  else if(pxlch == GFBOC1){
    if(one(pxl_fp) != i)
      error("Postamble %d points glyph %d\n",i,pxlch);
    del_m = one(pxl_fp);
    max_m = one(pxl_fp);
    del_n = one(pxl_fp);
    max_n = one(pxl_fp);
    min_m = max_m - del_m;
  } 
  else 
    error("%d should be GFBOC or GFBOC1\n",pxlch);
  
  fontp->W[i] = del_m;
  fontp->H[i] = del_n + 1;
  fontp->X[i] = - min_m;
  fontp->Y[i] = max_n;

  if((fontp->W[i]<=0) || (fontp->H[i]<=0))
    fprintf(stderr,"Strange font: %s\n  width = %dpixels, height = %dpixels\n",
	    fontp->filename,fontp->W[i],fontp->H[i]);
  
  arraywidth = (fontp->W[i] + 7) >> 3 ;

  if ((bits = (char *)malloc((del_n + 1) * arraywidth)) == NULL)
    error("Out of memory while reading character %d of font %s\n",
	    i, fontp->filename);

  fontp->Glyph[i] = bits;

  for (k = 0; k < arraywidth*fontp->H[i]; k++)
      bits[k] = 0;


  m = 0;      /* Glyph array element pointer (bit) */
  n0 = 0;     /* Row offset (byte) */
  paint_switch = WHITE;

  while((pxlch=one(pxl_fp)) != GFEOC){
    switch(pxlch){
    case GFPAINT0:
      INVERT(paint_switch);
      break;
      
    case GFPAINT1:
    case GFPAINT2:
    case GFPAINT3:
      runlen = num(pxl_fp,pxlch-GFPAINT1+1);
      if (paint_switch)
	for (k = 0; k < runlen; (++m, ++k)) SETBIT(m,n0);
      else
	m += runlen;
      INVERT(paint_switch);
      break;
      
    case GFSKIP0:
      n0 += arraywidth;
      m = 0;
      paint_switch = WHITE;
      break;
      
    case GFSKIP1:
    case GFSKIP2:
    case GFSKIP3:
      n0 += arraywidth * (num(pxl_fp,pxlch-GFSKIP0)+1);
      m = 0;
      paint_switch = WHITE;
      break;
      
    case GFXXX1:
    case GFXXX2:
    case GFXXX3:
    case GFXXX4:
      fseek(pxl_fp, snum(pxl_fp,pxlch-GFXXX1+1),1);
      break;
    case GFYYY:
      fseek(pxl_fp,4L,1);
      break;
    case GFNOOP:
      break;
      
    default: 
     if ((GFPAINT0 < pxlch) && (pxlch < GFPAINT1)){
	runlen = pxlch - GFPAINT0;
	if (paint_switch)
	  for (k = 0; k < runlen; (++m, ++k)) SETBIT(m,n0);
	else
	  m += runlen;
	INVERT(paint_switch);
      }
      else if ((GFNROW0 <= pxlch) && (pxlch <= GFNROWMAX)){
	n0 += arraywidth;
	paint_switch = BLACK;
	m = pxlch - GFNROW0;
      }
      else {
	fprintf(stderr,"GF invalid %d command in %s.\n",
		pxlch,fontp->filename);
	fseek(pxl_fp,-20L,1);
	for(k=0;k<20;k++) fprintf(stderr,"%3d ",one(pxl_fp));
	fprintf(stderr,"\n");
	for(k=0;k<20;k++) fprintf(stderr,"%3d ",one(pxl_fp));
	fprintf(stderr,"\n");
      }				/* end switch */
    }
  }					/* end while() */
}
