/*
**      jdviimp.c    Imagen driver for DVI format 2
**
**      (C) Hideki ISOZAKI for GF fonts
**       This program was derived from two dviimp programs.
**       I borrowed GF parser from Beebe's program.
**
**       PK fonts support codes are based on Sakurai's dvi2ps program.
**
**  Syntax:
**      jdviimp [options] <dvi-file>
**
**  Description:
**      Translates the DVI commands in the <dvi-file> into commands
**      for the Imagen ImPrint-10 Laser Printer.  These are placed
**      into a /tmp file which is then handed to the ipr(1) program.
**
**	The -L option takes a shot at Landscape mode.
**	The -U option disables centering
**	The -I option provides one-inch margins
**      The -v option makes the output be in decoded (verbose) form
**              -v implies -s
**      The -s option makes jdviimp refrain from calling ipr(1),
**		with Impress output on stdout.
**	The -cX option requests X copies (default 1)
**      The -d option makes jdviimp produce debugging information.
**      The -D option gets everything -d does and more, including
**          dumping out the glyphs in each font in a readable format.
**
**	Following option is added by tsuru on Thu Dec 17 11:05:46 JST 1987
**	The -Dstring Causes string to be included in  the  document
**		control language string(s) for the specified file(s).
**		this option is sent to ipr(1), and you can specify
**		default option by `setenv' in your .cshrc, for example,
**		% setenv PRINTOPTION pageduplex off
**	Notice: this option has different meanings from simple -D option.
**
**      Following options are added by yaski on 23 May 1987.
**      The -Bn option starts processing from page n.  Default is start page.
**      The -Nn option processes n pages from start page. Default is the whole.
**      If you give just one option, the others will take the default value.
**      The -Pprinter option specify to use 'printer'.
*/

#include <stdio.h>
#include <curses.h>
#include <pwd.h>
#include "dvi.h"
#include "font.h"

#define MAXFONTS    256		/* Due to 6-bit field in Imagen codes */
				/* I may fix this in the next version */

#ifndef MAXFONTSOPEN
#define MAXFONTSOPEN	10	/* Max # of pixel files open at a time */
#endif
#define PIXELS_PER_INCH      300
#define HARD_XPAGE_OFFSET    160     /* Damn it */
#define PIXELS_PER_LINE      (8 * PIXELS_PER_INCH - HARD_XPAGE_OFFSET)
                                     /* 8 inches of printing area */
#define PIXELS_PER_PAGE      (11 * PIXELS_PER_INCH)
                                     /* 11 inches of printing area */

#define DEFAULT_FONT_DIRECTORY	"/usr/lib/tex/fonts"

#ifndef PK_FONT_DIRECTORY
#define PK_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif
#ifndef GF_FONT_DIRECTORY
#define GF_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif
#ifndef PXL_FONT_DIRECTORY
#define PXL_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif

struct frame
{
        long    pxl_h, dvi_h, pxl_v, dvi_v, w, x, y, z;
};

struct frame    *stack;
int             stackp;

extern int	landscape;

int         debug = 0;
int         readstd = 0;
int         verbose = 0;
int         spoolit = 1;
int	    landscape = 0;
int	    center = 1;
int         inchmargin = 0;
int	    numcopies = 1;
int         beginpage = -1000;
int         endpage = 0;
int         numpages = 10000;
char        *prntr = "tex-luis"; /* default printer is tex-luis */
struct font *fontparray[MAXFONTS];
struct font *fontListHead = NULL;
char        job_id[300];
char	    InputName[300];
int         total_pages, maxstack, Xpage_offset, Ypage_offset;
int         fontnum = 0;
double      conv;
long        numerator, denominator, magnification;
char        *output_file_name;
char        *Usage_str = "Usage: jdviimp [-U] [-I] [-L] [-B] [-N] [-E] [-v] [-s] [-d] [-D[string]] <dvi-file>";
char	ipr_D_option[128];	/* added by tsuru */

long            read_postamble();
unsigned long   num();
long            snum();
char            *malloc();
char            *mktemp();

main(argc, argv)
int     argc;
char    *argv[];
{
        FILE    *fp;
	long    last_page;     /* file offset of last page of output */
	int	nc;
/* next 9 lines added by tsuru	*/
	extern  char *getenv();
 	char	*prn, *prnopt;
 	if (prn = getenv("PRINTER"))
 		prntr = prn;
 	if (prnopt = getenv("PRINTOPTION"))
 		strcat(strcpy(ipr_D_option, "-D"), prnopt);
 	else
 		strcpy(ipr_D_option, "-Djobheader on");

/* next line added by yaski to make inchmargin on as default */
        inchmargin = 1;

	fprintf(stderr,"jdviimp PK, GF, PXL beta version by isozaki\n");
        while (argc > 1 && argv[1][0] == '-')
	{
	    switch (argv[1][1])
	    {
		case 'c':
		    if ((nc=atoi(&argv[1][2])) > 0) numcopies=nc;
		    break;
		    
		case 'B':
		    if ((nc=atoi(&argv[1][2])) > 0) beginpage=nc;
		    break;

		case 'N':
		    if ((nc=atoi(&argv[1][2])) > 0) numpages=nc;
		    break;

		case 'P':
		    prntr = &(argv[1][2]);
		    break;

		case 's':
                    spoolit = 0;
                    break;
                    
		case 'S':
                    readstd = 1;
                    break;
                    
	        case 'd':
                    debug = 1;
                    break;
                
                case 'D':
 		    if (argv[1][2]) {	/* tsuru Thu Dec 17 11:40:35 */
 			if (*ipr_D_option)
 				strcat(ipr_D_option, ","); /* hetakuso */
 			strcat(ipr_D_option, &argv[1][2]);
 		    } else {
                     	debug = 2;
 		    }
                    break;
                
                case 'v':
                    verbose = 1;
                    spoolit = 0;
                    break;

		case 'L':
		    landscape = 1;
		    break;

		case 'U':
		    center = 0;
		    break;

		case 'I':
		    inchmargin = 1;
		    break;
                
                default:
                    error(Usage_str);
	    }
            
            argv++;
            argc--;
	}
        
  if (argc != 2 && readstd != 1) error(Usage_str);
  
  if (readstd) {
    strcpy(InputName, "Standard-Input");
    fp = stdin;
  }
  else {
    if ((fp = fopen(argv[1], "r")) == NULL) {
      char longername[256];
      strcpy (longername, argv[1]);
      strcat (longername, ".dvi");
      if ((fp = fopen (longername, "r")) == NULL)
	error("jdviimp: Can't open %s", argv[1]);
    }
    strcpy(InputName, argv[1]);
  }
	    
	if (beginpage == -1000) endpage = numpages;
	  else 	endpage = beginpage + numpages - 1;

	process_preamble(fp);
	
	find_postamble(fp);
	
	last_page = read_postamble(fp);
	
	start_output();
	
	do_pages(fp, last_page);
	
	close_pixel_files();

	finish_output();
}


/*
**      process_preamble(fp)
**
**      fp is a FILE pointer for a DVI file which is positioned at the
**      beginning.
**
**      process_preamble reads the information in the preamble and stores
**      it into global variables for later use.
*/

process_preamble(fp)
FILE    *fp;
{
        ubyte   k;
	double  fraction;

        if (one(fp) != PRE)
	    error("jdviimp: DVI file doesn't start with preamble");
	    
	if (one(fp) != 2)
	    error("jdviimp: Wrong version of DVI output for this program");
	
	numerator     = four(fp);
	denominator   = four(fp);
	magnification = four(fp);
	
	fraction = (((double) numerator * magnification)
	                                 / ((double) denominator * 1000.));
	
	conv = ((fraction / 100000) / 2.54) * PIXELS_PER_INCH;
	
	k = one(fp);
	fread(job_id, sizeof(char), k, fp);
	job_id[k] = '\0';
}


/*
**      find_postamble(fp)
**
**      fp is a FILE pointer for a DVI-format file.  find_postamble locates
**      the beginning of the postamble and leaves the file ready to start
**      reading at that location.
*/

find_postamble(fp)
FILE    *fp;
{
        ubyte   byte;
	long    offset = -4;        /* At least 4 TRAILERS */
	
	do
	{
	    offset--;
	    fseek(fp, offset, 2);
	    byte = one(fp);
	} while(byte == TRAILER);
	
	if (byte != 2)
	    error("jdviimp: Wrong version of DVI output for this program");
	
	offset -= 4;
	fseek(fp, offset, 2);

	fseek(fp, four(fp), 0);
}


/*
**      long
**      read_postamble(fp)
**
**      fp is a FILE pointer for a DVI-format file.  read_postamble reads the
**      information in the postamble, storing it into global variables.
**      It also takes care of reading in all of the PXL files for the fonts
**      used in the job.
**
**      read_postamble returns the file offset of the beginning of the last
**      page of DVI output.
*/

long
read_postamble(fp)
FILE    *fp;
{
        long    last_page;
        long    tallest, widest;
        ubyte   cmnd;

	fprintf(stderr, "Reading font info");
	fflush(stderr);
        if (one(fp) != POST)
	    error("jdviimp: Postamble doesn't begin with POST");
	
	last_page = four(fp);
	
	if (numerator != four(fp)
	          ||  denominator != four(fp)
		  ||  magnification != four(fp))
	    error("jdviimp: Postamble doesn't match preamble");
	      
	tallest      = four(fp);
	if (!landscape) {
	    Ypage_offset = (PIXELS_PER_PAGE - pixel_round(tallest)) / 2;
	    widest       = four(fp);
	    Xpage_offset = (PIXELS_PER_LINE - pixel_round(widest)) / 2
                                                        + HARD_XPAGE_OFFSET;
	} else {
	    Ypage_offset = (PIXELS_PER_LINE - pixel_round(tallest))/2
					+ HARD_XPAGE_OFFSET;
	    widest       = four(fp);
	    Xpage_offset = (PIXELS_PER_PAGE - pixel_round(widest))/2;
	}

	if (!center) Ypage_offset = Xpage_offset = 0;

	if (inchmargin) Ypage_offset = Xpage_offset = PIXELS_PER_INCH;
	    
	maxstack     = two(fp);
	total_pages  = two(fp);
        
        if (debug)
            remark("tall, wide, po, stk, pages = %d, %d, %d, %d, %d",
                    tallest, widest, Xpage_offset, maxstack, total_pages);
	
	do
	{
	    switch(cmnd = one(fp))
	    {
	        case FNTDEF1:
	        case FNTDEF2:
	        case FNTDEF3:
	        case FNTDEF4:
		    define_font(fp, cmnd);
		    fprintf(stderr, ".");
		    fflush(stderr);
		    break;
		    
		case POSTPOST:
		    break;
		
		default:
		    error("jdviimp: Non-fntdef cmnd found in postamble");
	    }
	} while (cmnd != POSTPOST);
	
	fprintf(stderr, "\n");
	fflush(stderr);
	return (last_page);
}


/*
**      define_font(fp, cmnd)
**
**      fp is a FILE pointer for a DVI file which is positioned right after
**      a fntdef command.  The opcode for the command is in cmnd.
**
**      define_font reads the rest of the fntdef command and then reads in
**      the specified PXL file, adding it to the global linked-list holding
**      all of the fonts used in the job.
*/

/* TC
** 	Made to read glyph information lazily on need.
*/

define_font(fp, cmnd)
FILE    *fp;
ubyte   cmnd;
{
        struct font *fontp;
	long        checksum;
	int         len;
	char        fontname[513];
	
	if (fontnum >= MAXFONTS)
	    error("jdviimp: Too many fonts.  Maximum is %d", MAXFONTS);
	
	fontp = (struct font *) malloc( sizeof(struct font) );
	
	if (fontp == NULL)
	    error("jdviimp: Out of memory while reading font %d", fontnum);

	fontp->next      = fontListHead;
	fontListHead     = fontp;
	
        fontp->TeXnumber = num(fp, cmnd - FNTDEF1 + 1);
	fontp->Inumber   = fontnum++;
	
	fontparray[fontp->TeXnumber] = fontp;

	checksum = four(fp);
	
	fontp->scale     = four(fp);
	fontp->design    = four(fp);
	
	len = one(fp) + one(fp);
	fread(fontname, sizeof(char), len, fp);
	fontname[len] = '\0';
        strcpy(fontp->fontname, fontname);
        
	read_pxl_file(fontp, fontname);
        
        if (debug > 1)
            dump_font(fontp, fontname);
}


/*
**	LRU management for pixel files
**
**	Recently used pixel files are kept open using LRU mechanism.
**	open_fonts keeps a doubly linked list of font structures whose
**	pixel files are open, first one being the least recently used
**	font.  When a pixel file is used, its font structure is moved
**	to the tail of the list.  Actually, the doubly linked list
**	forms a loop.  Thus, moving to the tail means moving to the
**	left of the first font in the open_fonts list.  When a pixel
**	file is required but is not open yet, the first font in the
**	list is closed and removed from the LRU list.
*/

struct font *open_fonts = NULL;
int open_font_count = 0;

#define remove_entry(fp)\
{ fp->left->right = fp->right; fp->right->left = fp->left; }

#define add_to_tail(fp)\
{ fp->left = open_fonts->left; fp->right = open_fonts;\
  open_fonts->left->right = fp; open_fonts->left = fp; }

use_font(fp)
     struct font *fp;
{
  if (fp==open_fonts) {
    open_fonts = open_fonts->right;
  } else if (fp!=open_fonts->left) {
    remove_entry(fp);
    add_to_tail(fp);
  }
}
  
close_one_font()
{
  fclose(open_fonts->pxlfp);
  open_fonts->pxlfp = NULL;
  remove_entry(open_fonts);
  open_fonts = open_fonts->right;
}

close_pixel_files()
{
  struct font *first = open_fonts;
  do {
    fclose(open_fonts->pxlfp);
    open_fonts = open_fonts->right;
  } while (open_fonts!=first);
}

/*
**      read_pxl_file(fontp, fontname)
**
**      fontp points to a (struct font) which has been partially initialised
**      with the information in the DVI-file FNTDEF command.  fontname is the
**      name of a font.
**
**      read_pxl_file constructs the expected name of the font's PXL file
**      from the info in *fontp and fontname.  It then reads the glyph
**      bitmaps into memory.
**
**      Note: the 'size' of a font (i.e. the 1200 in 'cmr10.1200pxl') is
**      given by the following formula (rounded):
**
**              scaled-size   PIXELS_PER_INCH
**              ----------- * --------------- * 1000
**              design-size         200
**
**	In the actual implementation, scaled-size/design-size hasn't been
**	stored with sufficient precision, hence the messing around to find
**	its actual value.
*/
char *rotate();

read_pxl_file(fontp, fontname)
     struct font     *fontp;
     char        fontname[513];
{
  char    file_name[1025];
  int	unmodsize;
  float	realsize;
  int     pksize, pxlsize;
  
  realsize = (magnification/1000.)*((float) fontp->scale / fontp->design);
  unmodsize = (realsize * 1000) + 0.5;
  /* a real hack to correct for rounding in some cases -- rkf */
  if(unmodsize==1095) realsize = 1.095445;	/* stephalf */
  else if(unmodsize==1315) realsize=1.314534;	/* stepihalf */
  else if(unmodsize==2074) realsize=2.0736;	/* stepiv */
  else if(unmodsize==2488) realsize=2.48832;	/* stepv */
  else if(unmodsize==2986) realsize=2.985984;	/* stepiv */
  /* the remaining magnification steps are represented with sufficient
     accuracy already */
  pksize = (realsize * PIXELS_PER_INCH) + 0.5;
  pxlsize = (realsize * PIXELS_PER_INCH * 5) + 0.5;
  
  if (open_font_count == MAXFONTSOPEN) {
    close_one_font();
  } else {
    open_font_count++;
  }

  fontp->format = PK_FORMAT;  
  sprintf(file_name, "%s/%s.%dpk", PK_FONT_DIRECTORY, fontname, pksize);
  if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
    if (debug)
      fprintf(stderr,"%s [not found]\n", file_name);
    fontp->format = GF_FORMAT;
    sprintf(file_name, "%s/%s.%dgf", GF_FONT_DIRECTORY, fontname, pksize);
    if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
      if (debug)
	fprintf(stderr,"%s [not found]\n", file_name);
      fontp->format = PXL_FORMAT;
      sprintf(file_name, "%s/%s.%dpxl", PXL_FONT_DIRECTORY, fontname, pxlsize);
      if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
        remark("scale = %d, design = %d",fontp->scale, fontp->design);
        fperror("Can't find font: \"%s\"", file_name);
      }
    }
  }
  if (debug)
    fprintf(stderr,"%s [using]\n", file_name);

  
  fontp->filename = malloc(strlen(file_name)+1);
  if (fontp->filename == NULL)
    error("Out of memory while allocating filename of font %s\n",
	  fontp->filename);
  strcpy(fontp->filename, file_name);
  
  switch (fontp->format) {
    case PK_FORMAT :
      read_PK_font(fontp);
      break;
    case GF_FORMAT :
      read_GF_font(fontp);
      break;
    case PXL_FORMAT :
      read_PXL_font(fontp);
      break;
  }

  if(open_fonts==NULL) {
    open_fonts = fontp->left = fontp->right = fontp;
  } else {
    add_to_tail(fontp);
  }
}


read_glyph(fontp, i)
     register struct font *fontp;
     int i;
{
  if(fontp->pxlfp==NULL) {
    close_one_font();
    if ((fontp->pxlfp = fopen(fontp->filename, "r")) == NULL) {
      remark("scale = %d, design = %d", fontp->scale, fontp->design);
      fperror("Can't find font: \"%s\"", fontp->filename);
    }
    add_to_tail(fontp);
  } else {
    use_font(fontp);
  }

  switch (fontp->format) {
    case PK_FORMAT :
      read_PK_glyph(fontp,i);
      break;
    case GF_FORMAT :
      read_GF_glyph(fontp,i);
      break;
    case PXL_FORMAT :
      read_PXL_glyph(fontp,i);
      break;
    default :
      error("Unknown font format style.");
  }

  if (landscape) {
    char	*holdglyph;
    int	width,height,x,y;

    holdglyph = fontp->Glyph[i];
    fontp->Glyph[i] = rotate(holdglyph, fontp->H[i], fontp->W[i]);

    width  = fontp->W[i]; 
    height = fontp->H[i];

    x = fontp->X[i];
    y = fontp->Y[i];

    fontp->W[i] = height;
    fontp->H[i] = width;

    fontp->X[i] = height-y; 
    fontp->Y[i] = x;
  }

  fontp->state[i] = Ready;
  if (debug > 1)
    dump_glyph(fontp, i);
}


/*
**              start_output()
**
**      Arrange for the stuff at the beginning of the output to the Imagen
**      to be sent out, including the jobid, etc.
**
*/
char   *tmp_template = "/tmp/impXXXXXX";

start_output()
{
        char            *buffer[50];
        struct passwd   *pwbuf, *getpwuid();
        
        pwbuf = getpwuid(getuid());
        sprintf(buffer, " (%s [%d pages])", pwbuf->pw_name, total_pages);
        strcat(job_id, buffer);
	if (spoolit) {
        	output_file_name = mktemp(tmp_template);
        	ImPressStart(output_file_name, job_id, verbose);
	}
	else
        	ImPressStart((char *)0, job_id, verbose);
	
}


/*
**              do_pages(fp, last_page)
**
**      Process the pages in the FILE fp.  last_pages is the offset of the
**      last page.  (We do the pages in reverse order, of course.)
**
*/

#define PXL_H   stack[stackp].pxl_h
#define PXL_V   stack[stackp].pxl_v
#define DVI_H   stack[stackp].dvi_h
#define DVI_V   stack[stackp].dvi_v
#define WW      stack[stackp].w
#define XX      stack[stackp].x
#define YY      stack[stackp].y
#define ZZ      stack[stackp].z

#define nope(str)       error("Can't hack %s", str)

#define correct()       diff = PXL_H - pixel_round(DVI_H);  \
                        if (diff < 0)                       \
		        {                                   \
		            while (diff++)                  \
                                ImPressMplus();             \
		        }                                   \
                        else if (diff > 0)                  \
		        {                                   \
		            while (diff--)                  \
                                ImPressMminus();            \
		        }                                   \
                        PXL_H = pixel_round(DVI_H);

do_pages(fp, last_page)
FILE    *fp;
long    last_page;
{
        char        input_left = 1;
        long        next_page;
        struct font *curr_font;
        int         diff;
        struct font *ptr;
        ubyte       ch;
                
        stack = (struct frame *) malloc( sizeof(struct frame) * (maxstack+1));
        
        if (stack == NULL)
            error("Can't allocate stack space.");
            
        fseek(fp, last_page, 0);
        
        while (input_left)
	{
            ch = one(fp);
            
 	    if (ch <= SETCHAR0 + 127)
	    {
                ImPressCharacter(ch);
                DVI_H += curr_font->dvi_adv[ch];
                PXL_H += curr_font->pxl_adv[ch];
                correct();
	    }
	    else if (FNTNUM0 <= ch  &&  ch <= FNTNUM0 + 63)
	    {
	      ptr = fontparray[ch-FNTNUM0];
                
                if (ptr == NULL)
                    error("FNTNUM to non-existant font #%d", ch - FNTNUM0);
                
                curr_font = ptr;
                ImPressFont(ptr);
            }
	    else
	    {
                long    a, b;
                
		switch (ch)
		{
		    case SET1:
		        a = one(fp);
		        ImPressCharacter(a);
                        DVI_H += curr_font->dvi_adv[a];
                        PXL_H += curr_font->pxl_adv[a];
                        correct();
			break;

		    case SETRULE:
			a = sfour(fp); b = sfour(fp);
                        if (a > 0  &&  b > 0)
			{
			    correct();
                            ImPressRule(pixel_round(b), pixel_round(a));
			}
			DVI_H += b;
                        diff = pixel_round(DVI_H);
			ImPressSetX(diff);
                        PXL_H = diff;
			break;

		    case PUT1:
		        a = one(fp);
		        ImPressCharacter(a);
			break;

		    case PUTRULE:
			a = sfour(fp); b = sfour(fp);
                        if (a > 0  &&  b > 0)
			{
			    correct();
                            ImPressRule(pixel_round(b), pixel_round(a));
			}
			break;

		    case NOP:
			break;

		    case BOP:
                        a = four(fp);
			if (debug)
                            remark("Beginning page %d", a);
			fflush(stderr);
			for (b=0; b < 9; b++)
			    four(fp);
                        next_page = sfour(fp);

			/* finish if you reach beginpage - 1 */
			if (a < beginpage)
			  { input_left = 0;
			    break; };
			/* skip to next page */
			if (a > endpage)
			  { fseek(fp, next_page, 0);
			    break; };

			fprintf(stderr, "[%d",a); /* for progress report */

                        stackp = 0;
                        DVI_H = dvi_round(Xpage_offset);
                        PXL_H = Xpage_offset;
                        DVI_V = dvi_round(Ypage_offset);
                        PXL_V = Ypage_offset;
                        WW = XX = YY = ZZ = 0;
                        curr_font = NULL;
                        ImPressBeginPage();
                        ImPressSetX(PXL_H);
			ImPressSetY(PXL_V); /* added by yaski */
			break;

		    case EOP:
                        ImPressEndPage();
                        if (stackp > 0)
                            remark("Stack not empty at EOP; stackp = %d",
                                                                      stackp);
			fprintf(stderr, "]"); /* for pregress report */
                        if (next_page < 0)
                            input_left = 0;
                        else
                            fseek(fp, next_page, 0);
			fflush(stderr);
			break;

		    case PUSH:
                        stackp++;
                        if (stackp > maxstack)
                            error("More PUSHes than were promised");
                        stack[stackp] = stack[stackp - 1];
			break;

		    case POP:
                        stackp--;
                        if (stackp < 0)
                            error("More POPs than PUSHes");
                        ImPressSetX(PXL_H);
                        ImPressSetY(PXL_V);
			break;

		    case RIGHT1:
		    case RIGHT2:
		    case RIGHT3:
		    case RIGHT4:
			DVI_H += snum(fp, ch - RIGHT1 + 1);
                        diff = pixel_round(DVI_H);
			ImPressSetX(diff);
                        PXL_H = diff;
			break;

		    case X0:
		    case X1:
		    case X2:
		    case X3:
		    case X4:
			if (ch != X0)
                        {
			    XX = snum(fp, ch - X0);
                            ImPressSetSpace(pixel_round(XX));
                        }
                        ImPressSpace();
                        DVI_H += XX;
                        PXL_H += pixel_round(XX);
                        correct();
			break;

		    case W0:
		    case W1:
		    case W2:
		    case W3:
		    case W4:
			if (ch != W0)
			    WW = snum(fp, ch - W0);
			DVI_H += WW;
                        diff = pixel_round(DVI_H);
			ImPressSetX(diff);
                        PXL_H = diff;
			break;

		    case Y0:
		    case Y1:
		    case Y2:
		    case Y3:
		    case Y4:
			if (ch != Y0)
			    YY = snum(fp, ch - Y0);
			DVI_V += YY;
                        PXL_V = pixel_round(DVI_V);
                        ImPressSetY(PXL_V);
			break;

		    case Z0:
		    case Z1:
		    case Z2:
		    case Z3:
		    case Z4:
			if (ch != Z0)
			    ZZ = snum(fp, ch - Z0);
			DVI_V += ZZ;
                        PXL_V = pixel_round(DVI_V);
                        ImPressSetY(PXL_V);
			break;

		    case DOWN1:
		    case DOWN2:
		    case DOWN3:
		    case DOWN4:
			DVI_V += snum(fp, ch - DOWN1 + 1);
                        PXL_V = pixel_round(DVI_V);
                        ImPressSetY(PXL_V);
			break;

		    case FNT1:
		    case FNT2:
		    case FNT3:
		    case FNT4:
			a = num(fp, ch - FNT1 + 1);

			ptr = fontparray[a];
                
                        if (ptr == NULL)
                            error("FNT to non-existant font #%d", a);
                
                        curr_font = ptr;
                        ImPressFont(ptr);
			break;

		    case XXX1:
		    case XXX2:
		    case XXX3:
		    case XXX4:
                        /*nope("XXXX");*/
			a = num(fp, ch-XXX1+1);
			if(a > 0)
			    DoSpecial (a, fp);
			break;

		    case FNTDEF1:
		    case FNTDEF2:
		    case FNTDEF3:
		    case FNTDEF4:
                        fseek(fp, (long)(12 + ch - FNTDEF1 + 1), 1);
                        a = one(fp) + one(fp);
                        fseek(fp, a, 1);
			break;

		    case PRE:
                        error("Can't Happen:  PRE encountered.");
			break;

		    case POST:
                        error("Can't Happen:  POST encountered.");
			break;

		    case POSTPOST:
                        error("Can't Happen:  POSTPOST encountered.");
			break;
		    
		    default:
			error("Unknown Op-code: %d; Offset: %d",
                                                               ch, ftell(fp));
		} /* end switch*/
	    } /* end else (ch not a SETCHAR or FNTNUM) */
	} /* end while */
}


/*
**              finish_output()
**
**      Wrap it up.  This routine does not return, since ImPressSendIt()
**      doesn't.
**
*/

finish_output()
{
        ImPressFinish();
        if (! spoolit)
            exit(0);
	fprintf(stderr, "\nSending to %s...\n", prntr);
	fflush(stderr);
        ImPressSendIt(output_file_name, NULL, InputName,
			total_pages, numcopies, 0, 0);
}


/*
**              num(fp, size)       snum(fp, size)
**
**      Read size bytes from the FILE fp, constructing them into a
**      signed/unsigned integer.
**
*/

unsigned long
num(fp, size)
FILE    *fp;
int     size;
{
        int     i;
	long    x;
	
	x = 0;
	for (i=0; i < size; i++)
	{
	    x = x * 256 + (unsigned) fgetc(fp);
	}
	
	return(x);
}

long
snum(fp, size)
FILE    *fp;
int     size;
{
        int     i;
	long    x;
	
	x = fgetc(fp) & 0377;
        if (x & 0200)
            x -= 256;
            
	for (i=1; i < size; i++)
	{
	    x = x * 256 + (unsigned) fgetc(fp);
	}
	
	return(x);
}


/*
**              dump_font(fontp, fontname)
**
**      Debugging routine which prints a representation of the information
**      read from a PXL file into a 'struct font'.
**
*/

dump_font(fontp, fontname)
struct  font    *fontp;
char            *fontname;
{
        printf("\n+++++DUMP OF FONT \"%s\":  ", fontname);
        printf("TeXnumber, Inumber, scale, design = %d, %d, %d, %d\n",
                        fontp->TeXnumber,
                        fontp->Inumber,
                        fontp->scale,
                        fontp->design);
        printf("\tfontp = %o\n", fontp);
}

dump_glyph(fontp, i)
     struct font *fontp;
     int i;
{
  char    *ptr;
  int	  j, k, l;

  printf("\n+++++DUMP OF CHARACTER IN \"%s\":  ", fontp->fontname);
  printf("TeXnumber, Inumber, scale, design = %d, %d, %d, %d\n",
	 fontp->TeXnumber,
	 fontp->Inumber,
	 fontp->scale,
	 fontp->design);
  printf("\tfontp = %o\n", fontp);
  printf("\nCharacter %d (%s):  ", i, unctrl(i));
  printf("H, W, X, Y = %d, %d, %d, %d\n",
	 fontp->H[i],
	 fontp->W[i],
	 fontp->X[i],
	 fontp->Y[i]);
  printf("\t\tdvi_adv, pxl_adv = %d, %d\n\n",
	 fontp->dvi_adv[i],
	 fontp->pxl_adv[i]);
            
  ptr = fontp->Glyph[i];
  for (j=0; j < fontp->H[i]; j++)
    {
  	{
  	  for (l=7; l >= 0; l--)
  	    {
	      if (*ptr & (1 << l))
		putchar('@');
	      else
		putchar(' ');
	    }
                    
	  ptr++;
	}
                
      putchar('\n');
    }
}
