/*
 * output.c - implements the ImPress output primitives
 *	This is the verbose (decoded) version,
 *	along with the encoded version, default unless
 *	
 *
 * written by Bill Nowicki, September 1981
 * Stanford University
 *
 * January 1982		Bill Nowicki
 *	- Added page buffering
 *	- Added glyph deletion
 */
 
# include <stdio.h>
# include "font.h"
/* # include "imPcodes.h" */
# include "newimPcodes.h"
# include "ipr.h"
# include "impress.h"

static struct font *CurrentFont = NULL;
static FILE	   *imPressFile = NULL;
static int	   Verbose = 0;
static int	   MemoryAvailable = GlyphSize;
static int	   MemoryNeeded = 0;
static int	   Family = 95;
static int	   Member = 127;
static int	   CurrentFamily = 95;

extern char	ipr_D_option[]; /* tsuru Thu Dec 17 11:27:14 JST 1987 */
extern int	landscape;
extern char	*prntr;
  /*
   * The page buffer.  A list of Glyphs, i.e. (font,char) pairs
   * that are needed on this page, and a list of the actual typesetting
   * commands.
   */
static struct font *GlyphTable[GlyphsPerPage];
static unsigned char GlyphChars[GlyphsPerPage];
static unsigned char PageBuffer[InputSize];
static int GlyphPointer = 0, PagePointer = 0;

ImPressPutc( c )
  {
	/*
	 * Put the indicated character into the page buffer
	 */
    if (PagePointer>=InputSize) fprintf( stderr, "Input Area overflow\n");
    PageBuffer[PagePointer++] = c;
  }


ImPressBufWord( w )
 {
	/*
	 * output a (big-endian) word to the ImPress page buffer
	 */
   ImPressPutc( (w>>8)&0377 );
   ImPressPutc( (w)&0377 );
 }


ImPressPrintf( s, arg1, arg2, arg3, arg4, arg5 )
  {
    /*
     * printf the string with arguments into the page buffer
     */
    char buf[512];
    int len;
    sprintf( buf, s, arg1, arg2, arg3, arg4, arg5 );
    len = strlen(buf);
    if (PagePointer+len>=InputSize) fprintf( stderr, "Input Area overflow\n");
    strncpy( PageBuffer+PagePointer, buf, len);
    PagePointer += len;
  }


ImPressFont( f )
 struct font *f;
  {
     /*
      * switch to font f
      */
    if (CurrentFont == f) return;
    CurrentFont = f;
  }


ImPressMplus()
{
    if (Verbose)
	ImPressPrintf("@MPLUS@");
    else
	ImPressPutc( iMPLUS );
}


ImPressMminus()
{
    if (Verbose)
	ImPressPrintf("@MMINUS@");
    else
	ImPressPutc( iMMINUS );
}

ImPressMove( deltaX )
  {
    /*
     * Move the X (major axis) coordinate 
     */
    if (Verbose) ImPressPrintf( "@H %d 1@\n", deltaX);
     else {
	     ImPressPutc( iMMOVE );
	     ImPressBufWord( deltaX );
	   }
  }


ImPressSetX( x )
  {
    /*
     * set the X (major axis) coordinate 
     */
     if (Verbose) ImPressPrintf(  "@H %d 0@\n", x);
      else {
	     ImPressPutc( iABS_H );
	     ImPressBufWord( x );
	   }
  }


ImPressSetSpace( s )
  {
    /*
     * set the "word space" value
     */
     if (Verbose) ImPressPrintf(  "@SETSP %d@\n", s);
      else {
	     ImPressPutc( iSET_SP );
	     ImPressBufWord( s );
	   }
  }


ImPressSpace()
  {
     /*
      * do a default word space
      */
     if (Verbose) ImPressPrintf(  "@SP0@\n" );
      else ImPressPutc( iSP );
  }


ImPressSetY(y)
  {
    /*
     * set the Y (minor axis) coordinate 
     */
     if (Verbose) ImPressPrintf(  "@V %d 0@\n", y);
      else {
	     ImPressPutc( iABS_V );
	     ImPressBufWord( y );
	   }
  }


ImPressCharacter( c )
 unsigned char c;

  {
    /*
     * Output the character c in the current font at the current
     * position.  First we check its state (loaded, wanted, etc.)
     */

   switch (CurrentFont->state[c])
       {
       case NotReady:
       read_glyph(CurrentFont, c); /* read glyph data lazily */
	case Ready:
		CurrentFont->state[c]=Wanted;
		GlyphChars[GlyphPointer	 ]=c;
		GlyphTable[GlyphPointer++]=CurrentFont;
		MemoryNeeded += ImPressSize( CurrentFont, c );

		if (MemoryNeeded >= MemoryAvailable) 
		  { remark("IMAGEN capacity full!! Glyph memory overflow");
		    abort(); }

		CurrentFont->fam[c] = Family;
		CurrentFont->mem[c] = Member;
		if (Member-- == 0)
		  { if (Family-- == 0)
		      { remark("IMAGEN capacity full!! Too many distinct char");
			abort(); }
		    else 
		      { Member = 127; }; };

		break;

	case Loaded:
		CurrentFont->state[c]=Kept;
		GlyphChars[GlyphPointer	 ]=c;
		GlyphTable[GlyphPointer++]=CurrentFont;
		break;

	case Wanted:
	case Kept:
	case InProm:
		break;
		
	default:
		remark("\nWarning: You used an undefined character %d of %s\n",
		        c, CurrentFont->fontname );
       }
       
   if (CurrentFamily != CurrentFont->fam[c])
     { if (Verbose)  ImPressPrintf(  "@F %d@", CurrentFont->fam[c] );
       else { ImPressPutc( iSET_FAMILY );
	      ImPressPutc( CurrentFont->fam[c] ); };
       CurrentFamily = CurrentFont->fam[c]; };

   if (Verbose && CurrentFont->mem[c]=='@')
     ImPressPrintf( "@AT@" );
   else
     ImPressPutc( CurrentFont->mem[c] );
  }

ImPressGlyph( f, c )
 struct font *f;
 unsigned char c;
  {
    register char *glyph = f->Glyph[c];
    int bytes = (f->W[c]+7) / 8;
    int row, col;

    /*
     * send the indicated glyph in the font f to the Imprint file.
     * "Small" glyphs have all dimensions less than eight bits.
     */

   if (f->W[c] == 0  ||	 f->H[c] == 0)
	return;

   if (Verbose)	 
   {
	 fprintf( imPressFile,	"@BGLY" );
    fprintf( imPressFile,  " 0 %d %d %d %d %d %d %d\n", f->fam[c], f->mem[c], 
	f->pxl_adv[c], f->W[c], f->X[c], 
	f->H[c], f->Y[c]);
    fprintf( imPressFile,  "f = %o, f->TeXnumber = %d\n", f, f->TeXnumber);

    for (row=0; row < f->H[c]; row++)
      {
	for (col=0; col<bytes; col++)
	  {
	    static char map[] = { '.', 'O' };

	    putc( map[ (*glyph >> 7) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 6) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 5) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 4) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 3) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 2) & 1 ], imPressFile );
	    putc( map[ (*glyph >> 1) & 1 ], imPressFile );
	    putc( map[ (*glyph	   ) & 1 ], imPressFile );
	    if (col&1) putc( ' ', imPressFile );
	    glyph++;
	  }
	if (bytes&1) fprintf( imPressFile,  "........" );
	putc( '\n', imPressFile );
      }
	
     fprintf( imPressFile, "@\n" );
    }
    else
     {
	/*
	 * the encoded form of glyph loading
	 */
      int rotation;
      if (landscape) rotation = 0100; else rotation = 0;
	  {
	putc( iBGLY, imPressFile );
	putc(rotation | (f->fam[c]>>1), imPressFile );
	putc( ((f->fam[c]&1)<<7)|(f->mem[c]), imPressFile );
	ImPressPutWord( f->pxl_adv[c] );
	ImPressPutWord( f->W[c] );
	ImPressPutWord( f->X[c] );
	ImPressPutWord( f->H[c] );
	ImPressPutWord( f->Y[c] );
       }
       fwrite( glyph, 1, f->H[c]*bytes, imPressFile );
     }
    f->state[c] = Loaded;
  }


ImPressRule( w, h )
  {
	/*
	 * draw a rule (rectangle) at current point with given dimensions
	 */
    if (Verbose)
      {
	ImPressPrintf( "@BRULE" );
	ImPressPrintf(	" %d %d %d@\n", w, h, -h);
	return;
      }
	 ImPressPutc( iBRULE );
	 ImPressBufWord( w );
	 ImPressBufWord( h );
	 ImPressBufWord(-h );
  }


ImPressPutWord( w )
 {
	/*
	 * output a (big-endian) word to the ImPress file
	 */
   putc( (w>>8)&0377, imPressFile );
   putc( (w)&0377, imPressFile );
 }


ImPressStart( fileName, jobID, verbosity )
 char *fileName;
 char *jobID;
 int verbosity; 
  {
    /*
     * open the indicated file name for imPress output
     * Verbosity is true to use the Verbose (decoded) mode.
     * Here we write a dummy job name and ImPress version.
     * Returns nonzero if it cannot open the file.
     */
    if (fileName == (char *)0) 
	imPressFile = stdout;
    else
	imPressFile = fopen( fileName, "w" );
    if (imPressFile==NULL)
      {
	if (debug) printf( "Unable to open %s\n", fileName );
	return(TRUE);
      }
    Verbose = verbosity;
#ifdef OLDIMAGEN
    fprintf( imPressFile, "2 %s", jobID );
    putc( 0, imPressFile );
    fprintf( imPressFile, "12345678");
#endif
    if (landscape)
    {
	if(Verbose)
	{
	    fprintf(imPressFile,"@SET-HV-SYSTEM");
	    fprintf(imPressFile,"  %o\n",0101);
	}
	else
	{
	    fprintf(imPressFile,"%c%c",iSET_HV_SYSTEM,0101);
	}
    }
    MemoryNeeded = 0;
    MemoryAvailable = GlyphSize;

/* Initial family should be set before starting page making */

    if (Verbose)  fprintf(imPressFile, "@F %d@", Family );
	else {
	       fprintf(imPressFile, "%c%c", iSET_FAMILY, Family );
	     };

    return(FALSE);
  }


ImPressBeginPage()
 {
   /*
    * begin a page of an imPress file
    */
    PagePointer = 0;
    GlyphPointer = 0;
 }


ImPressEndPage()
 {
   /*
    * end a page of an impress file.
    *
    * First we delete glyphs we do not use on this page if necessary,
    * then output all the glyphs we need, then finally dump the Page buffer.
    */
     register unsigned char *glyphc;
     register struct font **fp;
     
     if (debug) 
	printf( "Begin page, Needed = %d, available = %d\n",
		 MemoryNeeded, MemoryAvailable );

     for (glyphc=GlyphChars, fp=GlyphTable; glyphc < GlyphChars+GlyphPointer;
		glyphc++, fp++)
	  switch ( (*fp)->state[ *glyphc] )
	    {
	      case Wanted:  ImPressGlyph( *fp, *glyphc );
	      default:	    (*fp)->state[ *glyphc ] = Loaded;
	    }
/*     MemoryAvailable -= MemoryNeeded; */

     if (Verbose) fprintf( imPressFile, "@PAGE@\n" );
      else putc( iPAGE, imPressFile );
     fwrite( PageBuffer, 1, PagePointer, imPressFile );
     if (Verbose) fprintf( imPressFile, "@END@\n" );
      else  putc( iENDPAGE, imPressFile );
/*     MemoryNeeded = 0; */
 }


ImPressFinish()
 {
    /*
     * End and close out the imPress file
     */
    
     if (Verbose) fprintf( imPressFile, "@EOF@" );
      else putc( iEOF, imPressFile );
    fclose( imPressFile );
 }


ImPressSize( f, c )
 register struct font *f;
 register unsigned char c;
  {
	/*
	 * Returns the size of a glyph in the ImPrint's memory.
	 * VERY dependent on the controller code.
	 */
    register int size;
    int bytes = (f->W[c]+7)/8;

    if ( ( f->pxl_adv[c] | f->W[c] | f->H[c] | abs(2*f->X[c]) | abs(2*f->Y[c]) ) < 256 ) 
	  size = 12;
     else size = 16;
    size += bytes*f->H[c];
    if (bytes&1) size += f->H[c];
    if (bytes<=2 && f->H[c]&1) size += bytes;

    return(size);
  }
  
ImPressSendIt( fileName, banner, inputName, numberPages, 
     numberCopies, writeFlag, mailFlag)
  char *fileName;		/* name of temporary file */
  char *banner;			/* banner */
  char *inputName;		/* name of input file */
  int	numberPages;		/* hint of length */
  int	numberCopies;		/* number of copies wanted */
  int	writeFlag;		/* write when done */
  int	mailFlag;		/* mail when done */
   {
	/*
	 * Use the ipr spooler to send fileName to an ImPress printer.
	 *  Use banner as the tag for the spooler, or inputName if banner is null.
	 * Note that this routine execs, so it should normally never return.
	 */
    char *tag;
    char  pages[20], copies[20], printer[20];
    char *args[20];
    int	  count = 0;

    if (banner) tag = banner;
    else if (inputName) tag = inputName;
    else tag = "stdin";

    args[count++] = "ipr";

    if (prntr)
      {
	sprintf( printer, "-P%s", prntr );
	args[count++] = printer;
      }
    if (debug)	   args[count++] = "-d";
    args[count++] = "-Limpress";
/*    args[count++] = "-Djobheader on"; tsuru Thu Dec 17 11:25:59 JST 1987*/
     args[count++] = ipr_D_option;
    args[count++] = "-r";
/*    args[count++] = "-t";
    args[count++] = tag; 
    if (numberPages>0)
      {
	sprintf( pages, "-p%d", numberPages );
	args[count++] =	 pages;
      } */
    if (numberCopies>0)
      {
	sprintf( copies, "-c%d", numberCopies );
	args[count++] =	 copies;
      } 
    if (mailFlag)  args[count++] = "-m";
    if (writeFlag) args[count++] = "-n";
    args[count++] =  fileName;
    args[count++] =  NULL;
    if (debug)
      {
	int i;
	printf( "Command line: " );
	for (i=0; i<count; i++) printf( " %s", args[i] );
	putchar( '\n' );
	fflush(stdout);
      }
    execv( COMMAND, args );
    fperror("Couldn't execute ipr.  ImPress file in %s",fileName);
  }
