/*
 *      This file is derived form pk.c of dvi2ps.
 *
 *	This file is based on `texpr/dvi2lpb/pkfont.c'.
 *	The copyright of the original is:
 *		Copyright (c) Japan TeX Users Group, 1986 1987
 */

#include	<stdio.h>
#include	"font.h"
#include	"pkcom.h"

#define	min(x,y)	((x)<=(y) ? (x) : (y))

read_PK_font(fe)
    struct font *fe;
{
    register int cmd, i, t;
    FILE *fntfp;

    fntfp = fe->pxlfp;
    i = 0;
    do {
	if(fseek(fntfp, --i, 2) == -1)
	    error("can't seek to the end of pk file %s\n", fe->filename);
    } while((t=one(fntfp)) == PK_NO_OP);
    fseek(fntfp, 0L, 0);
    if(t != PK_POST ||
       one(fntfp) != PK_PRE || one(fntfp) != PK_ID)
	error("bad pk file %s\n", fe->filename);
    fseek(fntfp, one(fntfp), 1);		/* comment */
    fe->design = four(fntfp);		/* design size */
    fseek(fntfp, 12L, 1);	        /* checksum,hppp,vppp */

    for( ; ; ) {
	switch(cmd = one(fntfp)) {

	case PK_XXX1: case PK_XXX2: case PK_XXX3: case PK_XXX4:
	    fseek(fntfp, num(fntfp, cmd-PK_XXX1+1), 1);
	    continue;

	case PK_YYY:
	    four(fntfp);
	    continue;

	case PK_NO_OP:
	    continue;

	case PK_POST:
	    break;

	default:
	    if(cmd <= PK_FLAG)
		pkcharoffset(cmd, fe, fntfp);
	    else
		error("illegal pk command %d in %s\n", cmd, fe->filename);
	    continue;
	}
	break;
    }
}

pkcharoffset(flag, fe, fntfp)
int flag;
    struct font *fe;
    FILE *fntfp;
{
    int off, pl, cc;
    register int n;

    off = ftell(fntfp) - 1;
    if((n = flag&7) == 7) {	/* long form */
	pl = four(fntfp) - 8;	/* without tfmw,dx */
	if((cc = four(fntfp)) > MAXCHARS) {
	    fprintf(stderr,"cc %d in pk file %s\n", cc, fe->filename);
	    fseek(fntfp, pl+8, 1);
	    return;
	}
	fe->dvi_adv[cc] = ((float)four(fntfp)*(float)fe->scale) /
	    (float)(1<<20);
	fseek(fntfp, 4L, 1); 	/* dx */
	n = 4;
    } else if(n >= 4) {		/* extended short form */
	pl = (n-4)*65536 + two(fntfp) - 3;	/* without tfmw */
	cc = one(fntfp);
	fe->dvi_adv[cc] = ((float)num(fntfp, 3)*(float)fe->scale) /
	    (float)(1<<20);
	n = 2;
    } else {			/* short form */
	pl = n*256 + one(fntfp) - 3;	/* without tfmw */
	cc = one(fntfp);
	fe->dvi_adv[cc] = ((float)num(fntfp, 3)*(float)fe->scale) /
	    (float)(1<<20);
	n = 1;
    }
    (void)num(fntfp, n); 		/* dy or dm */
    fe->state[cc] = NotReady;
    fe->W[cc] = num(fntfp, n);
    fe->H[cc] = num(fntfp, n);
    fe->X[cc] = snum(fntfp, n);
    fe->Y[cc] = snum(fntfp, n);
    fe->glyph_ptr[cc] = off;
#ifdef TEXCOPY
    fe->pxl_adv[cc] = 0;
#else
    fe->pxl_adv[cc] = pixel_round(fe->dvi_adv[cc]);
#endif
    fseek(fntfp, pl-5*n, 1);
}

int	width, height, dyn_f;
int	b_width, black;
char	*pixel;
char	pat[8] = {0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01};
FILE	*pkfp;

/*	character shape definition	*/

read_PK_glyph(fe, c)
register struct font *fe;
int c;
{
    int flag, n, l;

    fseek(pkfp = fe->pxlfp, fe->glyph_ptr[c], 0);
    flag = one(pkfp);
    dyn_f = flag/16;
    black = (flag&8) >> 3;
    if((n = flag&7) == 7) {	/* long form */
	l = 16;
	n = 4;
    } else if(n >= 4) {		/* extended short form */
	l = 6;
	n = 2;
    } else {			/* short form */
	l = 5;
	n = 1;
    }
    fseek(pkfp, l+5*n, 1);
    width = fe->W[c];
    height = fe->H[c];
    b_width = (fe->W[c] + 7) >> 3;
    if((pixel = (char *)calloc(height, b_width)) == NULL)
	error("Unable to allocate memory for char\n");
    if(dyn_f == 14)
	unpack_raster();
    else
	unpack_run();
    fe->Glyph[c] = pixel;
}

/*	get nybble	*/

#define get_nyb()  ((nyb_f^=1)? (nyb_w=one(pkfp))/16: nyb_w&15)

/*	get run_count/repeat_count	*/

#define get_run()\
	for( ; ; ) {\
		if((i=get_nyb()) >= 14) {\
			rep = i*2-29;\
			continue;\
		} else if(i > dyn_f)\
			run = (i-dyn_f-1)*16 + get_nyb() + dyn_f + 1;\
		else if(i > 0)\
			run = i;\
		else /* i==0 */ {\
			do	++i;\
			while((run = get_nyb()) == 0);\
			while(--i >= 0)\
				run = run*16 + get_nyb();\
			run += (13-dyn_f)*16 + dyn_f - 15;\
		}\
		if(rep >= 0)\
			break;\
		rep = run;\
	}\

/*	unpack run_encoded packet	*/

unpack_run()
{
    register char *p, *q;
    register i, run, rep, h, h1, v;
    int	nyb_f, nyb_w;

    nyb_f = 0;
    p = pixel;
    black ^= 1;
    run = 0;
    for(v=0; v < height; v += rep+1) {
	rep = 0;
	for(h=0; h < width; h=h1) {
	    if(run == 0) {
		get_run();
		black ^= 1;
	    }
	    h1 = min(h+run, width);
	    if(black) {
		h1 = min(h1, (h+8)&~7);
		p[h/8] |= pat[h&7];
		if(h1 & 7)
		    p[h1/8] &= ~pat[h1&7];
	    }
	    run -= h1-h;
	}
	q = p;	p += b_width;
	for(h=rep*b_width; --h >= 0; )
	    *p++ = *q++;
    }
}

/*	unpack raster packet	*/

unpack_raster()
{
    register char *p;
    register n, m, t, h, v;

    p = pixel;
    n = 0;
    for(v=0; v < height; ++v) {
	for(h=0; h < width; h+=m) {
	    if(n == 0) {
		t = one(pkfp);
		n = 8;
	    }
	    m = min(n, 8-(h&7));
	    m = min(m, width-h);
	    p[h/8] |= t >> (h&7);
	    t <<= m; n -= m;
	}
	if(width & 7)
	    p[width/8] &= ~pat[width&7];
	p += b_width;
    }
}
