/*
**      texcopy.c    Printing Japanese text file by IMAGEN printer
**            by Hideki ISOZAKI    10 Feb. 1989
**               NTT Software Labs.
**               isozaki%nttlab.ntt.jp@relay.cs.net
**
**  Syntax:
**      texcopy [options] [dvi-file]
**
**  Description:
*/

#include <stdio.h>
#include <sys/time.h>
#include <curses.h>
#include <pwd.h>
#include "dvi.h"
#include "font.h"

#define YES 1
#define NO  0

#define MAXFONTS    256		/* Due to 6-bit field in Imagen codes */
				/* I may fix this in the next version */

#ifndef MAXFONTSOPEN
#define MAXFONTSOPEN	10	/* Max # of pixel files open at a time */
#endif
#define PIXELS_PER_INCH      300
#define HARD_XPAGE_OFFSET    160     /* Damn it */
#define PIXELS_PER_LINE      (8 * PIXELS_PER_INCH - HARD_XPAGE_OFFSET)
                                     /* 8 inches of printing area */
#define PIXELS_PER_PAGE      (11 * PIXELS_PER_INCH)
                                     /* 11 inches of printing area */

#define DEFAULT_FONT_DIRECTORY	"/usr/lib/tex/fonts"

#ifndef PK_FONT_DIRECTORY
#define PK_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif
#ifndef GF_FONT_DIRECTORY
#define GF_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif
#ifndef PXL_FONT_DIRECTORY
#define PXL_FONT_DIRECTORY	DEFAULT_FONT_DIRECTORY
#endif

#define FOLD_LINE    (fontsize==10 ? '\001' : '\014')
#define CONT_LINE    (fontsize==10 ? '\013' : '\013')

struct frame
{
        long    pxl_h, dvi_h, pxl_v, dvi_v, w, x, y, z;
};

struct frame    *stack;
int             stackp;

char        *string;
struct      timeval date;

int         impline =0;
int         totalline =0;
int         imppage =0;
int         column =0;

extern int	landscape;

int         try_A4 = YES;
int         readstd =NO;
int         countlines =NO;
int         countpages =YES;
int         debug = NO;
int         verbose = NO;
int         spoolit = YES;
int	    landscape = NO;
int         inchmargin = YES;
int	    numcopies = 1;
int         fontsize = 10;
int         hankaku;
int         baselineskip;
int         maxlines = 50;
int         maxcolumns = 50;
int         beginpage = -1000;
int         endpage = 0;
int         numpages = 10000;
char        *prntr = "tex-luis"; /* default printer is tex-luis */
struct font *fontparray[MAXFONTS];
struct font *fontListHead = NULL;
char        job_id[300];
char	    InputName[300];
int         total_pages, maxstack, Xpage_offset, Ypage_offset;
int         fontnum = 0;
double      conv;
long        numerator, denominator, magnification;
char        *output_file_name;
char        *Usage_str = "Usage: texcopy [-Eoption] [-Pprinter] [-p] [-f] [-C] [-S] [-L] [-Fn] [-Rm] [-v] [-s] [-d] [-D[string]] [text-file]\n -E explains options.";
char	    ipr_D_option[128];	/* added by tsuru */

long            read_postamble();
unsigned long   num();
long            snum();
char            *malloc();
char            *mktemp();


void explain(c)
     char c;
{
  switch(c){
  case 'c':
    fprintf(stderr,"c: Number of copies at one job\n");
    break;

  case 'p':
    fprintf(stderr,"p: Inhibit Page Header/Footer.\n");
    break;
    
  case 'F':
    fprintf(stderr,"F: Font size. ex. 8,9,10,12\n");
    break;
    
  case 'R':
    fprintf(stderr,"R: Rows (lines) in each page\n");
    break;
    
  case 'P':
    fprintf(stderr,"P: Send the ImPress file to this printer.\n");
    break;
    
  case 's':
    fprintf(stderr,"s: Standard Output\n");
    break;
    
  case 'S':
    fprintf(stderr,"S: Standard Input\n");
    break;
    
  case 'f':
    fprintf(stderr,"f: Free Size : Do not care about A4 size.\n");
    break;
    
  case 'C':
    fprintf(stderr,"C: Count lines\n");
    break;
    
  case 'E':
    fprintf(stderr,"E: Explain options\n");
    break;
    
  case 'd':
    fprintf(stderr,"d: rough debug info\n");
    break;
    
  case 'D':
    fprintf(stderr,"D: no argument : detailed debug info.\n");
    fprintf(stderr,"D: with argument : ipr options\n");
    break;
    
  case 'v':
    fprintf(stderr,"v: Explain Output\n");
    break;
    
  case 'L':
    fprintf(stderr,"L: Rotate it by 90 deg.\n");
    break;
    
  default:
    fprintf(stderr,"%d:No such option\n",c);
  }
}
  
  
main(argc, argv)
     int     argc;
     char    *argv[];
{
  FILE    *fp;
  int	nc;
  extern  char *getenv();
  char	*prn, *prnopt;

  if (prn = getenv("PRINTER")) prntr = prn;
  if (prnopt = getenv("PRINTOPTION"))
    strcat(strcpy(ipr_D_option, "-D"), prnopt);
  else
    strcpy(ipr_D_option, "-Djobheader on");

  (void) gettimeofday(&date, (struct timezone *)0);
  string = asctime(localtime(&date.tv_sec));

  while (argc > 1 && argv[1][0] == '-')
    {
      switch (argv[1][1])
	{
	case 'c':
	  if ((nc=atoi(&argv[1][2])) > 0) numcopies=nc;
	  break;

	case 'p':
	  countpages = NO;
	  break;
	  
	case 'F':
	  if ((nc=atoi(&argv[1][2])) > 0) fontsize=nc;
	  break;
	  
	case 'R':
	  if ((nc=atoi(&argv[1][2])) > 0) maxlines=nc;
	  break;
	  
	case 'P':
	  prntr = &(argv[1][2]);
	  break;
	  
	case 's':
	  spoolit = NO;
	  break;
	  
	case 'S':
	  readstd = YES;
	  break;
	  
	case 'f':
	  try_A4 =NO;
	  break;
	  
	case 'C':
	  countlines = YES;
	  break;
	  
	case 'E':
	  explain(argv[1][2]);
	  break;
	  
	case 'd':
	  debug = YES;
	  break;
	  
	case 'D':
	  if (argv[1][2]) {	/* tsuru Thu Dec 17 11:40:35 */
	    if (*ipr_D_option) strcat(ipr_D_option, ",");
	    strcat(ipr_D_option, &argv[1][2]);
	  } else {
	    debug = 2;
	  }
	  break;
	  
	case 'v':
	  verbose = YES;
	  spoolit = NO;
	  break;
	  
	case 'L':
	  landscape = YES;
	  break;
	  
	default:
	  error(Usage_str);
	}
      
      argv++;
      argc--;
    }
  
  if (argc != 2 && readstd != YES) error(Usage_str);
  
  if (readstd) {
    strcpy(InputName, "Standard-Input");
    fp = stdin;
  }
  else {
    if ((fp = fopen(argv[1], "r")) == NULL) {
      char longername[256];
      strcpy (longername, argv[1]);
      strcat (longername, ".tex");
      if ((fp = fopen (longername, "r")) == NULL)
	error("texcopy: Can't open %s", argv[1]);
    }
    strcpy(InputName, argv[1]);
  }

  fprintf(stderr,"texcopy PK, GF, PXL beta version by isozaki.\n");
  if(try_A4) fprintf(stderr,"Trying A4 size: ");
  else fprintf(stderr,"Free size: ");
  fprintf(stderr,"file = %s, -F%d -R%d -c%d\n",
	  InputName,fontsize,maxlines,numcopies);

  hankaku = (int)(1.9*(float)fontsize);
  baselineskip = (int)(3*hankaku);
  if (try_A4){
    while((baselineskip*maxlines > (int)(10.5*PIXELS_PER_INCH)) &&
	  (baselineskip - 2*hankaku > 2)){baselineskip--;}
  }
  
  if(baselineskip - 2*hankaku <= 2){
    fprintf(stderr,"Warning: This file will exceed A4 height.\n");
  }
  

  if (beginpage == -1000) endpage = numpages;
  else 	endpage = beginpage + numpages - 1;
  
  read_postamble(fp);
  
  start_output();
  
  do_pages(fp);
  
  close_pixel_files();
  
  finish_output();
}


/*
**      long
**      read_postamble(fp)
**
**      fp is a FILE pointer for a DVI-format file.  read_postamble reads the
**      information in the postamble, storing it into global variables.
**      It also takes care of reading in all of the PXL files for the fonts
**      used in the job.
**
**      read_postamble returns the file offset of the beginning of the last
**      page of DVI output.
*/

long
read_postamble(fp)
FILE    *fp;
{
	int f;

	fprintf(stderr, "Reading font info");
	fflush(stderr);

	numerator = 25400000;
        denominator = 473628672;
	magnification = 1000;

	if (inchmargin) Ypage_offset = Xpage_offset = PIXELS_PER_INCH;
	
#define CMTT  34
	/* Define Fonts */
	for(f=1;f<=CMTT;f++){
	  define_font(fp,FNTDEF1,f);
	  fprintf(stderr, ".");
	  fflush(stderr);
	}
	fprintf(stderr, "\n");
	fflush(stderr);
}


/*
**      define_font(fp, cmnd, f)
**
**      fp is a FILE pointer for a DVI file which is positioned right after
**      a fntdef command.  The opcode for the command is in cmnd.
**
**      define_font reads the rest of the fntdef command and then reads in
**      the specified PXL file, adding it to the global linked-list holding
**      all of the fonts used in the job.
*/

/* TC
** 	Made to read glyph information lazily on need.
*/

define_font(fp, cmnd, f)
FILE    *fp;
ubyte   cmnd;
int     f;
{
        struct font *fontp;
	char        fontname[513];
	
	if (fontnum >= MAXFONTS)
	    error("texcopy: Too many fonts.  Maximum is %d", MAXFONTS);
	
	fontp = (struct font *) malloc( sizeof(struct font) );
	
	if (fontp == NULL)
	    error("texcopy: Out of memory while reading font %d", fontnum);

	fontp->next      = fontListHead;
	fontListHead     = fontp;
	
        fontp->TeXnumber = f;
	fontp->Inumber   = fontnum++;
	fontparray[fontp->TeXnumber] = fontp;

	fontp->scale     = fontsize*256*256; /* four(fp); */
	fontp->design    = fontsize*256*256; /* four(fp); */

	switch(f){
	case CMTT: 
	  if(fontsize==10)
	    sprintf(fontname,"cmrt%d\0",fontsize); 
	  else
	    sprintf(fontname,"cmtt%d\0",fontsize); 
	  break;
	case 1: sprintf(fontname,"dmjsy%d\0",fontsize); break;
	case 2: sprintf(fontname,"dmjroma%d\0",fontsize); break;
	case 3: sprintf(fontname,"dmjhira%d\0",fontsize); break;
	case 4: sprintf(fontname,"dmjkata%d\0",fontsize); break;
	case 5: sprintf(fontname,"dmjgreek%d\0",fontsize); break;
	case 6: sprintf(fontname,"dmjrussian%d\0",fontsize); break;
	case 7: sprintf(fontname,"dmjkeisen%d\0",fontsize); break;
	default: sprintf(fontname,"dmjk%c%d\0",f-8+'a',fontsize);
	}
        strcpy(fontp->fontname, fontname);
        
	read_pxl_file(fontp, fontname);
        
        if (debug > 1)
            dump_font(fontp, fontname);
}


/*
**	LRU management for pixel files
**
**	Recently used pixel files are kept open using LRU mechanism.
**	open_fonts keeps a doubly linked list of font structures whose
**	pixel files are open, first one being the least recently used
**	font.  When a pixel file is used, its font structure is moved
**	to the tail of the list.  Actually, the doubly linked list
**	forms a loop.  Thus, moving to the tail means moving to the
**	left of the first font in the open_fonts list.  When a pixel
**	file is required but is not open yet, the first font in the
**	list is closed and removed from the LRU list.
*/

struct font *open_fonts = NULL;
int open_font_count = 0;

#define remove_entry(fp)\
{ fp->left->right = fp->right; fp->right->left = fp->left; }

#define add_to_tail(fp)\
{ fp->left = open_fonts->left; fp->right = open_fonts;\
  open_fonts->left->right = fp; open_fonts->left = fp; }

use_font(fp)
     struct font *fp;
{
  if (fp==open_fonts) {
    open_fonts = open_fonts->right;
  } else if (fp!=open_fonts->left) {
    remove_entry(fp);
    add_to_tail(fp);
  }
}
  
close_one_font()
{
  fclose(open_fonts->pxlfp);
  open_fonts->pxlfp = NULL;
  remove_entry(open_fonts);
  open_fonts = open_fonts->right;
}

close_pixel_files()
{
  struct font *first = open_fonts;
  do {
    fclose(open_fonts->pxlfp);
    open_fonts = open_fonts->right;
  } while (open_fonts!=first);
}

/*
**      read_pxl_file(fontp, fontname)
**
**      fontp points to a (struct font) which has been partially initialised
**      with the information in the DVI-file FNTDEF command.  fontname is the
**      name of a font.
**
**      read_pxl_file constructs the expected name of the font's PXL file
**      from the info in *fontp and fontname.  It then reads the glyph
**      bitmaps into memory.
**
**      Note: the 'size' of a font (i.e. the 1200 in 'cmr10.1200pxl') is
**      given by the following formula (rounded):
**
**              scaled-size   PIXELS_PER_INCH
**              ----------- * --------------- * 1000
**              design-size         200
**
**	In the actual implementation, scaled-size/design-size hasn't been
**	stored with sufficient precision, hence the messing around to find
**	its actual value.
*/
char *rotate();

read_pxl_file(fontp, fontname)
     struct font     *fontp;
     char        fontname[513];
{
  char    file_name[1025];
  int	unmodsize;
  float	realsize;
  int     pksize, pxlsize;
  
  realsize = (magnification/1000.)*((float) fontp->scale / fontp->design);
  unmodsize = (realsize * 1000) + 0.5;
  /* a real hack to correct for rounding in some cases -- rkf */
  if(unmodsize==1095) realsize = 1.095445;	/* stephalf */
  else if(unmodsize==1315) realsize=1.314534;	/* stepihalf */
  else if(unmodsize==2074) realsize=2.0736;	/* stepiv */
  else if(unmodsize==2488) realsize=2.48832;	/* stepv */
  else if(unmodsize==2986) realsize=2.985984;	/* stepiv */
  /* the remaining magnification steps are represented with sufficient
     accuracy already */
  pksize = (realsize * PIXELS_PER_INCH) + 0.5;
  pxlsize = (realsize * PIXELS_PER_INCH * 5) + 0.5;
  
  if (open_font_count == MAXFONTSOPEN) {
    close_one_font();
  } else {
    open_font_count++;
  }
  
  fontp->format = PK_FORMAT;  
  sprintf(file_name, "%s/%s.%dpk", PK_FONT_DIRECTORY, fontname, pksize);
  if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
    if (debug)
      fprintf(stderr,"%s [not found]\n", file_name);
    fontp->format = GF_FORMAT;
    sprintf(file_name, "%s/%s.%dgf", GF_FONT_DIRECTORY, fontname, pksize);
    if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
      if (debug)
	fprintf(stderr,"%s [not found]\n", file_name);
      fontp->format = PXL_FORMAT;
      sprintf(file_name, "%s/%s.%dpxl", PXL_FONT_DIRECTORY, fontname, pxlsize);
      if ((fontp->pxlfp = fopen(file_name, "r")) == NULL){
        remark("scale = %d, design = %d",fontp->scale, fontp->design);
        fperror("Can't find font: \"%s\"", file_name);
      }
    }
  }
  if (debug)
    fprintf(stderr,"%s [using]\n", file_name);

  
fontp->filename = malloc(strlen(file_name)+1);
  if (fontp->filename == NULL)
    error("Out of memory while allocating filename of font %s\n",
	  fontp->filename);
  strcpy(fontp->filename, file_name);
  
  switch (fontp->format) {
    case PK_FORMAT :
      read_PK_font(fontp);
      break;
    case GF_FORMAT :
      read_GF_font(fontp);
      break;
    case PXL_FORMAT :
      read_PXL_font(fontp);
      break;
  }

  if(open_fonts==NULL) {
    open_fonts = fontp->left = fontp->right = fontp;
  } else {
    add_to_tail(fontp);
  }
}


read_glyph(fontp, i)
     register struct font *fontp;
     int i;
{
  if(fontp->pxlfp==NULL) {
    close_one_font();
    if ((fontp->pxlfp = fopen(fontp->filename, "r")) == NULL) {
      remark("scale = %d, design = %d", fontp->scale, fontp->design);
      fperror("Can't find font: \"%s\"", fontp->filename);
    }
    add_to_tail(fontp);
  } else {
    use_font(fontp);
  }

  switch (fontp->format) {
    case PK_FORMAT :
      read_PK_glyph(fontp,i);
      break;
    case GF_FORMAT :
      read_GF_glyph(fontp,i);
      break;
    case PXL_FORMAT :
      read_PXL_glyph(fontp,i);
      break;
    default :
      error("Unknown font format style.");
  }

  if (landscape) {
    char	*holdglyph;
    int	width,height,x,y;

    holdglyph = fontp->Glyph[i];
    fontp->Glyph[i] = rotate(holdglyph, fontp->H[i], fontp->W[i]);

    width  = fontp->W[i]; 
    height = fontp->H[i];

    x = fontp->X[i];
    y = fontp->Y[i];

    fontp->W[i] = height;
    fontp->H[i] = width;

    fontp->X[i] = height-y; 
    fontp->Y[i] = x;
  }

  fontp->state[i] = Ready;
  if (debug > 1)
    dump_glyph(fontp, i);
}


/*
**              start_output()
**
**      Arrange for the stuff at the beginning of the output to the Imagen
**      to be sent out, including the jobid, etc.
**
*/
char	*tmp_template = "/tmp/impXXXXXX";

start_output()
{
        char            *buffer[50];
        struct passwd   *pwbuf, *getpwuid();
        
        pwbuf = getpwuid(getuid());
        sprintf(buffer, " (%s [%d pages])", pwbuf->pw_name, total_pages);
        strcat(job_id, buffer);
	if (spoolit) {
        	output_file_name = mktemp(tmp_template);
        	ImPressStart(output_file_name, job_id, verbose);
	}
	else
        	ImPressStart((char *)0, job_id, verbose);
	
}


/*              compute_f_c(ku,ten)        */

#define NUM_TEN       94
#define BASE_FNT1      8
#define BASE_FNT2     20
#define MAX_KU0        8
#define MIN_KU1       16
#define MAX_KU1       47
#define MIN_KU2       48
#define MAX_KU2       84

int compute_f_c(ku,ten)
     int ku,ten;
{
  int f,c,n;
  if ((ku<=0 || ku>MAX_KU0) && (ku<MIN_KU1 || ku>MAX_KU2))
    fprintf(stderr,"Invalid KU : %d\n",ku);
  if (ten<=0 || ten>NUM_TEN) 
    fprintf(stderr,"Invalid TEN : %d\n",ten);
  if (ku<=MAX_KU0) {
    switch(ku){
    case 1: f=1; c=ten; break;  /* sy */
    case 2: f=1; c=ten+100; break; /* sy */
    case 3: f=2; c=ten+32; break; /* roma */
    default: f=ku-1; c=ten;  /* hira, kata, greek, russian, keisen */
    }
  }
  else 
    if(ku<=MAX_KU1){ /* Daiichi Suijun */
      n=(ku-MIN_KU1)*NUM_TEN+ten-1; 
      f=n/256+BASE_FNT1;
      c=n-(f-BASE_FNT1)*256;
    }
    else {  /* Daini Suijun */
      n=(ku-MIN_KU2)*NUM_TEN+ten-1; 
      f=n/256+BASE_FNT2; 
      c=n-(f-BASE_FNT2)*256;
    }
  return(256*f+c);
}



/*
**              do_pages(fp)
**
**      Process the pages in the FILE fp.  last_pages is the offset of the
**      last page.  (We do the pages in reverse order, of course.)
**
*/

#define PXL_H   stack[stackp].pxl_h
#define PXL_V   stack[stackp].pxl_v
#define WW      stack[stackp].w
#define XX      stack[stackp].x
#define YY      stack[stackp].y
#define ZZ      stack[stackp].z

#define nope(str)       error("Can't hack %s", str)

#define correct()       diff = PXL_H - pixel_round(DVI_H);  \
                        if (diff < 0)                       \
		        {                                   \
		            while (diff++)                  \
                                ImPressMplus();             \
		        }                                   \
                        else if (diff > 0)                  \
		        {                                   \
		            while (diff--)                  \
                                ImPressMminus();            \
		        }                                   \
                        PXL_H = pixel_round(DVI_H);

#define ASCII 0
#define JIS   1

#define TAB   9
#define LF   10
#define PAGE 12
#define ESC  27
#define SPACE 32

#define GETCHAR getseven(fp)
#define TABSKIP 8
#define MOVETO(x,y)   (ImPressSetX(x),ImPressSetY(y))

#define ZENKAKU  ((int)(2*hankaku))
#define TEXTWIDTH  ((int)(80*hankaku))
#define USECMTT {ptr=fontparray[CMTT];  \
                  if (ptr == NULL) \
                     error("FNT to non-existent font #%d",CMTT); \
                  curr_font = ptr; ImPressFont(ptr);}
#define ImPressHankaku(c) {\
        if(c!=SPACE)ImPressCharacter(c);ImPressMove(hankaku);}
#define ImPressEndRule() {MOVETO(Xpage_offset, \
			     (int)(Ypage_offset+(maxlines+1)*baselineskip)); \
			  ImPressRule(TEXTWIDTH,4);}
#define ImPressBeginRule() { \
	MOVETO(Xpage_offset,(int)(Ypage_offset-1.7*baselineskip)); \
	ImPressRule(TEXTWIDTH,4); \
	MOVETO(Xpage_offset,Ypage_offset-2*baselineskip); \
	for(i=0;InputName[i]!='\0';i++) ImPressHankaku(InputName[i]); \
        ImPressMove(5*hankaku); \
	for(i=0;string[i]!='\n';i++) ImPressHankaku(string[i]); \
	MOVETO(Xpage_offset+TEXTWIDTH-5*hankaku,Ypage_offset-2*baselineskip); \
	if(countpages){\
	       ImPressHankaku('p'); \
	       ImPressHankaku('.'); \
	       ImPressHankaku((imppage/100+48)); \
	       ImPressHankaku((imppage/10-(imppage/100)*10+48)); \
	       ImPressHankaku((imppage%10+48)); }}

#define TOTALLINE  {\
	MOVETO(Xpage_offset-10*hankaku,baselineskip*impline+Ypage_offset);\
	ImPressHankaku((totalline/10000+48));\
	ImPressHankaku((totalline/1000-(totalline/10000)*10+48));\
	ImPressHankaku((totalline/100-(totalline/1000)*10+48));\
	ImPressHankaku((totalline/10-(totalline/100)*10+48));\
	ImPressHankaku((totalline%10+48));}

do_pages(fp)
FILE    *fp;
{
  char        input_left = YES;
  struct font *curr_font;
  int         diff,i;
  struct font *ptr;
  ubyte       ch;
  
  int c1,s, a1,a2, skip;

  maxstack=1;
  stack = (struct frame *) malloc( sizeof(struct frame) * (maxstack+1));
  if (stack == NULL) error("Can't allocate stack space.");

/* Read Eval Print Loop */
  s=ASCII; 
  stackp=0;
  imppage=1;
  impline=0; totalline=0;
  column=0;
  maxcolumns = 7*PIXELS_PER_INCH/hankaku;
  fprintf(stderr, "%d ",imppage); /* for progress report */
  ImPressBeginPage();
  USECMTT;
  ImPressBeginRule();
  MOVETO(Xpage_offset,Ypage_offset); /* added by yaski */

  while((c1=GETCHAR) != -1){
    switch(c1) {
    case TAB:
      skip = 8-(column+TABSKIP)%TABSKIP;
      column += skip;
      ImPressMove(skip*hankaku);
      break;

    case ESC:
      s = 1-s; 
      a1=GETCHAR; a2=GETCHAR;
      check_escape(s,a1,a2);
      if(s == ASCII) USECMTT;
      break;

    case PAGE:
      impline=maxlines-1;
    case LF:
      if(c1 == LF) {
	totalline++;
	if(countlines) TOTALLINE;
      }
      MOVETO(Xpage_offset,baselineskip*(++impline)+Ypage_offset);
      column=0;
      if(impline >= maxlines){
	ImPressEndRule();
	ImPressEndPage();
	impline=0; ++imppage;
	fprintf(stderr, "%d ",imppage); /* for progress report */
	fflush(stderr);
	ImPressBeginPage();
	USECMTT;
	ImPressBeginRule();
	MOVETO(Xpage_offset,Ypage_offset); /* added by yaski */
      } 
      break;
      
    default:
      if(try_A4 && (column > maxcolumns)){
	USECMTT;
	ImPressHankaku(FOLD_LINE); /* left arrow */
	MOVETO(Xpage_offset,baselineskip*(++impline)+Ypage_offset);
	column=0;
	if(impline >= maxlines){
	  ImPressEndRule();
	  ImPressEndPage();
	  impline=0; ++imppage;
	  fprintf(stderr, "%d ",imppage); /* for progress report */
	  fflush(stderr);
	  ImPressBeginPage();
	  USECMTT;
	  ImPressBeginRule();
	  MOVETO(Xpage_offset,Ypage_offset); /* added by yaski */
	} 
	ImPressHankaku(CONT_LINE); /* right arrow */
      }
      print_the_char(fp,s,curr_font,c1);
    }
  } /* EOF */

  MOVETO(Xpage_offset+TEXTWIDTH-6*hankaku,Ypage_offset-2*baselineskip);
  ImPressHankaku('*');
  ImPressEndRule();
  ImPressEndPage();
  fprintf(stderr, "."); /* for pregress report */
  fflush(stderr);
}

print_the_char(fp,s,curr_font,c1)
     FILE    *fp;
     int s,c1;
     struct font *curr_font;
{
  int c2, ans, f, c;
  struct font *ptr;
  switch(s){
  case ASCII:
    column++;
    ImPressHankaku(c1);
    break;
  case JIS:
    column += 2;
    c2=GETCHAR;
    ans=compute_f_c(c1-32,c2-32);
    c=ans%256;
    f=ans/256;
    if(f==1 && c==1){ /* Zenkaku Space */
      ImPressMove(ZENKAKU);
    }
    else {
      ptr=fontparray[f];
      if (ptr == NULL)
	error("FNT to non-existent font #%d",CMTT);
      curr_font = ptr;
      ImPressFont(ptr);
      ImPressCharacter(c);
      ImPressMove(ZENKAKU);
    }
  }
}


/*              check_escape(s,a1,a2)      */

check_escape(s,a1,a2)
     int s,a1,a2;
{
  switch(s){
  case ASCII:
    if(a1 != '(' || (a2 !='J' && a2 != 'H' && a2 !='B')){
      fprintf(stderr,"Invalid Ending Sequence: %d %d\n",a1,a2);
      exit(1);
    }
    break;
  case JIS:
    if(a1 != '$' || (a2 !='@' && a2 != 'B')){
      fprintf(stderr,"Invalid Escape Sequence: %d %d\n",a1,a2);
      exit(1);
    }
    break;
  default:
    fprintf(stderr,"Invalid use of check_escape: state = %d\n",s);
    exit(1);
  }
}



/*
 **              finish_output()
 **
 **      Wrap it up.  This routine does not return, since ImPressSendIt()
 **      doesn't.
 **
 */

finish_output()
{
        ImPressFinish();
        if (! spoolit)
            exit(0);
	fprintf(stderr, "\nSending to %s...\n", prntr);
	fflush(stderr);
        ImPressSendIt(output_file_name, NULL, InputName,
			imppage, numcopies, 0, 0);
}


/*
**              num(fp, size)       snum(fp, size)
**
**      Read size bytes from the FILE fp, constructing them into a
**      signed/unsigned integer.
**
*/

unsigned long
num(fp, size)
FILE    *fp;
int     size;
{
        int     i;
	long    x;
	
	x = 0;
	for (i=0; i < size; i++)
	{
	    x = x * 256 + (unsigned) fgetc(fp);
	}
	
	return(x);
}

long
snum(fp, size)
FILE    *fp;
int     size;
{
        int     i;
	long    x;
	
	x = fgetc(fp) & 0377;
        if (x & 0200)
            x -= 256;
            
	for (i=1; i < size; i++)
	{
	    x = x * 256 + (unsigned) fgetc(fp);
	}
	
	return(x);
}


/*
**              dump_font(fontp, fontname)
**
**      Debugging routine which prints a representation of the information
**      read from a PXL file into a 'struct font'.
**
*/

dump_font(fontp, fontname)
struct  font    *fontp;
char            *fontname;
{
        printf("\n+++++DUMP OF FONT \"%s\":  ", fontname);
        printf("TeXnumber, Inumber, scale, design = %d, %d, %d, %d\n",
                        fontp->TeXnumber,
                        fontp->Inumber,
                        fontp->scale,
                        fontp->design);
        printf("\tfontp = %o\n", fontp);
}

dump_glyph(fontp, i)
     struct font *fontp;
     int i;
{
  char    *ptr;
  int	  j, k, l;

  printf("\n+++++DUMP OF CHARACTER IN \"%s\":  ", fontp->fontname);
  printf("TeXnumber, Inumber, scale, design = %d, %d, %d, %d\n",
	 fontp->TeXnumber,
	 fontp->Inumber,
	 fontp->scale,
	 fontp->design);
  printf("\tfontp = %o\n", fontp);
  printf("\nCharacter %d (%s):  ", i, unctrl(i));
  printf("H, W, X, Y = %d, %d, %d, %d\n",
	 fontp->H[i],
	 fontp->W[i],
	 fontp->X[i],
	 fontp->Y[i]);
  printf("\t\tdvi_adv, pxl_adv = %d, %d\n\n",
	 fontp->dvi_adv[i],
	 fontp->pxl_adv[i]);
            
  ptr = fontp->Glyph[i];
  for (j=0; j < fontp->H[i]; j++)
    {
  	{
  	  for (l=7; l >= 0; l--)
  	    {
	      if (*ptr & (1 << l))
		putchar('@');
	      else
		putchar(' ');
	    }
                    
	  ptr++;
	}
                
      putchar('\n');
    }
}

int getseven(fp)
     FILE *fp;
{
  int ch,k;
  if((ch=getc(fp))>127){
    fprintf(stderr,"\nInvalid char %d found at line %d.\n",ch,totalline+1);
    for(k=0;k<20;k++){
      ch=getc(fp);
      fprintf(stderr,"%d ",ch);fflush(stderr);
    }
    fprintf(stderr,"\n");exit(1);
  }
  else
    return(ch);
}
