#include <stdio.h>
#include "defs.h"
#include "global.h"

void	OpenFontFile();
struct openfont_list openfont_files[MAXOPEN+1];
				/* list of open FNT file identifiers        */
int openfont_time = 0;

/*-->OpenFontFile*/
/**********************************************************************/
/************************** OpenFontFile  *****************************/
/**********************************************************************/

void
OpenFontFile(fe)
register struct font_entry *fe;
/***********************************************************************
    The original version of this dvi driver reopened the font file  each
    time the font changed, resulting in an enormous number of relatively
    expensive file  openings.   This version  keeps  a cache  of  up  to
    MAXOPEN open files,  so that when  a font change  is made, the  file
    pointer, fntfp, can  usually be  updated from the  cache.  When  the
    file is not found in  the cache, it must  be opened.  In this  case,
    the next empty slot  in the cache  is assigned, or  if the cache  is
    full, the least used font file is closed and its slot reassigned for
    the new file.  Identification of the least used file is based on the
    counts of the number  of times each file  has been "opened" by  this
    routine.  On return, the file pointer is always repositioned to  the
    beginning of the file.

***********************************************************************/
{
    register int i,least_used,current;
    int prio;
    FILE *fntfp;

#ifdef DEBUG
    if (Debug) printf("Open Font file\n");
#endif
    openfont_time++;
    if (pfontptr == fe)
	return;                 /* we need not have been called */

    for (current = 1;
	(current <= nopen) &&
	    (openfont_files[current].openfont_file_id != fe->font_file_id);
	++current)
	;                       /* try to find file in open list */

    if (current <= nopen)       /* file already open */
    {
	if( (fntfp = openfont_files[current].openfont_file_id) != NO_FILE )
		fseek(fntfp,0,0);	/* reposition to start of file */
    }
    else                        /* file not in open list */
    {
	if (nopen < MAXOPEN)    /* just add it to list */
	    current = ++nopen;
	else                    /* list full -- find least used file, */
	{                       /* close it, and reuse slot for new file */
	    least_used = 1;
	    prio = priority(least_used);
	    for (i = 2; i <= MAXOPEN; ++i)
		if(prio > priority(i))
		    prio = priority(least_used = i);
	    if (openfont_files[least_used].openfont_file_id != NO_FILE) {
		FILE *fid;
		struct font_entry *fp;

		fid = openfont_files[least_used].openfont_file_id;
		fp=hfontptr;    /* mark file as being closed in the entry */
		while (fp!=NULL && fp->font_file_id != fid)
			fp = fp->next;
		if (fp==NULL)
		  Fatal("Open file %x not found in font entry list.\n", fid);
		else {
			fp->font_file_id = NULL;
#ifdef STATS
			if (Stats)
				fprintf(stderr, "FNT file %s closed.\n", fp->name);
#endif
			}
		fclose( fid );
		}
	    current = least_used;
	}
	if ((fntfp=BINARYOPEN(fe->name,"r")) == NULL) {
	    Warning("FNT file %s could not be opened",fe->name);
	    fntfp = NO_FILE;
	    }
	else {
#ifdef STATS
	    if (Stats)
		fprintf(stderr, "FNT file %s opened.\n", fe->name);
#endif
	    }
	openfont_files[current].openfont_file_id = fntfp;
	openfont_files[current].use_count = 0;
    }
    pfontptr = fe;			/* make previous = current font */
    fe->font_file_id = fntfp;		/* set file identifier */
    openfont_files[current].use_count++;	/* update reference count */
    openfont_files[current].last_use = openfont_time;
}

priority(i)
int i;
{
    return(openfont_files[i].use_count -
	   (openfont_time-openfont_files[i].last_use));
}
