/*
 *	Copyright (c) Japan TeX Users Group, 1986 1987
 */
/*
 *				Modified by Higuchi
 */

#include <stdio.h>
#include <newsiop/lbp.h>
#include "pkcom.h"
#include "font_types.h"
#include "font.h"

#define NCHAR		128
#define FONT_DIR	"/usr/lib/tex/fonts/pk"

#define fix(_x)		((int)((_x) + 0.5))
#define min(x,y)	((x)<=(y) ? (x) : (y))
#define error(s1, s2)	\
	{ fprintf(stderr, s1, s2); if(pk != NULL) fclose(pk); return(-1); }

#ifdef PK_FORMAT

static FILE	*pk;
static float	ds;

pk_font_def(fp, check_sum, size, design_size, dirname, filename,  mag)
struct	font *fp;
int	check_sum, size, design_size;
char	*dirname, *filename;
float	mag;
{
	int	pixels;
	float	fmag, f;
	char	fn[BUFSIZ];
	float	ratio();
	register int	cmd, i, c;

	fp->type = PK;

	fmag	= ratio(size, design_size);
	pixels	= fix(DPI*fmag*mag);

	if (dirname == NULL || *dirname == '\0')
		dirname = FONT_DIR;
	sprintf(fn, "%s/%s.%dpk", dirname, filename, pixels);

	pk = NULL;
	if((pk = fopen(fn, "r")) == NULL)
		error("can't open pk file %s\n", fn);
	i = 0;
	do {
		if(fseek(pk, --i, 2) == -1)
			error("can't seek to the end of pk file %s\n", fn);
	} while((c=get_uint(pk, 1)) == PK_NO_OP);

	fseek(pk, 0, 0);
	if(c != PK_POST || get_uint(pk,1) != PK_PRE || get_uint(pk,1) != PK_ID)
		error("bad pk file %s\n", fn);

	fseek(pk, get_uint(pk, 1), 1);		/* comment */
	get_int(pk, 4);				/* design size */

	if((c = get_int(pk, 4)) && check_sum && c != check_sum)
		; /* error("pk file %s version error\n", fn); */

	fseek(pk, 8, 1);			/* hppp,vppp */

	if((fp->data.char_pxl =
	    (struct char_pxl *)malloc(sizeof(struct char_pxl)*NCHAR)) == NULL)
		error("%s", "can't allocate memory for font table.\n");

	ds = (float)design_size * fmag / (float)(1<<20);
	
	for( ; ; ) {
		switch(cmd = get_uint(pk, 1)) {

		case PK_XXX1: case PK_XXX2: case PK_XXX3: case PK_XXX4:
			fseek(pk, get_uint(pk, cmd-PK_XXX1+1), 1);
			continue;

		case PK_YYY:
			get_int(pk,4);
			continue;

		case PK_NO_OP:
			continue;

		case PK_POST:
			break;

		default:
			if(cmd <= PK_FLAG) {
				if(def_char(cmd, fp) == -1)
					error("%s","can't allocate memory for pixel\n");
			} else
				error("illegal pk command %d\n", cmd);
			continue;
		}
		break;
	}
	fclose(pk);
	return(0);
}

/*	character shape definition	*/

static int	width, height, dyn_f;
static int	b_width, black;
static char	*pixel;
static char	pat[8] = {0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01};

def_char(n, t)
register struct font *t;
{
	register struct char_pxl *p;

	dyn_f = n/16;
	black = (n&8) >> 3;
	if((n &= 7) == 7) {		/* long form */
		/*pl =*/ get_int(pk, 4);
		p = &t->data.char_pxl[get_int(pk, 4)];
		p->tfmw = fix((float)get_int(pk, 4) * ds);
		get_int(pk, 4); 	/* dx */
		n = 4;
	} else if(n >= 4) {		/* extended short form */
		/*pl = (n-4)*65536 +*/ get_uint(pk, 2);
		p = &t->data.char_pxl[get_uint(pk, 1)];
		p->tfmw = fix((float)get_uint(pk, 3) * ds);
		n = 2;
	} else {			/* short form */
		/*pl = n*256 +*/ get_uint(pk, 1);
		p = &t->data.char_pxl[get_uint(pk, 1)];
		p->tfmw = fix((float)get_uint(pk, 3) * ds);
		n = 1;
	}
	get_uint(pk, n); 		/* dy/dm */
	width = get_uint(pk, n);
	b_width = p->width = (width+7)/8;
	height = p->height = get_uint(pk, n);
	p->x_off = get_int(pk, n);
	p->y_off = get_int(pk, n);
	pixel = p->pxl = (char *)calloc(height, b_width);
	if(pixel == NULL)
		return(-1);
	if(dyn_f == 14)
		unpack_raster();
	else
		unpack_run();
	return(0);
}

/*	get nybble	*/

#define get_nyb()  ((nyb_f^=1)? (nyb_w=get_uint(pk,1))/16: nyb_w&15)

/*	get run_count/repeat_count	*/

#define get_run()\
	for( ; ; ) {\
		if((i=get_nyb()) >= 14) {\
			rep = i*2-29;\
			continue;\
		} else if(i > dyn_f)\
			run = (i-dyn_f-1)*16 + get_nyb() + dyn_f + 1;\
		else if(i > 0)\
			run = i;\
		else /* i==0 */ {\
			do	++i;\
			while((run = get_nyb()) == 0);\
			while(--i >= 0)\
				run = run*16 + get_nyb();\
			run += (13-dyn_f)*16 + dyn_f - 15;\
		}\
		if(rep >= 0)\
			break;\
		rep = run;\
	}\


/*	unpack run_encoded packet	*/

unpack_run()
{
	register char *p, *q;
	register i, run, rep, h, h1, v;
	int	nyb_f, nyb_w;

	nyb_f = 0;
	p = pixel;
	black ^= 1;
	run = 0;
	for(v=0; v < height; v += rep+1) {
		rep = 0;
		for(h=0; h < width; h=h1) {
			if(run == 0) {
				get_run();
				black ^= 1;
			}
			h1 = min(h+run, width);
			if(black) {
				h1 = min(h1, (h+8)&~7);
				p[h/8] |= pat[h&7];
				if(h1 & 7)
					p[h1/8] &= ~pat[h1&7];
			}
			run -= h1-h;
		}
		q = p;	p += b_width;
		for(h=rep*b_width; --h >= 0; )
			*p++ = *q++;
	}
}

/*	unpack raster packet	*/

unpack_raster()
{
	register char *p;
	register n, m, t, h, v;

	p = pixel;
	n = 0;
	for(v=0; v < height; ++v) {
		for(h=0; h < width; h+=m) {
			if(n == 0) {
				t = get_uint(pk, 1);
				n = 8;
			}
			m = min(n, 8-(h&7));
			m = min(m, width-h);
			p[h/8] |= t >> (h&7);
			t <<= m; n -= m;
		}
		if(width & 7)
			p[width/8] &= ~pat[width&7];
		p += b_width;
	}
}

#endif /* PK_FORMAT */
