/*
 * DEVICE dependent, font independent
 */

#include	<stdio.h>
#include	"defs.h"
#include	"emit.h"
#include	"global.h"
#include	"ps.h"

/* define for "optimal" relative postioning, rather
   than absolute.  Relative can reduce size of postcript
   output 20% (and reduce print time by almost as much */
#define	USERELPOS	1

#ifndef	RESOLUTION
#define	RESOLUTION	300
#endif

int resolution = RESOLUTION;
int hconvresolution, vconvresolution;

int ordinal = 0;		/* page's position in the page sequence */

int hconv, vconv;		/* converts DVI units to pixels */
int ps_h = 0;			/* current h on device */
int ps_v = 0;			/* current v on device */

int cur_psf;
char psfnamebuf[256];
int devicefont = 0;
int jcompfont = -1;

#define Letter		0
#define Legal		1
#define Tabloid		2
#define B4		3
#define B5		4
#define A5		5
#define A4		6
#define A3		7
#define Envelope	8

struct {
    char	*size;		/* paper size */
    char	*command;	/* paper command */
    int		width;		/* paper width */
    int		height;		/* paper height */
} paperinfo[] = {
    {"Letter",	"letter",	612, 792},
    {"Legal",	"legal",	612, 1008},
    {"Tabloid",	"tabloid",	792, 1224},
    {"B4",	"b4", 		709, 1001},
    {"B5",	"b5",		499, 709},
    {"A5",	"a5",		420, 595},
    {"A4",	"a4",		595, 842},
    {"A3",	"a3",		842, 1191},
    {"Letter",	"envelope",	612, 792}
    };

#ifndef	PAPERTYPE
#define	PAPERTYPE	Letter
#endif
#ifndef LARGEAREA
#define LARGEAREA	TRUE
#endif

int paper = PAPERTYPE;
int large = LARGEAREA;
int manualfeed = FALSE;
int landscape = FALSE;
/*
 * >= 0		dvi
 * -1		NONFONT
 * -2,-4,...	pack, jstfm
 * -3,-5,...	jcomp
 */
char *
psfname(psf)
int psf;
{
    if (psf >= 0)
	sprintf(psfnamebuf, "f%d", psf);
    else if (psf%2 == 0)
	sprintf(psfnamebuf, "F%d", -psf/2-1);
    else
	sprintf(psfnamebuf, "JF%d", -psf/2-1);
    return (psfnamebuf);
}

dev_nextdevfont()
{
    return (devicefont -= 2);
}

dev_nextjcompfont()
{
    return (jcompfont -= 2);
}

dev_arg(option, c)
char option;
char *c;
{
    switch (option) {
    case 'R':
	resolution = atoi(c);
	break;
    case 'o':
	if (!ChkOpt(c))
	    Fatal("%s is an invalid option\n", c);
	break;
    }
}

/*-->ChkOpt*/   /* check a user supplied option for validity */
#define ISOPT(s) (strcmp(str,s)==0)

ChkOpt(str)
char *str;
{
    if (ISOPT("landscape")) {
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("manualfeed")) {
	manualfeed = TRUE;
	return (TRUE);
    } else if (ISOPT("large")) {
	large = TRUE;
	return (TRUE);
    } else if (ISOPT("small")) {
	large = FALSE;
	return (TRUE);
    } else if (ISOPT("letter")) {
	paper = Letter;
	return (TRUE);
    } else if (ISOPT("note")) {
	paper = Letter;
	large = FALSE;
	return (TRUE);
    } else if (ISOPT("legal")) {
	paper = Legal;
	return (TRUE);
    } else if (ISOPT("tabloid")) {
	paper = Tabloid;
	return (TRUE);
    } else if (ISOPT("a3")) {
	paper = A3;
	return (TRUE);
    } else if (ISOPT("a4")) {
	paper = A4;
	return (TRUE);
    } else if (ISOPT("a5")) {
	paper = A5;
	return (TRUE);
    } else if (ISOPT("b4")) {
	paper = B4;
	return (TRUE);
    } else if (ISOPT("b5")) {
	paper = B5;
	return (TRUE);
    } else if (ISOPT("letterlandscape")) {
	paper = Letter;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("notelandscape")) {
	paper = Letter;
	large = FALSE;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("legallandscape")) {
	paper = Legal;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("tabloidlandscape")) {
	paper = Tabloid;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("a3landscape")) {
	paper = A3;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("a4landscape")) {
	paper = A4;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("a5landscape")) {
	paper = A5;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("b4landscape")) {
	paper = B4;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("b5landscape")) {
	paper = B4;
	landscape = TRUE;
	return (TRUE);
    } else if (ISOPT("envelope")) {
	paper = Envelope;
	manualfeed = TRUE;
	landscape = TRUE;
	return (TRUE);
    }

    return (FALSE);
}

dev_init()
{
    int i;

    EMIT(outfp, "%%!PS-Adobe-2.0\n");
    EMIT(outfp, "%%%%Creator: %s\n", G_progname);
    EMIT(outfp, "%%%%Title: %s\n", filename);
    EMIT(outfp, "%%%%Pages: (atend)\n");
    EMIT(outfp, "%%%%BoundingBox: 0 0 %d %d\n", paperinfo[paper].width, paperinfo[paper].height);
    EMIT(outfp, "%%%%DocumentPaperSizes: %s\n", paperinfo[paper].size);
    EMIT(outfp, "%%%%Orientation: Portrait\n");
    EMIT(outfp, "%%%%EndComments\n");
    if (G_header)
	ps_copyfile(HDRFILE);
    for (i = 0; i < nif; i++)	/* copy all included files */
	ps_copyfile(Ifile[i]);
    EMIT(outfp, "%%%%EndProlog\n");

    hconvresolution = resolution;
    vconvresolution = resolution;
    EMIT(outfp, "%%%%BeginSetup\n");
    EMIT(outfp, "%%%%PaperSize: %s\n", paperinfo[paper].size);
    EMIT(outfp, "%%%%Feature: *ManualFeed %s\n", (manualfeed?"True":"False"));
    EMIT(outfp, "%%%%Feature: *Resolution %d\n", resolution);
    EMIT(outfp, "TeXDict begin\n");
    if (manualfeed)
	EMIT(outfp, "@manualfeed\n");
    if (landscape)
	EMIT(outfp, "@landscape\n");
    EMIT(outfp, "@%s\n", (large?"large":"small"));
    EMIT(outfp, "@%s\n", paperinfo[paper].command);
    if (ncopies > 1)
	EMIT(outfp, "%d COPIES\n", ncopies);
    /* include file given in fontdesc */
    do_include(ps_copyfile);
    EMIT(outfp, "%d START\n", resolution);
}

dev_finish()
{
    end_string();
    EMIT(outfp,"%%%%Trailer\n");
    EMIT(outfp,"%%%%Pages: %d\n", ordinal);
    EMIT(outfp,"END\n");
    EMIT(outfp,"%%%%EOF\n");
}

void
ps_copyfile(str)
char *str;
{
    EMIT(outfp, "%%%%BeginFile: %s\n", str);
    copyfile(str);
    EMIT(outfp, "%%%%EndFile\n");
}

void
ps_copyfigfile(str)
char *str;
{
    EMIT(outfp, "%%%%BeginDocument: %s\n", str);
    copyfile(str);
    EMIT(outfp, "%%%%EndDocument\n");
}

dev_conv(num, den, mag)
int num, den, mag;
{
    hconv = DoConv(num, den, mag, hconvresolution);
    vconv = DoConv(num, den, mag, vconvresolution);
#ifdef DEBUG
    if (Debug) {
	fprintf(stderr, "hconv = %d\n", hconv);
	fprintf(stderr, "vconv = %d\n", vconv);
    }
#endif
}

float
dev_fontmag(fe)
struct font_entry *fe;
{
    return (((float)fe->s/(float)fe->d)
#ifdef USEGLOBALMAG
	    * ((float)mag/1000.0)
#endif
	    );
}

dev_initpage()
{
    ps_h = ps_v = 0;
    cur_psf = NONFONT;
}

dev_endsetup()
{
    EMIT(outfp, "%%%%EndSetup\n");
}

dev_bop(count)
int	count;
{
    end_string();
    EMIT(outfp, "\n%%%%Page: %d %d\n", count, ++ordinal);
    EMIT(outfp, "BP\n");
}

dev_eop()
{
    end_string();
    EMIT(outfp, "EP\n");
}

dev_setfont(psf)
register int psf;
{
    if (cur_psf != psf) {
	end_string();
	EMIT(outfp, "%s SF\n", psfname(psf));
	cur_psf = psf;
    }
}

dev_setposn(x, y)	/* output a positioning command */
int x, y;
{
    int rx, ry;
#ifdef USERELPOS
    if (y == ps_v) {	/* use relative movement if just moving horizontally */
	if (x != ps_h) {
	    if ((rx=pixround(x-ps_h,hconv)) != 0) {
		end_string();
		EMITN(rx);
		EMITS("r ");
		ps_h += rx*hconv;
	    }
	}
    } else {
#endif
	end_string();
	EMITN(rx=pixround(x,hconv));
	EMITN(ry=pixround(y,vconv));
	EMITS("p ");
	ps_h = rx*hconv;	/* must know where device "really" is horizontally, for rel. posning. */
	ps_v = y;		/* but we always use direct positioning for vertical movement */
#ifdef USERELPOS
    }
#endif
}

dev_setrule(a, b)
int a, b;
{
    end_string();
    EMITN(pixround(b,hconv));	/* width */
    EMITN(pixround(a,vconv));	/* height */
    EMITS("ru\n");
}

int instring = FALSE;

begin_string()
{
    if (!instring) {
	instring = TRUE;
	EMITC('(');
    }
}

end_string()
{
    if (instring) {
	instring = FALSE;
	EMITS(")s\n");
    }
}

/*-->DoConv*/
/*********************************************************************/
/********************************  DoConv  ***************************/
/*********************************************************************/
/*
	1 sp = (num/den)*10^(-5) cm
	1 inch = 254*10^(-2) cm
	Therefore, 1 sp = conv pixel-unit
		   return-value sp = 1 pixel-unit
*/

int DoConv(num, den, mag, convResolution)
int num, den, mag, convResolution;
{
    register float conv;
    conv = ((float)num/(float)den) *
#ifdef USEGLOBALMAG
/*	ActualFactor(mag) * why was this in as Actual Factor?  jls */
	((float) mag/1000.0) *
#endif
	((float)convResolution/254000.0);
    return((int) (1.0 / conv + 0.5));
}

int
pixround(x, conv)	/* return rounded number of pixels */
register int x;		/* in DVI units     */
int conv;		/* conversion factor */
{
    return((int)((x + (conv >> 1)) / conv));
}
