/*
 * DEVICE dependent, built-in font dependent
 */

#include	<stdio.h>
#include	"defs.h"
#include	"emit.h"
#include	"global.h"
#include	"bifont.h"
#include	"ps.h"


/* bifont & bikanji
 */
void getbifont();
struct confop bfcop = {
    "bifont",
    getbifont
};
void getbikanji();
struct confop bkcop = {
    "bikanji",
    getbikanji
};

struct bifont {
    char *bf_name;
    int bf_len;
    int bf_fam;
    int bf_fixed;
    char *bf_psfont;
    char *bf_psname;
    int bf_created;
    struct bifont *bf_next;
};
struct bifont *bifonts = NULL;
struct bifont **nextbf = &bifonts;
struct bifont *bikanjis = NULL;
struct bifont **nextbk = &bikanjis;
int bffam = 1;
void getbuiltinfont();

void
getbifont(c, f)
int c;
FILE *f;
{
    getbuiltinfont(c, f, "bifont", &nextbf);
}

void
getbikanji(c, f)
int c;
FILE *f;
{
    getbuiltinfont(c, f, "bikanji", &nextbk);
}

void
getbuiltinfont(c, f, cop, next)
int c;
FILE *f;
char *cop;
struct bifont ***next;
{
    char field_name[STRSIZE];
    char field_fixed[STRSIZE];
    char field_psfont[STRSIZE];
    register struct bifont *bf;

    c = getfield(field_name, c, f);
    c = getfield(field_fixed, c, f);
    c = getqfield(field_psfont, c, f);
    skipline(c, f);
    bf = NEW(struct bifont, cop);
    bf->bf_name = strsave(field_name);
    bf->bf_len = strlen(field_name);
    bf->bf_fam = bffam++;
    bf->bf_fixed = strcmp(field_fixed, "fixed") == 0;
    bf->bf_psfont = strsave(field_psfont);
    bf->bf_created = FALSE;
    bf->bf_next = NULL;
    **next = bf;
    *next = &(bf->bf_next);
}

struct bifont *
findbifont(name)
char *name;
{
    register struct bifont *bf;

    for (bf = bifonts; bf != NULL; bf = bf->bf_next)
	if (strcmp(name, bf->bf_name) == 0)
	    return (bf);
    return (NULL);
}

struct bifont *
findbuiltinkanji(name)
register char *name;
{
    register struct bifont *bf;

    for (bf = bikanjis; bf != NULL; bf = bf->bf_next)
	if (strncmp(name, bf->bf_name, bf->bf_len) == 0)
	    return (bf);
    return (NULL);
}

struct bifont *
findbikanji(name)
char *name;
{
    struct bifont *bk;

    if ((bk = findbuiltinkanji(name)) != NULL)
	return (bk);
    Fatal("%s is not built-in kanji", name);
}

dev_isbikanji(name)
char *name;
{
    return (findbuiltinkanji(name) != NULL);
}

char *
dev_bikanjiname(name)
char *name;
{
    return (findbikanji(name)->bf_psfont);
}

char *
newbifont(psn, bf)
char *psn;
struct bifont *bf;
{
    if (!(bf->bf_created)) {
	bf->bf_created = TRUE;
	if (psn[0] == FDQUO) {
	    /* It might be better to generate new name to avoid possible
	       collisions. */
	    bf->bf_psname = bf->bf_name;
	    EMIT(outfp, "/%s %s\n", bf->bf_psname, psn+1);
	} else {
	    bf->bf_psname = bf->bf_psfont;
	}
    }
    return (bf->bf_psname);
}


/* TYPE: tfm
 */
dev_tfm_initfontdict(fe)
struct font_entry *fe;
{
    struct bifont *bf;
    int size;
    char *psop, *dev_name;

    /* open font dict before first char */
    /*if (fe->ncdl == -1)*/ {
	if ((bf = findbifont(fe->n)) != NULL) {
	    if (bf->bf_fixed) {
		size = (tfmfinfo(fe)->ch+tfmfinfo(fe)->lastfntchar)->tfmw;
		psop = "LWFX";
	    } else {
		size = fe->s;
		psop = "LWF";
	    }
	    dev_name = newbifont(bf->bf_psfont, bf);
	} else {
	    size = fe->s;
	    psop = "LWF";
	    dev_name = fe->n;
	}
	EMIT(outfp, "/%s /%s %.3f %.3f %s\n",
	     psfname(fe->k), dev_name,
	     (float)size/(float)hconv, -(float)size/(float)vconv, psop);
	fe->ncdl = 0;
    }
}

/* TYPE: jstfm
 */
struct jstfmfdict {
    int jd_jfam;
    int jd_s;
    int jd_jfont;
    int jd_open;
    struct jstfmfdict *jd_next;
};
struct jstfmfdict *jstfmfdicts = NULL;
struct jstfmfdict **nextjd = &jstfmfdicts;

dev_getjfont(fe)
register struct font_entry *fe;
{
    register int s, jfam;
    register struct jstfmfdict *jd;
    struct bifont *bk;

    s = fe->s;
    jfam = (bk = findbikanji(fe->n))->bf_fam;
    for (jd = jstfmfdicts; jd != NULL; jd = jd->jd_next)
	if (jd->jd_s == s && jd->jd_jfam == jfam) {
	    jstfmfinfo(fe)->dev_font = jd->jd_jfont;
	    jstfmfinfo(fe)->dictopen = &(jd->jd_open);
	    jstfmfinfo(fe)->dev_name = bk->bf_psfont;
	    return (getjsubfont(fe->n+bk->bf_len));
	}
    jd = NEW(struct jstfmfdict, "jstfmfdict");
    jd->jd_s = s;
    jd->jd_jfam = jfam;
    jd->jd_jfont = jstfmfinfo(fe)->dev_font = dev_nextdevfont();
    jd->jd_open = FALSE;
    jd->jd_next = NULL;
    *nextjd = jd;
    nextjd = &(jd->jd_next);
    jstfmfinfo(fe)->dictopen = &(jd->jd_open);
    jstfmfinfo(fe)->dev_name = bk->bf_psfont;
    return (getjsubfont(fe->n+bk->bf_len));
}

dev_jstfm_initfontdict(fe)
struct font_entry *fe;
{
    register struct jstfmfntinfo *jstfmfi;
    struct bifont *bf;
    char *dev_name;
    int size;

    /* assumption:
       width and height of fixed-pitch japanese buit-in char are equal */
    if (!*((jstfmfi = jstfmfinfo(fe))->dictopen)) {
	bf = findbikanji(fe->n);
	dev_name = newbifont(jstfmfi->dev_name, bf);
	size = bf->bf_fixed
		? (jstfmfi->ch+jstfmfi->lastfntchar)->tfmw
		: fe->s;
	EMIT(outfp, "/%s /%s %.3f %.3f LWJF\n",
	     psfname(jstfmfi->dev_font), dev_name,
	     (float)size/(float)hconv, -(float)size/(float)vconv);
	*(jstfmfi->dictopen) = TRUE;
    }
}

/* TYPE: jfm
 */
dev_jfm_initfontdict(fe)
struct font_entry *fe;
{
    register struct jfmfntinfo *jfmfi;
    struct bifont *bf;
    char *dev_name;
    int size;

    /* open font dict before first char */
    /*if (fe->ncdl == -1)*/ {
	jfmfi = jfmfinfo(fe);
	bf = findbikanji(fe->n);
	dev_name = newbifont(jfmfi->dev_name, bf);
	size = bf->bf_fixed ? jfmfi->ch->tfmw : fe->s;
	EMIT(outfp, "/%s /%s %.3f %.3f LWF\n",
	     psfname(fe->k), dev_name,
	     (float)size/(float)hconv, -(float)size/(float)vconv);
	fe->ncdl = 0;
    }
}

/* TYPE: bitex
 */
dev_bitex_initfontdict(fe)
struct font_entry *fe;
{
    /* open font dict before first char */
    /*if (fe->ncdl == -1)*/ {
	EMIT(outfp, "/%s /%s %.3f %.3f LWF\n",
	     psfname(fe->k), bitexfinfo(fe)->dev_name,
	     (float)fe->s/(float)hconv, -(float)fe->s/(float)vconv);
	fe->ncdl = 0;
    }
}


/*
 * LW dependent setchar/setstring
 */
dev_tfm_initfe(fe)
register struct font_entry *fe;
{
    int tfm_fontdict(), tfm_setchar(), tfm_setstring();

    fe->dev_fontdict = tfm_fontdict;
    fe->dev_setchar = tfm_setchar;
    fe->dev_setstring = tfm_setstring;
}

tfm_setchar(c)
int c;
{
    register struct tfmchar_entry *ce = &(tfmfinfo(fontptr)->ch[c]);
    register int cw;

    begin_string();
    pschar(ce->dev_char);
    ps_h += (cw = ce->tfmw);
    return (cw);
}

tfm_setstring(s, len)
char *s;
int len;
{
    register char *sp;
    register struct tfmchar_entry *ce = tfmfinfo(fontptr)->ch;
    register int cw = 0;

    begin_string();
    for (sp=s; sp < s+len; sp++) {
	pschar((ce+*sp)->dev_char);
	cw += (ce+*sp)->tfmw;
    }
    ps_h += cw;
    return (cw);
}

dev_jstfm_initfe(fe)
register struct font_entry *fe;
{
    int jstfm_fontdict(), jstfm_setchar(), jstfm_setstring();

    fe->dev_fontdict = jstfm_fontdict;
    fe->dev_setchar = jstfm_setchar;
    fe->dev_setstring = jstfm_setstring;
}

jstfm_setchar(c)
int c;
{
    register struct jstfmchar_entry *ce = &(jstfmfinfo(fontptr)->ch[c]);
    register int cw;

    begin_string();
    pschar(ce->dev_ku);
    pschar(ce->dev_ten);
    ps_h += (cw = ce->tfmw);
    return (cw);
}

jstfm_setstring(s, len)
char *s;
int len;
{
    register char *sp;
    register struct jstfmchar_entry *ce = jstfmfinfo(fontptr)->ch;
    register int cw = 0;

    begin_string();
    for (sp=s; sp < s+len; sp++) {
	pschar((ce+*sp)->dev_ku);
	pschar((ce+*sp)->dev_ten);
	cw += (ce+*sp)->tfmw;
    }
    ps_h += cw;
    return (cw);
}

dev_jfm_initfe(fe)
register struct font_entry *fe;
{
    int tfm_fontdict(), jfm_setchar(), jfm_setstring();

    fe->dev_fontdict = tfm_fontdict;
    fe->dev_setchar = jfm_setchar;
    fe->dev_setstring = jfm_setstring;
}

jfm_setchar(c)
register int c;
{
    register int cw;
    register struct jfmfntinfo *jfmfi = jfmfinfo(fontptr);

    begin_string();
    pschar(c>>8);
    pschar(c&0xff);
    ps_h += (cw = jfmfi->ch[getctype(c,jfmfi)].tfmw);
    /* quick hack to disable USERELPOS */
    ps_v = -1;
    return (cw);
}

jfm_setstring(s, len)
char *s;
int len;
{
    Fatal("dvi2ps implementation error: jfm_setstring");
}
