/*                                       */ 
/*  File : ctw.c                         */ 
/*                                       */ 
/*  ctw : Change(Correct) TFM Width      */ 
/*                                       */ 
/*  This file was drived from dvi2ps     */ 
/*  and xdvi.                            */ 
/*                                       */ 
/*  Usage: ctw [-d] [-D] [-nb] TFM_File  */ 
/*                                       */ 
/*  Option: -d    Debug Mode             */ 
/*          -D    Debug Mode (in Detail) */ 
/*          -nb   No Backup Mode         */ 
/*                                       */ 

#include <stdio.h>
#include "pk.h"

ULONG TfmDesign;
ULONG PkDesign;
ULONG WidthTbl[MAXCHARS];
int TfmWidth[MAXCHARS];

FILE *FpRead, *FpWrite;
int Debug = 0;
int Backup = 1;
char *Usage = "ctw [-d] [-D] [-nb] TFM_File";
char FileName[MAXCHARS];
char FontName[MAXCHARS];
char *tmplate = "tmpXXXXXX";
char *mktemp(), *index(), *strcpy();
ULONG num(), numio();


main(argc, argv)
int argc;
char *argv[];
{
  while (argc > 1 && argv[1][0] == '-') {
    switch (argv[1][1]) {
      case 'd': 
        Debug = 1;
	break;
      case 'D':
        Debug = 2;
        break;
      case 'n':
        if (strcmp(&argv[1][2], "b") == 0)
          Backup = 0;
        else
          Error(Usage);
        break;
      default:
	Error(Usage);
    }
    argv++;
    argc--;
  }

  if (argc <= 1)
    Error("TFM file is not specified.");

  if (strncmp(argv[1], "dm", 2) && strncmp(argv[1], "dg", 2))
    Error("File %s does not start with dm or dg.", argv[1]);

  strcpy(FileName, argv[1]);
  ReadTfmFile();
  ChangeTfmWidth();
}  


ReadTfmFile()
{
  int i;
  int hsize, first, last, wsize;
  char *p;

  p = index(FileName, '.');
  if ((p == NULL) || (strcmp(p, ".tfm") != 0))
    Error("File %s is not FTM file.", FileName);

  sprintf(FontName, "%s/%s", TFM_DIRECTORY, FileName);
  if ((FpRead = fopen(FontName, "r")) == NULL) 
    Error("Can't open FTM file %s.", FileName);
  else if (Debug) {
    printf("\nTFM File %s, processing start.\n", FileName);
    fflush(stdout);
  }

  fseek(FpRead, 2L, 0);
  hsize = (int)two(FpRead);
  first = (int)two(FpRead);
  last = (int)two(FpRead);
  wsize = (int)two(FpRead);
  fseek(FpRead, 28L, 0);
  TfmDesign = four(FpRead);
  fseek(FpRead, (long)(hsize-2)*4L, 1);

  if (Debug) {
    printf("Design = %ld, Hsize = %d, First = %d, Last = %d, Wsize = %d\n",
	   TfmDesign, hsize, first, last, wsize);
    fflush(stdout);
  }

  for (i = first; i <= last; i++) {
    TfmWidth[i] = one(FpRead);
    fseek(FpRead, 3L, 1);
  }
  for (i = 0; i < wsize; i++) 
    WidthTbl[i] = four(FpRead);

  fclose(FpRead);
  if (Debug) {
    printf("TFM File %s, processing end.\n\n", FileName);
    fflush(stdout);
  }
}


ChangeTfmWidth()
{
  int i;
  int fontsize;
  float realsize;
  char *p;

  for (i = 0; i <= 8; i++) {
    switch (i) {
      case 0:  realsize = 1.095445;  break;  /* MagStephalf */
      case 1:  realsize = 1.2;       break;  /* MagStep1 */
      case 2:  realsize = 1.44;      break;  /* MagStep2 */
      case 3:  realsize = 1.728;     break;  /* MagStep3 */
      case 4:  realsize = 2.0736;    break;  /* MagStep4 */
      case 5:  realsize = 2.48832;   break;  /* MagStep5 */
      case 6:  realsize = 2.985984;  break;  /* MagStep6 */
      case 7:  realsize = 3.583181;  break;  /* MagStep7 */
#if (PIXELS_PER_INCH == 240)
      case 8:  realsize = 1.0;       break;  /* MagStep0 */
#else
      case 8:  return;
#endif
    }
    fontsize = (realsize * PIXELS_PER_INCH) + 0.5;
    p = index(FileName, '.');  
    *p = '\0';
    sprintf(p, ".%dpk", fontsize);
    sprintf(FontName, "%s/%s", PK_DIRECTORY, FileName);
    ReadPkFile();
  }
}


ReadPkFile()
{
  int i, cmd;
  UCHAR ch;
  char tmpfile[MAXCHARS];
  char backup[MAXCHARS];

  if ((FpRead = fopen(FontName, "r")) == NULL) {
    printf("PK File %s does not exist.\n", FileName);
    fflush(stdout);
    return;
  } else if (Debug) {
    printf("PK File %s, processing start...", FileName);
    fflush(stdout);
  }

  sprintf(tmpfile, "%s/%s", PK_DIRECTORY, mktemp(tmplate));
  if ((FpWrite = fopen(tmpfile, "w")) == NULL)
    Error("Can't open temp file.");

  i = 0;
  do {
    if (fseek(FpRead, (long)(--i), 2) == -1)
      Error("Can't seek to the end of pk file %s.", FileName);
  } while ((ch = one(FpRead)) == PK_NOOP);

  if (ch != PK_POST)
    Error("Bad pk file %s.", FileName);

  fseek(FpRead, 0L, 0);

  if (oneio(FpRead, FpWrite) != PK_PRE || oneio(FpRead, FpWrite) != PK_ID)
    Error("Bad pk file %s.", FileName);
  
  /* Comment */
  for (ch = oneio(FpRead, FpWrite), i = 0; i < ch; i++)
    putc(getc(FpRead), FpWrite);
  /* Design Size */
  PkDesign = fourio(FpRead, FpWrite);
  /* Checksum, Hppp, Vppp */
  for (i = 0; i < 12; i++)
    putc(getc(FpRead), FpWrite);

  for ( ; ; ) {
    switch(cmd = oneio(FpRead, FpWrite)) {
      case PK_XXX1: case PK_XXX2: case PK_XXX3: case PK_XXX4:
        for (ch = numio(FpRead, FpWrite, cmd-PK_XXX1+1), i = 0; i < ch; i++)
	  putc(getc(FpRead), FpWrite);
        continue;
      case PK_YYY:
        (void)fourio(FpRead, FpWrite);
        continue;
      case PK_NOOP:
        continue;
      case PK_POST:
        break;
      default:
        if (cmd <= PK_FLAG)
	  PkChar(cmd);
        else
	  Error("Illegal PK command %d in %s.", cmd, FileName);
        continue;
    }
    break;
  }

  while ((i = getc(FpRead)) != EOF) {
    putc((char)i, FpWrite);
  }

  fclose(FpRead);
  fclose(FpWrite);
  if (Backup) {
    sprintf(backup, "%s.%s", FontName, "orig");
    link(FontName, backup);
  }
  unlink(FontName);
  link(tmpfile, FontName);
  unlink(tmpfile);
  if (Debug) {
    printf("end.\n", FileName);
    fflush(stdout);
  }
}


PkChar(flag)
int flag;
{
  int pl;     /* Packet Length */
  int cc;     /* Character Code */
  int n, i;
  ULONG tfm;  /* Tfm Width */

  if ((n = flag & 7) == 7) {	/* long form */
    pl = fourio(FpRead, FpWrite) - 8;
    if ((cc = fourio(FpRead, FpWrite)) > MAXCHARS) {
      Error("Character Code %d in pk file %s.", cc, FileName);
    }
    tfm = four(FpRead);
    TfmPut(FpWrite, tfm, cc, 4);
    (void)fourio(FpRead, FpWrite); /* dx */
  } else if (n >= 4) {		/* extended short form */
    pl = (n-4)*65536 + twoio(FpRead, FpWrite) - 3;
    cc = oneio(FpRead, FpWrite);
    tfm = num(FpRead, 3);
    TfmPut(FpWrite, tfm, cc, 3);
  } else {			/* short form */
    pl = n*256 + oneio(FpRead, FpWrite) - 3;
    cc = oneio(FpRead, FpWrite);
    tfm = num(FpRead, 3);
    TfmPut(FpWrite, tfm, cc, 3);
  }

   /* dy or dm, width, height, hoff, voff, bitmap data */
  for (i = pl; i > 0; i--) 
    putc(getc(FpRead), FpWrite);
}


TfmPut(fp, pktfm, cc, size)
FILE *fp;
ULONG pktfm;
int cc;
int size;
{
  ULONG tfm;
  int i;

  tfm = TfmDesign / PkDesign * WidthTbl[TfmWidth[cc]];
  if (Debug > 1) {
    printf("Char %3d : PK TFM %8ld --> %8ld.\n" , cc, pktfm, tfm);
    fflush(stdout);
  }
  for (i = size-1; i >= 0; i--)
    putc((char)(tfm >> 8*i), fp);
}


ULONG num(fp, size)
FILE *fp;
int size;
{
  register int i;
  register long x;

  x = 0;
  for (i = 0; i < size; i += 1)
    x = x*0x100 + (UCHAR)getc(fp);
  if (feof(fp))
    x = 0;
  return (x);
}


ULONG numio(fpr, fpw, size)
FILE *fpr, *fpw;
int size;
{
  register int i;
  register long x;
  UCHAR ch;

  x = 0;
  for (i = 0; i < size; i += 1) {
    putc(ch = (UCHAR)getc(fpr), fpw);
    x = x*0x100 + ch;
  }
  if (feof(fpr))
    x = 0;
  return (x);
}


Error(mesg, var1, var2)
char *mesg;
int var1, var2;
{
  fprintf(stderr, "Error : ");
  fprintf(stderr, mesg, var1, var2);
  fprintf(stderr, "\n");
  fflush(stderr);
  exit(1);
}
