; GF file format
; a stream of 8-bit bytes
; preamble
; pre 247 i[1] k[1] x[k] : 247 131 6 "kanji0"
; 
; characters
; boc 67 c[4] p[4] min_m[4] max_m[4] min_n[4] max_n[4]
; boc1 68 c[1] del_m[1] max_m[1] del_n[1] max_n[1]
; paint_0 0, change paint_switch from black to white or vice versa
; paint_1 .. paint_63 (1 .. 63), blacken d pixels and advance m by d
; paint1 64 d[1], when 64 <= d < 256
; paint2 65 d[2], when d < 65535
; paint3 66 d[3], d can be as high as 2^24-1
; skip0 70, decrease n by 1 and m <- min_m, paint_switch <- white
; skip1 71 d[1], decrease n by d+1 and same as above.  d all-white rows.
; skip2 72 d[2], same as skip1 but d < 65535
; skip3 73 d[3], same as skip1 but d can be as large as 2^24-1
; new_row_0 74, decrease n by 1 and m <- min_m, paint_switch <- black
; new_row_1 .. new_row_164 (75 .. 238), same as new_row_0 but with
;                            m <- min_m + 1 .. m <- min_m + 164
; eoc 69
;
; postamble
; post 248 p[4] ds[4] cs[4] hppp[4] vppp[4] min_m[4] max_m[4] min_n[4] max_n[4]
; ds = design size : 10 (same as TFM file)
; cs = check sum :  (same as TFM file)
; hppp and vppp = ratios of pixels per point in scaled integers (* 2^16)
; hppp = vppp = 300dpi/72.27pt(per inch) = 4.1511 dot/pt
; character locators
; char_loc 245 c[1] dx[4] dy[4] w[4] p[4]
; c = character residue
; dx and dy = character escapements in scaled pixels. dx = 38 * 2^16, dy = 0
; w = character width, a fix_word value relative to the design size (as in TFM)
; w = 0.91542 * 2^20
; p = a pointer to the beginning of that character
; char_loc0 246 c[1] dm[1] w[4] p[4], same as char_loc with dy=0, dx=65536*dm
;                                              where 0 <= dm < 256
; dm = 38 ( since 65536=2^16)
; post_post 249 q[4] i[1] 223's[>=4]
; q = pointer to the post command, i = identification byte, 131
;
; other command
; xxx1 239 k[1] x[k] : undefined in general, used for special
; xxx2 240 k[2] x[k] : like xxx1, but 0 <= k < 65536
; xxx3 241 k[3] x[k] : like xxx1, but 0 <= k < 2^24
; xxx4 242 k[4] x[k] : like xxx1, but k can be rediculously large
; yyy 243 y[4] : undefined in general, used for numspecial
; no_op 244 : No operation, do nothing

(load '|lsp:trans8-7|) ; for 8 bits I/O

(defun out-fix-word(n ofobj) ; we assume n to be already integer
  (tyo8 (boole 1 #o377 (lsh n -24.)) ofobj)
  (tyo8 (boole 1 #o377 (lsh n -16.)) ofobj)
  (tyo8 (boole 1 #o377 (lsh n -8.)) ofobj)
  (tyo8 (boole 1 #o377 n) ofobj))

;; routines to make gf file (and tfm file)
(array charloc fixnum 256.)
(array width-table fixnum 256.)

; file directory and extention
(declare
 (special default-jgf-dir default-jtfm-dir jgf-extention jtfm-extention))
(setq default-jgf-dir `(bs ,'|tex.jgf|))
(setq default-jtfm-dir `(bs ,'|tex.jtfm|))
(setq jgf-extention '300gf)
(setq jtfm-extention 'tfm)

(declare(special bitsize tyo8-pointer tyo8-word tyi8-pointer tyi8-word))
(declare(special null-pattern-p))

(defun make-gf-file
  (fname w-in-dot ht-in-dot d-in-dot ds-in-pt index-list sub-font-num
   make-tfm-p mag)
; mag is used to setup hppp correctly
; refrain from making TFM file when generating magnified fonts
; call it by (make-gf-file 'JSY36 38. 30. 6. 10. jsy 1 T 0)
; index-list is in general of the form:
;      ((start1 num1 char-code-start1)
;       (start2 num2 char-code-start2)..(startn numn char-code-startn))
; n can be 1
 (let((ofobj(open (list default-jgf-dir fname jgf-extention) '(out fixnum)))
;      (char-code 0)
      (bc (caddr(car index-list)))
      (ec 0)
      (min-m 0)(min-n (- d-in-dot))(max-m bitsize)(max-n bitsize)
      (after-last-eoc 0)
      (hppp (ifix (*$ (//$ 300.0 72.27)(float (^ 2 16.))(expt 1.2 mag))))
      (dm w-in-dot)
      (design-size (* ds-in-pt (^ 2 20.)))
      (checksum 0) ; arbitrary but should be equal to checksum in TFM
      (width (compute-fix-word w-in-dot ds-in-pt mag)))
   (setq tyo8-pointer 0 tyo8-word 0) ; initialize tyo8 variables
   (do((i 0 (1+ i)))((= i 256.))(store(charloc i)0)) ; clear charloc array
; output preamble
   (tyo8 247. ofobj)
   (tyo8 131. ofobj)
   (tyo8 (flatsize fname) ofobj)
   (out-string fname ofobj)
; output characters
   (do((i-list index-list (cdr i-list)))((null i-list))
     (let((start-index (car(car i-list)))
	  (char-code(caddr(car i-list)))
	  (last-index (+ (car(car i-list))(cadr(car i-list)))))
       (do((i start-index (1+ i)))
	  ((= i last-index)(setq ec (1- char-code)))
	 (clear-pattern bitsize)
	 (setq null-pattern-p t) ; may be set to NIL in set-array-element
	 (read-pattern i bitsize)
	 (cond(null-pattern-p) ; do nothing
	      (t(store(charloc char-code)
		      (+ tyo8-pointer (* 4 (filepos ofobj))))
		(make-min-m-table)
		(setq min-m (min-bound))
		(setq max-n (max-non-blank (- d-in-dot)))
		(cond((or(> 0 max-n)(> 0 max-m))
		      (tyo8 67. ofobj) ; boc
		      (out-fix-word char-code ofobj)
		      (out-fix-word -1 ofobj)
		      (out-fix-word min-m ofobj)
		      (out-fix-word max-m ofobj)
		      (out-fix-word min-n ofobj)
		      (out-fix-word max-n ofobj))
		     (t(tyo8 68. ofobj) ; boc1
		       (tyo8 char-code ofobj)
		       (tyo8 (- max-m min-m) ofobj)
		       (tyo8 max-m ofobj)
		       (tyo8 (- max-n min-n) ofobj)
		       (tyo8 max-n ofobj)))
		(out-one-char (- d-in-dot) min-m ofobj)
		(tyo8 69. ofobj))) ;eoc
	 (setq char-code (1+ char-code)))))
; output postamble
   (setq after-last-eoc (+ tyo8-pointer (* 4 (filepos ofobj))))
   (tyo8 248. ofobj) ; post
   (out-fix-word after-last-eoc ofobj)
   (out-fix-word design-size ofobj)
   (out-fix-word checksum ofobj)
   (out-fix-word hppp ofobj)
   (out-fix-word hppp ofobj) ; vppp = hppp
   (out-fix-word  0 ofobj) ; min-m
   (out-fix-word bitsize ofobj) ; max-m
   (out-fix-word (- d-in-dot) ofobj) ; min-n
   (out-fix-word bitsize ofobj) ; max-n (need not to be the best possible)
; character locators
   (do((c bc (1+ c)))((> c ec))
     (cond((zerop(charloc c))) ; do nothing (this character does not exists)
	  (t(tyo8 246. ofobj)
	    (tyo8 c ofobj)
	    (tyo8 dm ofobj)
	    (out-fix-word (cond((zerop(width-table c))width)
			       (t(compute-fix-word(width-table c)ds-in-pt mag)))
			  ofobj)
	    (out-fix-word (charloc c) ofobj))))
   (tyo8 249. ofobj)
   (out-fix-word after-last-eoc ofobj)
   (tyo8 131. ofobj)
   (do((i tyo8-pointer (1+ i)))((> i 3))(tyo8 223. ofobj))
   (tyo8 223. ofobj)
   (tyo8 223. ofobj)
   (tyo8 223. ofobj)
   (tyo8 223. ofobj)
   (close ofobj)
   (cond(make-tfm-p
     (make-tfm-file (list default-jtfm-dir fname jtfm-extention)
	bc ec w-in-dot ht-in-dot d-in-dot ds-in-pt sub-font-num)))
   ))

(defun out-string(str ofobj)
  (let((chlist(exploden str)))
    (mapc '(lambda(x)(tyo8 x ofobj)) chlist)))

;(setq array-bitsize 96.)
(setq array-bitsize 256.) ; increased to handle larger dnp fonts

(array pattern fixnum array-bitsize array-bitsize)
(array work fixnum array-bitsize array-bitsize)
(declare(special infobj))

(defun triple-the-pattern(n)
;(show-array 'pattern)
  (clear-work (* 3 n))
  (do((j 0 (1+ j)))((= j n))
    (do((i 0 (1+ i)))((= i n))
      (cond((= 1 (pattern i j))
	    (do((jj(* 3 j)(1+ jj)))((= jj (+ 3 (* 3 j))))
	      (do((ii(* 3 i)(1+ ii)))((= ii (+ 3 (* 3 i))))
		(store(work ii jj)1)))))))
  (clear-pattern n)
;(show-array 'work)
  (copy-work-to-pattern (* 3 n))
;(show-array 'pattern)
)

(defun double-the-pattern(n)
 (clear-work (* 2 n))
 (do((j 0 (1+ j)))((= j n))
  (do((i 0 (1+ i)))((= i n))
    (cond((= 1 (pattern i j))
	  (store(work (* 2 i)(* 2 j))1)
	  (store(work (* 2 i)(1+ (* 2 j)))1)
	  (store(work (1+ (* 2 i))(* 2 j))1)
	  (store(work (1+ (* 2 i))(1+ (* 2 j)))1)))))
 (clear-pattern n)
 (copy-work-to-pattern (* 2 n)))

(defun copy-work-to-pattern(n)
 (do((j 0 (1+ j)))((= j n))
  (do((i 0 (1+ i)))((= i n))
    (cond((= 1 (work i j))(store(pattern i j)1))))))

(declare(special dnp-making-mode))
(setq dnp-making-mode nil)

;; change for jis 24 dot reading
;; redefine read-pattern
(defun read-pattern(n bsize)
  (cond(dnp-making-mode(dnp-read-into-array n bsize))
       ((= bsize 36.)(read-into-array-48 n))
       ((= bsize 38.)(read-into-array-48 n)) ; to read 38dots font from dnp
       ((= bsize 32.)(read-into-array n)) ;added to read 32dot fonts from Gasho
       ((= bsize 48.)(read-into-array-24 n)
		     (double-the-pattern 24.))
       ((= bsize 72.)(read-into-array-24 n)
		     (triple-the-pattern 24.))
       ((= bsize 24.)(read-into-array-24 n))))

(defun set-file-pointer-to-start-byte(infobj startp)
  (cond((zerop(\ startp 4))(filepos infobj (// startp 4)))
       (t(filepos infobj (// startp 4))
	 (do((i 0 (1+ i)))((= i (\ startp 4)))(tyi8 infobj)))))

(defun dnp-read-into-array(n dsize)
  (let*((number-of-bytes-in-row (// (+ dsize 7) 8.))
	(number-of-bytes-per-char (* dsize number-of-bytes-in-row))
	(startp (* (1- n) number-of-bytes-per-char)))
    (setq tyi8-pointer 0)
    (setq tyi8-word 0)
;    (filepos infobj (// startp 4))
; It is not guaranteed that each character starts from word boundary
    (set-file-pointer-to-start-byte infobj startp)
    (do((pt startp (+ pt number-of-bytes-in-row))
	(row 0 (1+ row)))
       ((= pt (+ startp number-of-bytes-per-char)))
      (do((i 0 (1+ i)))((= i number-of-bytes-in-row))
	(set-array-element row (* i 8.) (tyi8 infobj))))
;    (show-pattern 40.)
    ))

;(setq infobj (open '|js:<gothic>jg1038.rf| '(in fixnum)))
(defun dnp-show(n dsize)
  (let*((number-of-bytes-in-row (// (+ dsize 7) 8.))
	(number-of-bytes-per-char (* dsize number-of-bytes-in-row))
	(startp (* (1- n) number-of-bytes-per-char)))
    (setq tyi8-pointer 0)
    (setq tyi8-word 0)
;    (filepos infobj (// startp 4))
; It is not guaranteed that each character starts from word boundary
    (set-file-pointer-to-start-byte infobj startp)
    (terpri)
    (do((pt startp (+ pt number-of-bytes-in-row)))
       ((= pt (+ startp number-of-bytes-per-char))'end)
      (do((i 0 (1+ i)))((= i number-of-bytes-in-row))
	(type-bit-pattern(tyi8 infobj)))
      (terpri))))

(defun read-into-array(n)
   (let((startp(* (1- n) 128.)))
      (setq tyi8-pointer 0)
      (setq tyi8-word 0)
      (filepos infobj (// startp 4))
      (do((pt startp (+ pt 4))(row 0 (1+ row)))((= pt (+ startp 128.)))
	 (set-array-element row 0 (tyi8 infobj))
	 (set-array-element row 8. (tyi8 infobj))
	 (set-array-element row 16. (tyi8 infobj))
	 (set-array-element row 24. (tyi8 infobj)))))

(defun read-into-array-24(n)
   (let((startp(* (1- n) 72.)))
      (setq tyi8-pointer 0)
      (setq tyi8-word 0)
      (filepos infobj (// startp 4))
      (do((pt startp (+ pt 3))(row 0 (1+ row)))((= pt (+ startp 72.)))
	 (set-array-element row 0 (tyi8 infobj))
	 (set-array-element row 8. (tyi8 infobj))
	 (set-array-element row 16. (tyi8 infobj))
	 )))

; In the following function 288 (number of bytes to contain 48*48 dot data)
; is replaced by 216 (6*36) to read in dot data from dnp (dainippon printing).
; again it is changed to 6*38=228 (for 38 dots)
; The value 288 is for jisfont36.dat
; The value 216 is for dnp 36 dot fonts
; The value 228 is for dnp 38 dot fonts
(declare(special byte-count-for-48))

(defun read-into-array-48(n)
   (let((startp(* (1- n) byte-count-for-48)))
      (setq tyi8-pointer 0)
      (setq tyi8-word 0)
      (filepos infobj (// startp 4))
      (do((pt startp (+ pt 6))(row 0 (1+ row)))
	 ((= pt (+ startp byte-count-for-48)))
	 (set-array-element row 0 (tyi8 infobj))
	 (set-array-element row 8. (tyi8 infobj))
	 (set-array-element row 16. (tyi8 infobj))
	 (set-array-element row 24. (tyi8 infobj))
	 (set-array-element row 32. (tyi8 infobj))
	 (set-array-element row 40. (tyi8 infobj)))))

(defun set-array-element(row column data)
  (do((c column(1+ c))(i 0 (1+ i)))((= c (+ column 8.)))
    (cond((not(zerop data))(setq null-pattern-p nil)))
    (store (pattern c row)(ith-bit i data))))

(defun ith-bit(i w)(boole 1 1 (lsh w (- i 7))))

(defun show-pattern(n)
 (do((j 0 (1+ j)))((= j n))
   (do((i 0 (1+ i)))((= i n))
     (cond((zerop(pattern i j))(tyo 32.))(t(tyo 42.))))
   (terpri)))

(defun clear-pattern(n)
 (do((j 0 (1+ j)))((= j n))
  (do((i 0 (1+ i)))((= i n))
    (store(pattern i j) 0))))

(defun clear-work(n)
 (do((j 0 (1+ j)))((= j n))
  (do((i 0 (1+ i)))((= i n))
    (store(work i j) 0))))

(array min-m-table fixnum array-bitsize)

(defun out-one-char(depth min-m ofobj)
 (let((prev-j (max-non-blank depth)))
   (do((j prev-j (1- j)))((< j depth)) ; do for each row
     (cond((= 1 (- prev-j j))) ; do nothing here
	  ((< 1 (- prev-j j))
	   (do((k j (1- k)))((or(= k depth)
				(not(= (min-m-table (updown k depth))
				       bitsize)))
			     (setq j k)))
	   (cond((= j depth)(setq prev-j (1+ j)))
		(t(tyo8 71. ofobj)(tyo8 (- prev-j j 2) ofobj) ; skip1
		  (setq prev-j (1+ j))))))
     (let((jj (updown j depth))
	  (color 1)
	  (prev-i(min-m-table (updown j depth))))
       (cond((= (min-m-table jj) bitsize)) ; blank-line do nothing
	    (t(cond((= prev-j j)(cond((< (- prev-i min-m) 64.)
				      (tyo8 (- prev-i min-m) ofobj))
				 ; paint_0 through paint_63 (first blanks)
				     ((< (- prev-i min-m) 256.)
				      (tyo8 64. ofobj) ; paint_1
				      (tyo8 (- prev-i min-m) ofobj))
				     (t(print "You need paint_2 or _3"))))
		   (t(cond((< (- prev-i min-m) 165.) ; new_row_0 .. new_row_164
			   (tyo8 (+ (- prev-i min-m) 74.) ofobj))
			  (t(print "You cannot have new_row_more-than-164")))))
	      (do((i (min-m-table jj)(1+ i)))((> i bitsize))
		(cond((= (pattern i jj) color)) ; do nothing
		     (t(cond((< (- i prev-i) 64.)
			     (tyo8 (- i prev-i) ofobj)) ; paint_1 thru paint_63
			    ((< (- i prev-i) 256.)
			     (tyo8 64. ofobj) ; paint_1
			     (tyo8 (- i prev-i) ofobj))
			    (t(print "You need paint_2 or paint_3")))
		       (setq prev-i i)
		       (setq color (cond((= color 1)0)(t 1))))))
	      (setq prev-j j)))))))

(defun updown(x depth)(- bitsize 1 (+ x (- depth))))

(defun make-min-m-table()
  (do((j 0 (1+ j)))((= j bitsize))
    (do((i 0 (1+ i)))
       ((or(= i bitsize)(= 1 (pattern i j)))
	(store(min-m-table j) i)))))

(defun min-bound()
  (let((res bitsize))
    (do((j 0 (1+ j)))((= j bitsize)res)
      (cond((< (min-m-table j) res)(setq res (min-m-table j)))))))

(defun max-non-blank(depth)
  (do((j 0 (1+ j)))((not(= (min-m-table j) bitsize))(updown j depth))))

;(defun tyo8*(ch ofobj)(princ ch)(princ '/ )(tyo8 ch ofobj))

; Font metric file format (TFM file)
; the first 24 bytes = 6 words (each is 16 bits wide)
; 
; lf = length of the entire file in words : 277
; lf = 6 + lh + (ec-bc+1) + nw + nh + nd + ni + nl + nk + ne + np
;    = 6 + 2  + 256       + 2  + 2  + 1  + 1  + 0  + 0  + 0  + 7  = 277
; lh = length of the header data in words : 2
; bc = smallest character code in the font : 0
; ec = largest character code in the font : 255
; nw = number of words in the width table : 2
; nh = number of words in the height table : 2
; nd = number of words in the depth table : 2
; ni = number of words in the italic correction table : 1
; nl = number of words in the lig/kern table : 0
; nk = number of words in the kern table : 0
; ne = number of words in the extensible character table : 0
; np = number of font parameter words : 7
;
; the rest, a sequence of ten data arrays
; in fix_word = 12 bits . 20 bits (i.e. 1.0 = 2^20)
; header: array [0 .. lh-1] of stuff
; header[0] = 32 bit check sum : 
; header[1] = design size in TeX points : 10
; char_info: array [bc .. ec] of char_info_word : #b1000100010000000000000000
;     char_info_word 
;          first byte : width_index : 1
;          second byte : height_index (4 bits) * 16 + depth_index (4 bits) : 16
;          third byte : italic_index (6 bits) * 4 + tag (2 bits) : 0
;          fourth byte : remainder (8 bits) : 0
; width: array [0 .. nw-1] of fix_word
; width[0] : 0
; width[1] : 0.91542 (= 38dots on 300dpi device)
; height: array [0 .. nh-1] of fix_word
; height[0] : 0
; height[1] : 36 dots
; depth: array [0 .. nd-1] of fix_word
; depth[0] : 0
; depth[1] : 2 dots
; italic: array [0 .. ni-1] of fix_word
; italic[0] : 0
; lig_kern: array [0 .. nl-1] of lig_kern_command
; kern: array [0 .. nk-1] of fix_word
; exten: array [0 .. ne-1] of extensible_recipe
; param: array [1 .. np] of fix_word
; param[1] = slant, the amount of italic slant : 0
; param[2] = space, the normal spacing between words in text : 0
; param[3] = space_stretch, the amount of glue stretching between words : 0
; param[4] = space_shrink, the amount of glue shrinking between words : 0
; param[5] = x_height, the size of one ex in the font : 0.91542
; param[6] = quad, the size of one em in the font : 0.91542
; param[7] = extra_space, the amount added to space at the ends of sntncs : 0

(defun out32(ofobj n)(out ofobj (lsh n 4)))
(defun out-two-half-word(hw1 hw2 ofobj)(out32 ofobj (+ (lsh hw1 16.) hw2)))

;; make tfm file
(array width-pointer fixnum 256.)

(defun compute-fix-word(n ds mag)
  (ifix (*$ (//$ (*$ (float n)(//$ 72.27 (float ds))) 300.0 (expt 1.2 mag))
	    (float(^ 2 20.)))))

(defun make-tfm-file
  (fname bchar echar w-in-dot ht-in-dot d-in-dot ds-in-pt sub-font-num)
; call it by (make-tfm-file '|bs:<pattern>new.tfm| 38. 30. 6. 10.)
 (let((ofobj(open fname '(out fixnum)))
      (lf 278.);(lh 2.)
      (lh 4.) ; third header indicates japanese fonts
              ; and fourth sub font number (1 .. 33)
      (bc bchar)(ec echar)
      (nw 2.)(nh 2.)
      (nd 2.)(ni 1.)(nl 0.)(nk 0.)(ne 0.)(np 7.)
      (checksum w-in-dot) ; arbitrary
      (design-size (* ds-in-pt (^ 2 20.)))
      (japanese-font-flag 142857.) ; arbitrary
      (char-info-word #b1000100010000000000000000)
      (char-info-word-0 #b100010000000000000000)
      (width (compute-fix-word w-in-dot ds-in-pt 0))
      (width-list (make-width-list ds-in-pt))
      (height (compute-fix-word ht-in-dot ds-in-pt 0))
      (depth (compute-fix-word d-in-dot ds-in-pt 0)))
   (setq nw (+ nw (length width-list)))
   (setq lf (+ 6 lh (- ec bc -1) nw nh nd ni nl nk ne np))
   (out-two-half-word lf lh ofobj)
   (out-two-half-word bc ec ofobj)
   (out-two-half-word nw nh ofobj)
   (out-two-half-word nd ni ofobj)
   (out-two-half-word nl nk ofobj)
   (out-two-half-word ne np ofobj)
   (out32 ofobj checksum)
   (out32 ofobj design-size)
   (out32 ofobj japanese-font-flag)
   (out32 ofobj sub-font-num) ; integer (1 .. 33)
   (do((i bc (1+ i)))((> i ec))
     (cond((zerop(charloc i))(out32 ofobj char-info-word-0))
	  (t(cond((zerop(width-pointer i))(out32 ofobj char-info-word))
		 (t(out32 ofobj (boole 7 (lsh (width-pointer i)24.)
				       char-info-word-0)))))))
   (out32 ofobj 0)(out32 ofobj width) ; width table
   (cond((null width-list)) ; if there are extra width
	(t(do((l width-list(cdr l)))((null l))(out32 ofobj(car l)))))
   (out32 ofobj 0)(out32 ofobj height) ; height table
   (out32 ofobj 0)(out32 ofobj depth) ; depth table
   (out32 ofobj 0) ; italic table
   (out32 ofobj 0) ; param[1]
   (out32 ofobj 0) ; param[2]
   (out32 ofobj 0) ; param[3]
   (out32 ofobj 0) ; param[4]
   (out32 ofobj width) ; param[5]
   (out32 ofobj width) ; param[6]
   (out32 ofobj 0) ; param[7]
   (close ofobj)))

(defun make-width-list(ds)
  (let((non-zero-width-list(make-non-zero-list))
       (dif-width-list nil)
       (pointer 2))
    (do((i 0 (1+ i)))((= i 256.))(store(width-pointer i)0))
    (do((l non-zero-width-list(cdr l)))
       ((null l)(make-fix-w-list ds dif-width-list))
      (let((pt(assq (cdar l) dif-width-list))
	   (wd(cdar l))
	   (cc(caar l)))
	(cond((null pt)(setq dif-width-list
			     (cons(cons wd pointer)dif-width-list))
		       (store (width-pointer cc)pointer)
		       (setq pointer (1+ pointer)))
	     (t(store(width-pointer cc)(cdr pt))))))))

(defun make-fix-w-list(ds l)
  (do((ll l (cdr ll))(res nil))((null ll)res)
    (setq res(cons (compute-fix-word (car(car ll)) ds 0) res))))

(defun make-non-zero-list()
  (do((i 0 (1+ i))(res nil))((= i 256.)(reverse res))
    (cond((zerop(width-table i)))
	 (t(setq res(cons (cons i (width-table i)) res))))))

(defun set-up-width-table-36() ; for 36 dots fonts
  (clear-width-table)
  (store(width-table 2)20.) ; tou-ten
  (store(width-table 3)18.) ; ku-ten
  (store(width-table 4)15.) ; comma
  (store(width-table 5)21.) ; period
  (store(width-table 11.)23.) ; daku-ten
  (store(width-table 12.)17.) ; han-daku-ten
  (store(width-table 30.)36.) ; hyphen
  (store(width-table 39.)15.) ; left single quotation mark
  (store(width-table 41.)18.) ; left double quotation mark
  (store(width-table 75.)15.) ; do
  (store(width-table 76.)14.) ; fun
  (store(width-table 77.)20.)) ; byou

(defun set-up-width-table-24() ; for 36 dots fonts
  (clear-width-table)
  (store(width-table 2)14.) ; tou-ten
  (store(width-table 3)12.) ; ku-ten
  (store(width-table 4)10.) ; comma
  (store(width-table 5)14.) ; period
  (store(width-table 11.)15.) ; daku-ten
  (store(width-table 12.)11.) ; han-daku-ten
  (store(width-table 30.)24.) ; hyphen
  (store(width-table 39.)10.) ; left single quotation mark
  (store(width-table 41.)12.) ; left double quotation mark
  (store(width-table 75.)10.) ; do
  (store(width-table 76.)9.) ; fun
  (store(width-table 77.)13.)) ; byou

(defun clear-width-table()
  (do((i 0 (1+ i)))((= i 256.))(store(width-table i)0)))

; routines to create actual font files
; It is necessary to have convention for font file names.
; Following is the tentative convention:
;
; It is obvious that 36 does not refer to pt, and it means dot size of dot font
; JSY36 for JIS symbols : 1 and 2 ku
; JROMA36 for JIS romaji (ascii) : 3 ku
; JHIRA36 for JIS hiragana : 4 ku
; JKATA36 for JIS katakana : 5 ku
; JGREEK36 for JIS Greek characters : 6 ku
; JRUSSIAN36 for JIS Russian characters
; JKA36 .. JKL36 for JIS Daiichi Suijun Kanji : 16 .. 47 ku
; JKM36 .. JKZ36 for JIS Daini Suijun Kanji : 48 .. 84 ku
; For 24 dots character, change 36 to 24
;
; For example, GF file for 300 dpi device without magnification should be
; called JSY36.300GF, JKA36.300GF etc.
; Likewise, TFM files should be called as JHIRA36.TFM etc.
;
; And in the future, if you create Japanese fonts using Metafont, name them
; JKA10.300GF for 10 point font etc.

; "hiragana a" = 3 5, "katakana a" = 3 88,
(defun jis(n m)(+ 92. m (* 94. (- n 1)))) ; index for symbols ("A"=2 47)
(defun kanji(n m)(+ 567. (* 94. (- n 16.)) m)) ; index for kanji n-ku m-ten

(declare(special symbol-start romaji-start hiragana-start katakana-start))
(declare(special greek-start russian-start))
(setq symbol-start (jis 1 2))
(setq romaji-start (jis 2 37.))
(setq hiragana-start (jis 3 5))
(setq katakana-start (jis 3 88.))
(setq greek-start (+ katakana-start 86.))
(setq russian-start (+ greek-start 48.))

(declare(special jsy jroma jhira jkata jgreek jrussian jkeisen))
(declare(special jka jkb jkc jkd jke jkf jkg jkh jki jkj jkk jkl))
(declare(special jkm jkn jko jkp jkq jkr jks jkt jku jkv jkw jkx jky jkz))

(setq jsy `((,symbol-start 93. 2)(,(+ symbol-start 93.) 14. 101.)))
(setq jroma `((,romaji-start 10. 48.)
	      (,(+ romaji-start 10.) 26. 65.)
	      (,(+ romaji-start 36.) 26. 97.)))
(setq jhira `((,hiragana-start 83. 1)))
(setq jkata `((,katakana-start 86. 1)))
(setq jgreek `((,greek-start 24. 1)(,(+ greek-start 24.) 24. 33.)))
(setq jrussian `((,russian-start 33. 1)(,(+ russian-start 33.) 33. 49.)))
(setq jka `((,(kanji 16. 1) 256. 0))) ; (kanji 16 1) = 568.
(setq jkb '((824. 256. 0)))
(setq jkc '((1080. 256. 0)))
(setq jkd '((1336. 256. 0)))
(setq jke '((1592. 256. 0)))
(setq jkf '((1848. 256. 0)))
(setq jkg '((2104. 256. 0)))
(setq jkh '((2360. 256. 0)))
(setq jki '((2616. 256. 0)))
(setq jkj '((2872. 256. 0)))
(setq jkk '((3128. 256. 0)))
(setq jkl '((3384. 149. 0)))
(setq jkm '((3533. 256. 0))) ; Daini Suijun
(setq jkn `((,(+ (caar jkm) 256.) 256. 0)))
(setq jko `((,(+ (caar jkn) 256.) 256. 0)))
(setq jkp `((,(+ (caar jko) 256.) 256. 0)))
(setq jkq `((,(+ (caar jkp) 256.) 256. 0)))
(setq jkr `((,(+ (caar jkq) 256.) 256. 0)))
(setq jks `((,(+ (caar jkr) 256.) 256. 0)))
(setq jkt `((,(+ (caar jks) 256.) 256. 0)))
(setq jku `((,(+ (caar jkt) 256.) 256. 0)))
(setq jkv `((,(+ (caar jku) 256.) 256. 0)))
(setq jkw `((,(+ (caar jkv) 256.) 256. 0)))
(setq jkx `((,(+ (caar jkw) 256.) 256. 0)))
(setq jky `((,(+ (caar jkx) 256.) 256. 0)))
(setq jkz `((,(+ (caar jky) 256.) 56. 0))) ; 60 for new JIS

; Following routines are used to make originally used fonts
; from Takusou-shitsu (obsolete)

(defun make-all-gf-tfm-36()
  (setq bitsize 36.)
  (setq infobj (open '|bs:<pattern>kanji48.bit| '(in fixnum)))
  (clear-width-table)
;  (set-up-width-table-36)
  (make-gf-file 'jsy36 38. 30. 6. 10. jsy 1. T 0)
;  (clear-width-table)
  (make-gf-file 'jroma36 38. 30. 6. 10. jroma 2. T 0)
  (make-gf-file 'jhira36 38. 30. 6. 10. jhira 3. T 0)
  (make-gf-file 'jkata36 38. 30. 6. 10. jkata 4. T 0)
  (make-gf-file 'jgreek36 38. 30. 6. 10. jgreek 5. T 0)
  (make-gf-file 'jrussian36 38. 30. 6. 10. jrussian 6. T 0)
  (make-gf-file 'jka36 38. 30. 6. 10. jka 8. T 0)
  (make-gf-file 'jkb36 38. 30. 6. 10. jkb 9. T 0)
  (make-gf-file 'jkc36 38. 30. 6. 10. jkc 10. T 0)
  (make-gf-file 'jkd36 38. 30. 6. 10. jkd 11. T 0)
  (make-gf-file 'jke36 38. 30. 6. 10. jke 12. T 0)
  (make-gf-file 'jkf36 38. 30. 6. 10. jkf 13. T 0)
  (make-gf-file 'jkg36 38. 30. 6. 10. jkg 14. T 0)
  (make-gf-file 'jkh36 38. 30. 6. 10. jkh 15. T 0)
  (make-gf-file 'jki36 38. 30. 6. 10. jki 16. T 0)
  (make-gf-file 'jkj36 38. 30. 6. 10. jkj 17. T 0)
  (make-gf-file 'jkk36 38. 30. 6. 10. jkk 18. T 0)
  (make-gf-file 'jkl36 38. 30. 6. 10. jkl 19. T 0)
  (make-gf-file 'jkm36 38. 30. 6. 10. jkm 20. T 0)
  (make-gf-file 'jkn36 38. 30. 6. 10. jkn 21. T 0)
  (make-gf-file 'jko36 38. 30. 6. 10. jko 22. T 0)
  (make-gf-file 'jkp36 38. 30. 6. 10. jkp 23. T 0)
  (make-gf-file 'jkq36 38. 30. 6. 10. jkq 24. T 0)
  (make-gf-file 'jkr36 38. 30. 6. 10. jkr 25. T 0)
  (make-gf-file 'jks36 38. 30. 6. 10. jks 26. T 0)
  (make-gf-file 'jkt36 38. 30. 6. 10. jkt 27. T 0)
  (make-gf-file 'jku36 38. 30. 6. 10. jku 28. T 0)
  (make-gf-file 'jkv36 38. 30. 6. 10. jkv 29. T 0)
  (make-gf-file 'jkw36 38. 30. 6. 10. jkw 30. T 0)
  (make-gf-file 'jkx36 38. 30. 6. 10. jkx 31. T 0)
  (make-gf-file 'jky36 38. 30. 6. 10. jky 32. T 0)
  (make-gf-file 'jkz36 38. 30. 6. 10. jkz 33. T 0)
  (close infobj))

(defun make-all-gf-tfm-24()
  (setq bitsize 24.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
;  (set-up-width-table-24)
  (make-gf-file 'jsy24 26. 20. 4. 7. jsy 1. T 0)
;  (clear-width-table)
  (make-gf-file 'jroma24 26. 20. 4. 7. jroma 2. T 0)
  (make-gf-file 'jhira24 26. 20. 4. 7. jhira 3. T 0)
  (make-gf-file 'jkata24 26. 20. 4. 7. jkata 4. T 0)
  (make-gf-file 'jgreek24 26. 20. 4. 7. jgreek 5. T 0)
  (make-gf-file 'jrussian24 26. 20. 4. 7. jrussian 6. T 0)
  (make-gf-file 'jka24 26. 20. 4. 7. jka 8. T 0)
  (make-gf-file 'jkb24 26. 20. 4. 7. jkb 9. T 0)
  (make-gf-file 'jkc24 26. 20. 4. 7. jkc 10. T 0)
  (make-gf-file 'jkd24 26. 20. 4. 7. jkd 11. T 0)
  (make-gf-file 'jke24 26. 20. 4. 7. jke 12. T 0)
  (make-gf-file 'jkf24 26. 20. 4. 7. jkf 13. T 0)
  (make-gf-file 'jkg24 26. 20. 4. 7. jkg 14. T 0)
  (make-gf-file 'jkh24 26. 20. 4. 7. jkh 15. T 0)
  (make-gf-file 'jki24 26. 20. 4. 7. jki 16. T 0)
  (make-gf-file 'jkj24 26. 20. 4. 7. jkj 17. T 0)
  (make-gf-file 'jkk24 26. 20. 4. 7. jkk 18. T 0)
  (make-gf-file 'jkl24 26. 20. 4. 7. jkl 19. T 0)
  (make-gf-file 'jkm24 26. 20. 4. 7. jkm 20. T 0)
  (make-gf-file 'jkn24 26. 20. 4. 7. jkn 21. T 0)
  (make-gf-file 'jko24 26. 20. 4. 7. jko 22. T 0)
  (make-gf-file 'jkp24 26. 20. 4. 7. jkp 23. T 0)
  (make-gf-file 'jkq24 26. 20. 4. 7. jkq 24. T 0)
  (make-gf-file 'jkr24 26. 20. 4. 7. jkr 25. T 0)
  (make-gf-file 'jks24 26. 20. 4. 7. jks 26. T 0)
  (make-gf-file 'jkt24 26. 20. 4. 7. jkt 27. T 0)
  (make-gf-file 'jku24 26. 20. 4. 7. jku 28. T 0)
  (make-gf-file 'jkv24 26. 20. 4. 7. jkv 29. T 0)
  (make-gf-file 'jkw24 26. 20. 4. 7. jkw 30. T 0)
  (make-gf-file 'jkx24 26. 20. 4. 7. jkx 31. T 0)
  (make-gf-file 'jky24 26. 20. 4. 7. jky 32. T 0)
  (make-gf-file 'jkz24 26. 20. 4. 7. jkz 33. T 0)
  (close infobj))

(defun make-all-gf-tfm-48()
  (setq bitsize 48.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy48 50. 40. 8. 14. jsy 1. T 0)
  (make-gf-file 'jroma48 50. 40. 8. 14. jroma 2. T 0)
  (make-gf-file 'jhira48 50. 40. 8. 14. jhira 3. T 0)
  (make-gf-file 'jkata48 50. 40. 8. 14. jkata 4. T 0)
  (make-gf-file 'jgreek48 50. 40. 8. 14. jgreek 5. T 0)
  (make-gf-file 'jrussian48 50. 40. 8. 14. jrussian 6. T 0)
  (make-gf-file 'jka48 50. 40. 8. 14. jka 8. T 0)
  (make-gf-file 'jkb48 50. 40. 8. 14. jkb 9. T 0)
  (make-gf-file 'jkc48 50. 40. 8. 14. jkc 10. T 0)
  (make-gf-file 'jkd48 50. 40. 8. 14. jkd 11. T 0)
  (make-gf-file 'jke48 50. 40. 8. 14. jke 12. T 0)
  (make-gf-file 'jkf48 50. 40. 8. 14. jkf 13. T 0)
  (make-gf-file 'jkg48 50. 40. 8. 14. jkg 14. T 0)
  (make-gf-file 'jkh48 50. 40. 8. 14. jkh 15. T 0)
  (make-gf-file 'jki48 50. 40. 8. 14. jki 16. T 0)
  (make-gf-file 'jkj48 50. 40. 8. 14. jkj 17. T 0)
  (make-gf-file 'jkk48 50. 40. 8. 14. jkk 18. T 0)
  (make-gf-file 'jkl48 50. 40. 8. 14. jkl 19. T 0)
  (make-gf-file 'jkm48 50. 40. 8. 14. jkm 20. T 0)
  (make-gf-file 'jkn48 50. 40. 8. 14. jkn 21. T 0)
  (make-gf-file 'jko48 50. 40. 8. 14. jko 22. T 0)
  (make-gf-file 'jkp48 50. 40. 8. 14. jkp 23. T 0)
  (make-gf-file 'jkq48 50. 40. 8. 14. jkq 24. T 0)
  (make-gf-file 'jkr48 50. 40. 8. 14. jkr 25. T 0)
  (make-gf-file 'jks48 50. 40. 8. 14. jks 26. T 0)
  (make-gf-file 'jkt48 50. 40. 8. 14. jkt 27. T 0)
  (make-gf-file 'jku48 50. 40. 8. 14. jku 28. T 0)
  (make-gf-file 'jkv48 50. 40. 8. 14. jkv 29. T 0)
  (make-gf-file 'jkw48 50. 40. 8. 14. jkw 30. T 0)
  (make-gf-file 'jkx48 50. 40. 8. 14. jkx 31. T 0)
  (make-gf-file 'jky48 50. 40. 8. 14. jky 32. T 0)
  (make-gf-file 'jkz48 50. 40. 8. 14. jkz 33. T 0)
  (close infobj))

(defun make-all-gf-tfm-72()
  (setq bitsize 72.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy72 74. 60. 12. 20. jsy 1. T 0)
  (make-gf-file 'jroma72 74. 60. 12. 20. jroma 2. T 0)
  (make-gf-file 'jhira72 74. 60. 12. 20. jhira 3. T 0)
  (make-gf-file 'jkata72 74. 60. 12. 20. jkata 4. T 0)
  (make-gf-file 'jgreek72 74. 60. 12. 20. jgreek 5. T 0)
  (make-gf-file 'jrussian72 74. 60. 12. 20. jrussian 6. T 0)
  (make-gf-file 'jka72 74. 60. 12. 20. jka 8. T 0)
  (make-gf-file 'jkb72 74. 60. 12. 20. jkb 9. T 0)
  (make-gf-file 'jkc72 74. 60. 12. 20. jkc 10. T 0)
  (make-gf-file 'jkd72 74. 60. 12. 20. jkd 11. T 0)
  (make-gf-file 'jke72 74. 60. 12. 20. jke 12. T 0)
  (make-gf-file 'jkf72 74. 60. 12. 20. jkf 13. T 0)
  (make-gf-file 'jkg72 74. 60. 12. 20. jkg 14. T 0)
  (make-gf-file 'jkh72 74. 60. 12. 20. jkh 15. T 0)
  (make-gf-file 'jki72 74. 60. 12. 20. jki 16. T 0)
  (make-gf-file 'jkj72 74. 60. 12. 20. jkj 17. T 0)
  (make-gf-file 'jkk72 74. 60. 12. 20. jkk 18. T 0)
  (make-gf-file 'jkl72 74. 60. 12. 20. jkl 19. T 0)
  (make-gf-file 'jkm72 74. 60. 12. 20. jkm 20. T 0)
  (make-gf-file 'jkn72 74. 60. 12. 20. jkn 21. T 0)
  (make-gf-file 'jko72 74. 60. 12. 20. jko 22. T 0)
  (make-gf-file 'jkp72 74. 60. 12. 20. jkp 23. T 0)
  (make-gf-file 'jkq72 74. 60. 12. 20. jkq 24. T 0)
  (make-gf-file 'jkr72 74. 60. 12. 20. jkr 25. T 0)
  (make-gf-file 'jks72 74. 60. 12. 20. jks 26. T 0)
  (make-gf-file 'jkt72 74. 60. 12. 20. jkt 27. T 0)
  (make-gf-file 'jku72 74. 60. 12. 20. jku 28. T 0)
  (make-gf-file 'jkv72 74. 60. 12. 20. jkv 29. T 0)
  (make-gf-file 'jkw72 74. 60. 12. 20. jkw 30. T 0)
  (make-gf-file 'jkx72 74. 60. 12. 20. jkx 31. T 0)
  (make-gf-file 'jky72 74. 60. 12. 20. jky 32. T 0)
  (make-gf-file 'jkz72 74. 60. 12. 20. jkz 33. T 0)
  (close infobj))

(declare(special test-index-list))
(setq test-index-list `((,symbol-start 5 1)(,(kanji 16. 1) 5. 101.)))

(defun test()
  (setq bitsize 36.)
  (setq infobj (open '|bs:<pattern>kanji48.bit| '(in fixnum)))
;  (set-up-width-table-36)
  (make-gf-file 'jsy36 38. 30. 6. 10. jsy 1. T 0))

(defun test2()
  (setq bitsize 72.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
;  (set-up-width-table-24)
  (make-gf-file 'jsy72 74. 60. 12. 20. jsy 1. T 0))

(defun test-gf(n m)
  (setq bitsize 72.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (make-gf-file 'test2 74. 60. 12. 20. (list(list n m 0)) 1. T 0))

;; addition to make font from Gasho available to JTeX

(defun jc(m n) ; returns serial number for a JIS character at m-ku n-ten
  (+ n (* 94. (1- m))))

(defun make-all-gf-tfm-32()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32.dat| '(in fixnum)))
  (clear-width-table)
  (setq jsy `((,(jc 1 2) 93. 2)(,(jc 2 1) 14. 101.)))
  (setq jroma `((,(jc 3 16.) 10. 48.)(,(jc 3 33.) 26. 65.)
				     (,(jc 3 65.) 26. 97.)))
  (setq jhira `((,(jc 4 1) 83. 1)))
  (setq jkata `((,(jc 5 1) 86. 1)))
; 6-ku 24-ten (capital omega) is missing, there is no replacement
; so it is left blank
  (setq jgreek `((,(jc 6 1) 24. 1)(,(jc 6 33.) 24. 33.)))
; 7-ku 1-ten is missing from this font, so I reuse 6-ku 1-ten for that.
  (setq jrussian `((,(jc 6 1) 1 1)(,(jc 7 2) 32. 2)(,(jc 7 49.) 33. 49.)))
  (setq jka `((,(jc 16. 1) 256. 0)))
  (setq jkb `((,(jc 18. 69.) 256. 0)))
  (setq jkc `((,(jc 21. 43.) 256. 0)))
  (setq jkd `((,(jc 24. 17.) 256. 0)))
  (setq jke `((,(jc 26. 85.) 256. 0)))
  (setq jkf `((,(jc 29. 59.) 256. 0)))
  (setq jkg `((,(jc 32. 33.) 256. 0)))
  (setq jkh `((,(jc 35. 7.) 256. 0)))
  (setq jki `((,(jc 37. 75.) 256. 0)))
  (setq jkj `((,(jc 40. 49.) 256. 0)))
  (setq jkk `((,(jc 43. 23.) 256. 0)))
  (setq jkl `((,(jc 45. 91.) 256. 0)))

  (make-gf-file 'g1jsy32 34. 27. 5. 9. jsy 1. T 0)
  (make-gf-file 'g1jroma32 34. 27. 5. 9. jroma 2. T 0)
  (make-gf-file 'g1jhira32 34. 27. 5. 9. jhira 3. T 0)
  (make-gf-file 'g1jkata32 34. 27. 5. 9. jkata 4. T 0)
  (make-gf-file 'g1jgreek32 34. 27. 5. 9. jgreek 5. T 0)
  (make-gf-file 'g1jrussian32 34. 27. 5. 9. jrussian 6. T 0)
  (make-gf-file 'g1jka32 34. 27. 5. 9. jka 8. T 0)
  (make-gf-file 'g1jkb32 34. 27. 5. 9. jkb 9. T 0)
  (make-gf-file 'g1jkc32 34. 27. 5. 9. jkc 10. T 0)
  (make-gf-file 'g1jkd32 34. 27. 5. 9. jkd 11. T 0)
  (make-gf-file 'g1jke32 34. 27. 5. 9. jke 12. T 0)
  (make-gf-file 'g1jkf32 34. 27. 5. 9. jkf 13. T 0)
  (make-gf-file 'g1jkg32 34. 27. 5. 9. jkg 14. T 0)
  (make-gf-file 'g1jkh32 34. 27. 5. 9. jkh 15. T 0)
  (make-gf-file 'g1jki32 34. 27. 5. 9. jki 16. T 0)
  (make-gf-file 'g1jkj32 34. 27. 5. 9. jkj 17. T 0)
  (make-gf-file 'g1jkk32 34. 27. 5. 9. jkk 18. T 0)
  (make-gf-file 'g1jkl32 34. 27. 5. 9. jkl 19. T 0)
  (close infobj))


(defun make-all-gf-tfm-32-2()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32-2.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jgftemp|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jtfmtemp|))
  (setq jsy `((,(jc 1 2) 93. 2)(,(jc 2 1) 14. 101.)))
  (setq jroma `((,(jc 3 16.) 10. 48.)(,(jc 3 33.) 26. 65.)
				     (,(jc 3 65.) 26. 97.)))
  (setq jhira `((,(jc 4 1) 83. 1)))
  (setq jkata `((,(jc 5 1) 86. 1)))
  (setq jgreek `((,(jc 6 1) 24. 1)(,(jc 6 33.) 24. 33.)))
; 7-ku 1-ten is missing from this font, so I reuse 6-ku 1-ten for that.
  (setq jrussian `((,(jc 6 1) 1 1)(,(jc 7 2) 32. 2)(,(jc 7 49.) 33. 49.)))
  (setq jka `((,(jc 16. 1) 256. 0)))
  (setq jkb `((,(jc 18. 69.) 256. 0)))
  (setq jkc `((,(jc 21. 43.) 256. 0)))
  (setq jkd `((,(jc 24. 17.) 256. 0)))
  (setq jke `((,(jc 26. 85.) 256. 0)))
  (setq jkf `((,(jc 29. 59.) 256. 0)))
  (setq jkg `((,(jc 32. 33.) 256. 0)))
  (setq jkh `((,(jc 35. 7.) 256. 0)))
  (setq jki `((,(jc 37. 75.) 256. 0)))
  (setq jkj `((,(jc 40. 49.) 256. 0)))
  (setq jkk `((,(jc 43. 23.) 256. 0)))
  (setq jkl `((,(jc 45. 91.) 256. 0)))

  (make-gf-file 'g2jsy32 34. 27. 5. 9. jsy 1. T 0)
  (make-gf-file 'g2jroma32 34. 27. 5. 9. jroma 2. T 0)
  (make-gf-file 'g2jhira32 34. 27. 5. 9. jhira 3. T 0)
  (make-gf-file 'g2jkata32 34. 27. 5. 9. jkata 4. T 0)
  (make-gf-file 'g2jgreek32 34. 27. 5. 9. jgreek 5. T 0)
  (make-gf-file 'g2jrussian32 34. 27. 5. 9. jrussian 6. T 0)
  (make-gf-file 'g2jka32 34. 27. 5. 9. jka 8. T 0)
  (make-gf-file 'g2jkb32 34. 27. 5. 9. jkb 9. T 0)
  (make-gf-file 'g2jkc32 34. 27. 5. 9. jkc 10. T 0)
  (make-gf-file 'g2jkd32 34. 27. 5. 9. jkd 11. T 0)
  (make-gf-file 'g2jke32 34. 27. 5. 9. jke 12. T 0)
  (make-gf-file 'g2jkf32 34. 27. 5. 9. jkf 13. T 0)
  (make-gf-file 'g2jkg32 34. 27. 5. 9. jkg 14. T 0)
  (make-gf-file 'g2jkh32 34. 27. 5. 9. jkh 15. T 0)
  (make-gf-file 'g2jki32 34. 27. 5. 9. jki 16. T 0)
  (make-gf-file 'g2jkj32 34. 27. 5. 9. jkj 17. T 0)
  (make-gf-file 'g2jkk32 34. 27. 5. 9. jkk 18. T 0)
  (make-gf-file 'g2jkl32 34. 27. 5. 9. jkl 19. T 0)
  (close infobj))

(defun make-all-gf-tfm-32-3()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32-3.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jgftemp2|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jtfmtemp2|))
  (setq jsy `((,(jc 1 2) 93. 2)(,(jc 2 1) 14. 101.)))
  (setq jroma `((,(jc 3 16.) 10. 48.)(,(jc 3 33.) 26. 65.)
				     (,(jc 3 65.) 26. 97.)))
  (setq jhira `((,(jc 4 1) 83. 1)))
  (setq jkata `((,(jc 5 1) 86. 1)))
  (setq jgreek `((,(jc 6 1) 24. 1)(,(jc 6 33.) 24. 33.)))
; 7-ku 1-ten is missing from this font, so I reuse 6-ku 1-ten for that.
  (setq jrussian `((,(jc 6 1) 1 1)(,(jc 7 2) 32. 2)(,(jc 7 49.) 33. 49.)))
  (setq jka `((,(jc 16. 1) 256. 0)))
  (setq jkb `((,(jc 18. 69.) 256. 0)))
  (setq jkc `((,(jc 21. 43.) 256. 0)))
  (setq jkd `((,(jc 24. 17.) 256. 0)))
  (setq jke `((,(jc 26. 85.) 256. 0)))
  (setq jkf `((,(jc 29. 59.) 256. 0)))
  (setq jkg `((,(jc 32. 33.) 256. 0)))
  (setq jkh `((,(jc 35. 7.) 256. 0)))
  (setq jki `((,(jc 37. 75.) 256. 0)))
  (setq jkj `((,(jc 40. 49.) 256. 0)))
  (setq jkk `((,(jc 43. 23.) 256. 0)))
  (setq jkl `((,(jc 45. 91.) 256. 0)))

  (make-gf-file 'g3jsy32 34. 27. 5. 9. jsy 1. T 0)
  (make-gf-file 'g3jroma32 34. 27. 5. 9. jroma 2. T 0)
  (make-gf-file 'g3jhira32 34. 27. 5. 9. jhira 3. T 0)
  (make-gf-file 'g3jkata32 34. 27. 5. 9. jkata 4. T 0)
  (make-gf-file 'g3jgreek32 34. 27. 5. 9. jgreek 5. T 0)
  (make-gf-file 'g3jrussian32 34. 27. 5. 9. jrussian 6. T 0)
  (make-gf-file 'g3jka32 34. 27. 5. 9. jka 8. T 0)
  (make-gf-file 'g3jkb32 34. 27. 5. 9. jkb 9. T 0)
  (make-gf-file 'g3jkc32 34. 27. 5. 9. jkc 10. T 0)
  (make-gf-file 'g3jkd32 34. 27. 5. 9. jkd 11. T 0)
  (make-gf-file 'g3jke32 34. 27. 5. 9. jke 12. T 0)
  (make-gf-file 'g3jkf32 34. 27. 5. 9. jkf 13. T 0)
  (make-gf-file 'g3jkg32 34. 27. 5. 9. jkg 14. T 0)
  (make-gf-file 'g3jkh32 34. 27. 5. 9. jkh 15. T 0)
  (make-gf-file 'g3jki32 34. 27. 5. 9. jki 16. T 0)
  (make-gf-file 'g3jkj32 34. 27. 5. 9. jkj 17. T 0)
  (make-gf-file 'g3jkk32 34. 27. 5. 9. jkk 18. T 0)
  (make-gf-file 'g3jkl32 34. 27. 5. 9. jkl 19. T 0)
  (close infobj))

;; I found a bug in the definition of jsy above, so I will remake all
;; jsy files in Japanese font

(defun remake-jsy-gf-tfm-36()
  (setq bitsize 36.)
  (setq infobj (open '|bs:<pattern>kanji48.bit| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy36 38. 30. 6. 10. jsy 1. T 0))

(defun remake-jsy-gf-tfm-24()
  (setq bitsize 24.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy24 26. 20. 4. 7. jsy 1. T 0))

(defun remake-jsy-gf-tfm-48()
  (setq bitsize 48.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy48 50. 40. 8. 14. jsy 1. T 0))

(defun remake-jsy-gf-tfm-72()
  (setq bitsize 72.)
  (setq infobj (open '|bs:<pattern>kanjz.txt8| '(in fixnum)))
  (clear-width-table)
  (make-gf-file 'jsy72 74. 60. 12. 20. jsy 1. T 0))

(defun make-jroma-gf-tfm-32()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32.dat| '(in fixnum)))
  (clear-width-table)
  (setq jroma `((,(jc 3 16.) 10. 48.)(,(jc 3 33.) 26. 65.)
				     (,(jc 3 65.) 26. 97.)))
  (make-gf-file 'jroma32 34. 27. 5. 9. jroma 2. T 0))

(defun remake-jgreek-gf-tfm-32()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32.dat| '(in fixnum)))
  (clear-width-table)
; 6-ku 24-ten (capital omega) is missing, there is no replacement
; so it is left blank
  (setq jgreek `((,(jc 6 1) 23. 1)(,(jc 6 33.) 24. 33.)))
  (make-gf-file 'jgreek32 34. 27. 5. 9. jgreek 5. T 0))

(defun make-jsy-gf-tfm-32()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>gfont32.dat| '(in fixnum)))
  (clear-width-table)
  (setq jsy `((,(jc 1 2) 93. 2)(,(jc 2 1) 14. 101.)))
  (make-gf-file 'jsy32 34. 27. 5. 9. jsy 1. T 0))

;; addition to make font files from jis 24 dot font

(defun make-jis-font()
  (make-jis-gf-tfm-24)
  (make-jis-gf-tfm-48)
  (make-jis-gf-tfm-72))

(defun make-jis-gf-tfm-24()
  (setq bitsize 24.)
  (setq infobj (open '|bs:<pattern>font24.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jisfont|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jisfont|))
  (setq jsy `((,(jc 1 2) 93. ,(jc 1 2))(95. 14. 101.)(109. 8. 126.)
				       (117. 7. 142.)(124. 15. 160.)
				       (139. 8. 182.)(147. 1. 194.)))
  (setq jroma '((148. 10. 48.)(158. 26. 65.)(184. 26. 97.)))
  (setq jhira '((210. 83. 1)))
  (setq jkata '((293. 86. 1)))
  (setq jgreek '((379. 24. 1)(403. 24. 33.)))
  (setq jrussian '((427. 33. 1)(460. 33. 49.)))
  (setq jkeisen '((493. 32. 1)))
  (setq jka '((525. 256. 0)))
  (setq jkb '((781. 256. 0)))
  (setq jkc '((1037. 256. 0)))
  (setq jkd '((1293. 256. 0)))
  (setq jke '((1549. 256. 0)))
  (setq jkf '((1805. 256. 0)))
  (setq jkg '((2061. 256. 0)))
  (setq jkh '((2317. 256. 0)))
  (setq jki '((2573. 256. 0)))
  (setq jkj '((2829. 256. 0)))
  (setq jkk '((3085. 256. 0)))
  (setq jkl '((3341. 149. 0)))
  (setq jkm '((3490. 256. 0))) ; Daini suijun
  (setq jkn '((3746. 256. 0)))
  (setq jko '((4002. 256. 0)))
  (setq jkp '((4258. 256. 0)))
  (setq jkq '((4514. 256. 0)))
  (setq jkr '((4770. 256. 0)))
  (setq jks '((5026. 256. 0)))
  (setq jkt '((5282. 256. 0)))
  (setq jku '((5538. 256. 0)))
  (setq jkv '((5794. 256. 0)))
  (setq jkw '((6050. 256. 0)))
  (setq jkx '((6306. 256. 0)))
  (setq jky '((6562. 256. 0)))
  (setq jkz '((6818. 60. 0)))

  (make-gf-file 'jisjsy24 24. 20. 4. 6. jsy 1. T 0)
  (make-gf-file 'jisjroma24 24. 20. 4. 6. jroma 2. T 0)
  (make-gf-file 'jisjhira24 24. 20. 4. 6. jhira 3. T 0)
  (make-gf-file 'jisjkata24 24. 20. 4. 6. jkata 4. T 0)
  (make-gf-file 'jisjgreek24 24. 20. 4. 6. jgreek 5. T 0)
  (make-gf-file 'jisjrussian24 24. 20. 4. 6. jrussian 6. T 0)
  (make-gf-file 'jisjkeisen24 24. 20. 4. 6. jkeisen 7. T 0)
  (make-gf-file 'jisjka24 24. 20. 4. 6. jka 8. T 0)
  (make-gf-file 'jisjkb24 24. 20. 4. 6. jkb 9. T 0)
  (make-gf-file 'jisjkc24 24. 20. 4. 6. jkc 10. T 0)
  (make-gf-file 'jisjkd24 24. 20. 4. 6. jkd 11. T 0)
  (make-gf-file 'jisjke24 24. 20. 4. 6. jke 12. T 0)
  (make-gf-file 'jisjkf24 24. 20. 4. 6. jkf 13. T 0)
  (make-gf-file 'jisjkg24 24. 20. 4. 6. jkg 14. T 0)
  (make-gf-file 'jisjkh24 24. 20. 4. 6. jkh 15. T 0)
  (make-gf-file 'jisjki24 24. 20. 4. 6. jki 16. T 0)
  (make-gf-file 'jisjkj24 24. 20. 4. 6. jkj 17. T 0)
  (make-gf-file 'jisjkk24 24. 20. 4. 6. jkk 18. T 0)
  (make-gf-file 'jisjkl24 24. 20. 4. 6. jkl 19. T 0)
  (make-gf-file 'jisjkm24 24. 20. 4. 6. jkm 20. T 0)
  (make-gf-file 'jisjkn24 24. 20. 4. 6. jkn 21. T 0)
  (make-gf-file 'jisjko24 24. 20. 4. 6. jko 22. T 0)
  (make-gf-file 'jisjkp24 24. 20. 4. 6. jkp 23. T 0)
  (make-gf-file 'jisjkq24 24. 20. 4. 6. jkq 24. T 0)
  (make-gf-file 'jisjkr24 24. 20. 4. 6. jkr 25. T 0)
  (make-gf-file 'jisjks24 24. 20. 4. 6. jks 24. T 0)
  (make-gf-file 'jisjkt24 24. 20. 4. 6. jkt 27. T 0)
  (make-gf-file 'jisjku24 24. 20. 4. 6. jku 28. T 0)
  (make-gf-file 'jisjkv24 24. 20. 4. 6. jkv 29. T 0)
  (make-gf-file 'jisjkw24 24. 20. 4. 6. jkw 30. T 0)
  (make-gf-file 'jisjkx24 24. 20. 4. 6. jkx 31. T 0)
  (make-gf-file 'jisjky24 24. 20. 4. 6. jky 32. T 0)
  (make-gf-file 'jisjkz24 24. 20. 4. 6. jkz 33. T 0)
  (close infobj))

(defun make-jis-gf-tfm-36()
  (setq bitsize 36.)
  (setq infobj (open '|bs:<pattern>jisfont36.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jisfont|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jisfont|))
  (setq jsy `((,(jc 1 2) 93. ,(jc 1 2))(95. 14. 101.)(109. 8. 126.)
				       (117. 7. 142.)(124. 15. 160.)
				       (139. 8. 182.)(147. 1. 194.)))
  (setq jroma '((148. 10. 48.)(158. 26. 65.)(184. 26. 97.)))
  (setq jhira '((210. 83. 1)))
  (setq jkata '((293. 86. 1)))
  (setq jgreek '((379. 24. 1)(403. 24. 33.)))
  (setq jrussian '((427. 33. 1)(460. 33. 49.)))
  (setq jkeisen '((493. 32. 1)))
  (setq jka '((525. 256. 0)))
  (setq jkb '((781. 256. 0)))
  (setq jkc '((1037. 256. 0)))
  (setq jkd '((1293. 256. 0)))
  (setq jke '((1549. 256. 0)))
  (setq jkf '((1805. 256. 0)))
  (setq jkg '((2061. 256. 0)))
  (setq jkh '((2317. 256. 0)))
  (setq jki '((2573. 256. 0)))
  (setq jkj '((2829. 256. 0)))
  (setq jkk '((3085. 256. 0)))
  (setq jkl '((3341. 149. 0)))
  (setq jkm '((3490. 256. 0))) ; Daini suijun
  (setq jkn '((3746. 256. 0)))
  (setq jko '((4002. 256. 0)))
  (setq jkp '((4258. 256. 0)))
  (setq jkq '((4514. 256. 0)))
  (setq jkr '((4770. 256. 0)))
  (setq jks '((5026. 256. 0)))
  (setq jkt '((5282. 256. 0)))
  (setq jku '((5538. 256. 0)))
  (setq jkv '((5794. 256. 0)))
  (setq jkw '((6050. 256. 0)))
  (setq jkx '((6306. 256. 0)))
  (setq jky '((6562. 256. 0)))
  (setq jkz '((6818. 60. 0)))

  (make-gf-file 'jisjsy36 36. 30. 6. 10. jsy 1. T 0)
  (make-gf-file 'jisjroma36 36. 30. 6. 10. jroma 2. T 0)
  (make-gf-file 'jisjhira36 36. 30. 6. 10. jhira 3. T 0)
  (make-gf-file 'jisjkata36 36. 30. 6. 10. jkata 4. T 0)
  (make-gf-file 'jisjgreek36 36. 30. 6. 10. jgreek 5. T 0)
  (make-gf-file 'jisjrussian36 36. 30. 6. 10. jrussian 6. T 0)
  (make-gf-file 'jisjkeisen36 36. 30. 6. 10. jkeisen 7. T 0)
  (make-gf-file 'jisjka36 36. 30. 6. 10. jka 8. T 0)
  (make-gf-file 'jisjkb36 36. 30. 6. 10. jkb 9. T 0)
  (make-gf-file 'jisjkc36 36. 30. 6. 10. jkc 10. T 0)
  (make-gf-file 'jisjkd36 36. 30. 6. 10. jkd 11. T 0)
  (make-gf-file 'jisjke36 36. 30. 6. 10. jke 12. T 0)
  (make-gf-file 'jisjkf36 36. 30. 6. 10. jkf 13. T 0)
  (make-gf-file 'jisjkg36 36. 30. 6. 10. jkg 14. T 0)
  (make-gf-file 'jisjkh36 36. 30. 6. 10. jkh 15. T 0)
  (make-gf-file 'jisjki36 36. 30. 6. 10. jki 16. T 0)
  (make-gf-file 'jisjkj36 36. 30. 6. 10. jkj 17. T 0)
  (make-gf-file 'jisjkk36 36. 30. 6. 10. jkk 18. T 0)
  (make-gf-file 'jisjkl36 36. 30. 6. 10. jkl 19. T 0)
  (make-gf-file 'jisjkm36 36. 30. 6. 10. jkm 20. T 0)
  (make-gf-file 'jisjkn36 36. 30. 6. 10. jkn 21. T 0)
  (make-gf-file 'jisjko36 36. 30. 6. 10. jko 22. T 0)
  (make-gf-file 'jisjkp36 36. 30. 6. 10. jkp 23. T 0)
  (make-gf-file 'jisjkq36 36. 30. 6. 10. jkq 24. T 0)
  (make-gf-file 'jisjkr36 36. 30. 6. 10. jkr 25. T 0)
  (make-gf-file 'jisjks36 36. 30. 6. 10. jks 26. T 0)
  (make-gf-file 'jisjkt36 36. 30. 6. 10. jkt 27. T 0)
  (make-gf-file 'jisjku36 36. 30. 6. 10. jku 28. T 0)
  (make-gf-file 'jisjkv36 36. 30. 6. 10. jkv 29. T 0)
  (make-gf-file 'jisjkw36 36. 30. 6. 10. jkw 30. T 0)
  (make-gf-file 'jisjkx36 36. 30. 6. 10. jkx 31. T 0)
  (make-gf-file 'jisjky36 36. 30. 6. 10. jky 32. T 0)
  (make-gf-file 'jisjkz36 36. 30. 6. 10. jkz 33. T 0)
  (close infobj))

(defun make-jis-gf-tfm-48()
  (setq bitsize 48.)
  (setq infobj (open '|bs:<pattern>font24.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jisfont|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jisfont|))
  (setq jsy `((,(jc 1 2) 93. ,(jc 1 2))(95. 14. 101.)(109. 8. 126.)
				       (117. 7. 142.)(124. 15. 160.)
				       (139. 8. 182.)(147. 1. 194.)))
  (setq jroma '((148. 10. 48.)(158. 26. 65.)(184. 26. 97.)))
  (setq jhira '((210. 83. 1)))
  (setq jkata '((293. 86. 1)))
  (setq jgreek '((379. 24. 1)(403. 24. 33.)))
  (setq jrussian '((427. 33. 1)(460. 33. 49.)))
  (setq jkeisen '((493. 32. 1)))
  (setq jka '((525. 256. 0)))
  (setq jkb '((781. 256. 0)))
  (setq jkc '((1037. 256. 0)))
  (setq jkd '((1293. 256. 0)))
  (setq jke '((1549. 256. 0)))
  (setq jkf '((1805. 256. 0)))
  (setq jkg '((2061. 256. 0)))
  (setq jkh '((2317. 256. 0)))
  (setq jki '((2573. 256. 0)))
  (setq jkj '((2829. 256. 0)))
  (setq jkk '((3085. 256. 0)))
  (setq jkl '((3341. 149. 0)))
  (setq jkm '((3490. 256. 0))) ; Daini suijun
  (setq jkn '((3746. 256. 0)))
  (setq jko '((4002. 256. 0)))
  (setq jkp '((4258. 256. 0)))
  (setq jkq '((4514. 256. 0)))
  (setq jkr '((4770. 256. 0)))
  (setq jks '((5026. 256. 0)))
  (setq jkt '((5282. 256. 0)))
  (setq jku '((5538. 256. 0)))
  (setq jkv '((5794. 256. 0)))
  (setq jkw '((6050. 256. 0)))
  (setq jkx '((6306. 256. 0)))
  (setq jky '((6562. 256. 0)))
  (setq jkz '((6818. 60. 0)))

  (make-gf-file 'jisjsy48 48. 40. 8. 12. jsy 1. T 0)
  (make-gf-file 'jisjroma48 48. 40. 8. 12. jroma 2. T 0)
  (make-gf-file 'jisjhira48 48. 40. 8. 12. jhira 3. T 0)
  (make-gf-file 'jisjkata48 48. 40. 8. 12. jkata 4. T 0)
  (make-gf-file 'jisjgreek48 48. 40. 8. 12. jgreek 5. T 0)
  (make-gf-file 'jisjrussian48 48. 40. 8. 12. jrussian 6. T 0)
  (make-gf-file 'jisjkeisen48 48. 40. 8. 12. jkeisen 7. T 0)
  (make-gf-file 'jisjka48 48. 40. 8. 12. jka 8. T 0)
  (make-gf-file 'jisjkb48 48. 40. 8. 12. jkb 9. T 0)
  (make-gf-file 'jisjkc48 48. 40. 8. 12. jkc 10. T 0)
  (make-gf-file 'jisjkd48 48. 40. 8. 12. jkd 11. T 0)
  (make-gf-file 'jisjke48 48. 40. 8. 12. jke 12. T 0)
  (make-gf-file 'jisjkf48 48. 40. 8. 12. jkf 13. T 0)
  (make-gf-file 'jisjkg48 48. 40. 8. 12. jkg 14. T 0)
  (make-gf-file 'jisjkh48 48. 40. 8. 12. jkh 15. T 0)
  (make-gf-file 'jisjki48 48. 40. 8. 12. jki 16. T 0)
  (make-gf-file 'jisjkj48 48. 40. 8. 12. jkj 17. T 0)
  (make-gf-file 'jisjkk48 48. 40. 8. 12. jkk 18. T 0)
  (make-gf-file 'jisjkl48 48. 40. 8. 12. jkl 19. T 0)
  (make-gf-file 'jisjkm48 48. 40. 8. 12. jkm 20. T 0)
  (make-gf-file 'jisjkn48 48. 40. 8. 12. jkn 21. T 0)
  (make-gf-file 'jisjko48 48. 40. 8. 12. jko 22. T 0)
  (make-gf-file 'jisjkp48 48. 40. 8. 12. jkp 23. T 0)
  (make-gf-file 'jisjkq48 48. 40. 8. 12. jkq 24. T 0)
  (make-gf-file 'jisjkr48 48. 40. 8. 12. jkr 25. T 0)
  (make-gf-file 'jisjks48 48. 40. 8. 12. jks 26. T 0)
  (make-gf-file 'jisjkt48 48. 40. 8. 12. jkt 27. T 0)
  (make-gf-file 'jisjku48 48. 40. 8. 12. jku 28. T 0)
  (make-gf-file 'jisjkv48 48. 40. 8. 12. jkv 29. T 0)
  (make-gf-file 'jisjkw48 48. 40. 8. 12. jkw 30. T 0)
  (make-gf-file 'jisjkx48 48. 40. 8. 12. jkx 31. T 0)
  (make-gf-file 'jisjky48 48. 40. 8. 12. jky 32. T 0)
  (make-gf-file 'jisjkz48 48. 40. 8. 12. jkz 33. T 0)
  (close infobj))

(defun make-jis-gf-tfm-72()
  (setq bitsize 72.)
  (setq infobj (open '|bs:<pattern>font24.dat| '(in fixnum)))
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.jisfont|))
;  (setq default-jtfm-dir `(bs ,'|pattern.jisfont|))
  (setq jsy `((,(jc 1 2) 93. ,(jc 1 2))(95. 14. 101.)(109. 8. 126.)
				       (117. 7. 142.)(124. 15. 160.)
				       (139. 8. 182.)(147. 1. 194.)))
  (setq jroma '((148. 10. 48.)(158. 26. 65.)(184. 26. 97.)))
  (setq jhira '((210. 83. 1)))
  (setq jkata '((293. 86. 1)))
  (setq jgreek '((379. 24. 1)(403. 24. 33.)))
  (setq jrussian '((427. 33. 1)(460. 33. 49.)))
  (setq jkeisen '((493. 32. 1)))
  (setq jka '((525. 256. 0)))
  (setq jkb '((781. 256. 0)))
  (setq jkc '((1037. 256. 0)))
  (setq jkd '((1293. 256. 0)))
  (setq jke '((1549. 256. 0)))
  (setq jkf '((1805. 256. 0)))
  (setq jkg '((2061. 256. 0)))
  (setq jkh '((2317. 256. 0)))
  (setq jki '((2573. 256. 0)))
  (setq jkj '((2829. 256. 0)))
  (setq jkk '((3085. 256. 0)))
  (setq jkl '((3341. 149. 0)))
  (setq jkm '((3490. 256. 0))) ; Daini suijun
  (setq jkn '((3746. 256. 0)))
  (setq jko '((4002. 256. 0)))
  (setq jkp '((4258. 256. 0)))
  (setq jkq '((4514. 256. 0)))
  (setq jkr '((4770. 256. 0)))
  (setq jks '((5026. 256. 0)))
  (setq jkt '((5282. 256. 0)))
  (setq jku '((5538. 256. 0)))
  (setq jkv '((5794. 256. 0)))
  (setq jkw '((6050. 256. 0)))
  (setq jkx '((6306. 256. 0)))
  (setq jky '((6562. 256. 0)))
  (setq jkz '((6818. 60. 0)))

  (make-gf-file 'jisjsy72 72. 60. 12. 20. jsy 1. T 0)
  (make-gf-file 'jisjroma72 72. 60. 12. 20. jroma 2. T 0)
  (make-gf-file 'jisjhira72 72. 60. 12. 20. jhira 3. T 0)
  (make-gf-file 'jisjkata72 72. 60. 12. 20. jkata 4. T 0)
  (make-gf-file 'jisjgreek72 72. 60. 12. 20. jgreek 5. T 0)
  (make-gf-file 'jisjrussian72 72. 60. 12. 20. jrussian 6. T 0)
  (make-gf-file 'jisjkeisen72 72. 60. 12. 20. jkeisen 7. T 0)
  (make-gf-file 'jisjka72 72. 60. 12. 20. jka 8. T 0)
  (make-gf-file 'jisjkb72 72. 60. 12. 20. jkb 9. T 0)
  (make-gf-file 'jisjkc72 72. 60. 12. 20. jkc 10. T 0)
  (make-gf-file 'jisjkd72 72. 60. 12. 20. jkd 11. T 0)
  (make-gf-file 'jisjke72 72. 60. 12. 20. jke 12. T 0)
  (make-gf-file 'jisjkf72 72. 60. 12. 20. jkf 13. T 0)
  (make-gf-file 'jisjkg72 72. 60. 12. 20. jkg 14. T 0)
  (make-gf-file 'jisjkh72 72. 60. 12. 20. jkh 15. T 0)
  (make-gf-file 'jisjki72 72. 60. 12. 20. jki 16. T 0)
  (make-gf-file 'jisjkj72 72. 60. 12. 20. jkj 17. T 0)
  (make-gf-file 'jisjkk72 72. 60. 12. 20. jkk 18. T 0)
  (make-gf-file 'jisjkl72 72. 60. 12. 20. jkl 19. T 0)
  (make-gf-file 'jisjkm72 72. 60. 12. 20. jkm 20. T 0)
  (make-gf-file 'jisjkn72 72. 60. 12. 20. jkn 21. T 0)
  (make-gf-file 'jisjko72 72. 60. 12. 20. jko 22. T 0)
  (make-gf-file 'jisjkp72 72. 60. 12. 20. jkp 23. T 0)
  (make-gf-file 'jisjkq72 72. 60. 12. 20. jkq 24. T 0)
  (make-gf-file 'jisjkr72 72. 60. 12. 20. jkr 25. T 0)
  (make-gf-file 'jisjks72 72. 60. 12. 20. jks 26. T 0)
  (make-gf-file 'jisjkt72 72. 60. 12. 20. jkt 27. T 0)
  (make-gf-file 'jisjku72 72. 60. 12. 20. jku 28. T 0)
  (make-gf-file 'jisjkv72 72. 60. 12. 20. jkv 29. T 0)
  (make-gf-file 'jisjkw72 72. 60. 12. 20. jkw 30. T 0)
  (make-gf-file 'jisjkx72 72. 60. 12. 20. jkx 31. T 0)
  (make-gf-file 'jisjky72 72. 60. 12. 20. jky 32. T 0)
  (make-gf-file 'jisjkz72 72. 60. 12. 20. jkz 33. T 0)
  (close infobj))

;; new function added on 1:12pm  Wednesday, 11 March 1987
;; to make etlfont available to JTeX
;; A modification to make-gf-tfm is made to skip null pattern in the input
;; file (new predicate null-pattern-p is introduced).

(defun new-make-etl-gf-tfm()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>etlfont-jis.dat2| '(in fixnum)))
  (clear-width-table)
  (setq default-jgf-dir `(bs ,'|tex.jgf|))
  (setq default-jtfm-dir `(bs ,'|tex.jtfm|))
  (setq jsy '((2 146. 2)))
  (setq jroma '((148. 10. 48.)(158. 26. 65.)(184. 26. 97.)))
  (setq jhira '((210. 83. 1)))
  (setq jkata '((293. 86. 1)))
  (setq jgreek '((379. 24. 1)(403. 24. 33.)))
; jrussian, jkeisen does not exist
  (setq jka '((525. 256. 0)))
  (setq jkb '((781. 256. 0)))
  (setq jkc '((1037. 256. 0)))
  (setq jkd '((1293. 256. 0)))
  (setq jke '((1549. 256. 0)))
  (setq jkf '((1805. 256. 0)))
  (setq jkg '((2061. 256. 0)))
  (setq jkh '((2317. 256. 0)))
  (setq jki '((2573. 256. 0)))
  (setq jkj '((2829. 256. 0)))
  (setq jkk '((3085. 256. 0)))
  (setq jkl '((3341. 149. 0)))

  (setq jkm '((3490. 256. 0)))
  (setq jkn '((3746. 256. 0)))
  (setq jko '((4002. 256. 0)))
  (setq jkp '((4258. 256. 0)))
  (setq jkq '((4514. 256. 0)))
  (setq jkr '((4770. 256. 0)))
  (setq jks '((5026. 256. 0)))
  (setq jkt '((5282. 256. 0)))
  (setq jku '((5538. 256. 0)))
  (setq jkv '((5794. 256. 0)))
  (setq jkw '((6050. 256. 0)))
  (setq jkx '((6306. 256. 0)))
  (setq jky '((6562. 256. 0)))
  (setq jkz '((6818. 60. 0)))
  
  (make-gf-file 'etljsy32 32. 27. 5. 9. jsy 1. T 0)
  (make-gf-file 'etljroma32 32. 27. 5. 9. jroma 2. T 0)
  (make-gf-file 'etljhira32 32. 27. 5. 9. jhira 3. T 0)
  (make-gf-file 'etljkata32 32. 27. 5. 9. jkata 4. T 0)
  (make-gf-file 'etljgreek32 32. 27. 5. 9. jgreek 5. T 0)

  (make-gf-file 'etljka32 32. 27. 5. 9. jka 8. T 0)
  (make-gf-file 'etljkb32 32. 27. 5. 9. jkb 9. T 0)
  (make-gf-file 'etljkc32 32. 27. 5. 9. jkc 10. T 0)
  (make-gf-file 'etljkd32 32. 27. 5. 9. jkd 11. T 0)
  (make-gf-file 'etljke32 32. 27. 5. 9. jke 12. T 0)
  (make-gf-file 'etljkf32 32. 27. 5. 9. jkf 13. T 0)
  (make-gf-file 'etljkg32 32. 27. 5. 9. jkg 14. T 0)
  (make-gf-file 'etljkh32 32. 27. 5. 9. jkh 15. T 0)
  (make-gf-file 'etljki32 32. 27. 5. 9. jki 16. T 0)
  (make-gf-file 'etljkj32 32. 27. 5. 9. jkj 17. T 0)
  (make-gf-file 'etljkk32 32. 27. 5. 9. jkk 18. T 0)
  (make-gf-file 'etljkl32 32. 27. 5. 9. jkl 19. T 0)
  (make-gf-file 'etljkm32 32. 27. 5. 9. jkm 20. T 0)
  (make-gf-file 'etljkn32 32. 27. 5. 9. jkn 21. T 0)
  (make-gf-file 'etljko32 32. 27. 5. 9. jko 22. T 0)
  (make-gf-file 'etljkp32 32. 27. 5. 9. jkp 23. T 0)
  (make-gf-file 'etljkq32 32. 27. 5. 9. jkq 24. T 0)
  (make-gf-file 'etljkr32 32. 27. 5. 9. jkr 25. T 0)
  (make-gf-file 'etljks32 32. 27. 5. 9. jks 26. T 0)
  (make-gf-file 'etljkt32 32. 27. 5. 9. jkt 27. T 0)
  (make-gf-file 'etljku32 32. 27. 5. 9. jku 28. T 0)
  (make-gf-file 'etljkv32 32. 27. 5. 9. jkv 29. T 0)
  (make-gf-file 'etljkw32 32. 27. 5. 9. jkw 30. T 0)
  (make-gf-file 'etljkx32 32. 27. 5. 9. jkx 31. T 0)
  (make-gf-file 'etljky32 32. 27. 5. 9. jky 32. T 0)
  (make-gf-file 'etljkz32 32. 27. 5. 9. jkz 33. T 0)

  (close infobj))

; New function added on 1:33pm  Monday, 6 April 1987
; to make GF and TFM files from dnp (dainippon printing) 36dot fonts.

(defun make-dnp-gf-tfm()
  (setq bitsize 36.)
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.dnp|))
;  (setq default-jtfm-dir `(bs ,'|pattern.dnp|))
; non-kanji (from jm01.rf)
; non existent symbol is omitted by make-gf-file
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq jroma '((189. 90. 33.)))
  (setq jhira '((283. 83. 1)))
  (setq jkata '((377. 86. 1)))
  (setq jgreek '((471. 56. 1)))
  (setq jrussian '((565. 81. 1)))
  (setq jkeisen '((659. 32. 1)))
; daiichi suijun (from jm1.rf)
  (setq jka '((1. 256. 0)))
  (setq jkb '((257. 256. 0)))
  (setq jkc '((513. 256. 0)))
  (setq jkd '((769. 256. 0)))
  (setq jke '((1025. 256. 0)))
  (setq jkf '((1281. 256. 0)))
  (setq jkg '((1537. 256. 0)))
  (setq jkh '((1793. 256. 0)))
  (setq jki '((2049. 256. 0)))
  (setq jkj '((2305. 256. 0)))
  (setq jkk '((2561. 256. 0)))
  (setq jkl '((2817. 149. 0)))
; daini suijun (from jm2.rf)
  (setq jkm '((1. 256. 0)))
  (setq jkn '((257. 256. 0)))
  (setq jko '((513. 256. 0)))
  (setq jkp '((769. 256. 0)))
  (setq jkq '((1025. 256. 0)))
  (setq jkr '((1281. 256. 0)))
  (setq jks '((1537. 256. 0)))
  (setq jkt '((1793. 256. 0)))
  (setq jku '((2049. 256. 0)))
  (setq jkv '((2305. 256. 0)))
  (setq jkw '((2561. 256. 0)))
  (setq jkx '((2817. 256. 0)))
  (setq jky '((3073. 256. 0)))
  (setq jkz '((3329. 60. 0)))
  
  (setq infobj (open '|bs:<pattern.dnp>jm01.rf| '(in fixnum)))
  (make-gf-file 'dnpjsy36 38. 30. 6. 10. jsy 1. T 0)
  (make-gf-file 'dnpjroma36 38. 30. 6. 10. jroma 2. T 0)
  (make-gf-file 'dnpjhira36 38. 30. 6. 10. jhira 3. T 0)
  (make-gf-file 'dnpjkata36 38. 30. 6. 10. jkata 4. T 0)
  (make-gf-file 'dnpjgreek36 38. 30. 6. 10. jgreek 5. T 0)
  (make-gf-file 'dnpjrussian36 38. 30. 6. 10. jrussian 6. T 0)
  (make-gf-file 'dnpjkeisen36 38. 30. 6. 10. jkeisen 7. T 0)
  (close infobj)
  (setq infobj (open '|bs:<pattern.dnp>jm1.rf| '(in fixnum)))
  (make-gf-file 'dnpjka36 38. 30. 6. 10. jka 8. T 0)
  (make-gf-file 'dnpjkb36 38. 30. 6. 10. jkb 9. T 0)
  (make-gf-file 'dnpjkc36 38. 30. 6. 10. jkc 10. T 0)
  (make-gf-file 'dnpjkd36 38. 30. 6. 10. jkd 11. T 0)
  (make-gf-file 'dnpjke36 38. 30. 6. 10. jke 12. T 0)
  (make-gf-file 'dnpjkf36 38. 30. 6. 10. jkf 13. T 0)
  (make-gf-file 'dnpjkg36 38. 30. 6. 10. jkg 14. T 0)
  (make-gf-file 'dnpjkh36 38. 30. 6. 10. jkh 15. T 0)
  (make-gf-file 'dnpjki36 38. 30. 6. 10. jki 16. T 0)
  (make-gf-file 'dnpjkj36 38. 30. 6. 10. jkj 17. T 0)
  (make-gf-file 'dnpjkk36 38. 30. 6. 10. jkk 18. T 0)
  (make-gf-file 'dnpjkl36 38. 30. 6. 10. jkl 19. T 0)
  (close infobj)
  (setq infobj (open '|bs:<pattern.dnp>jm2.rf| '(in fixnum)))
  (make-gf-file 'dnpjkm36 38. 30. 6. 10. jkm 20. T 0)
  (make-gf-file 'dnpjkn36 38. 30. 6. 10. jkn 21. T 0)
  (make-gf-file 'dnpjko36 38. 30. 6. 10. jko 22. T 0)
  (make-gf-file 'dnpjkp36 38. 30. 6. 10. jkp 23. T 0)
  (make-gf-file 'dnpjkq36 38. 30. 6. 10. jkq 24. T 0)
  (make-gf-file 'dnpjkr36 38. 30. 6. 10. jkr 25. T 0)
  (make-gf-file 'dnpjks36 38. 30. 6. 10. jks 26. T 0)
  (make-gf-file 'dnpjkt36 38. 30. 6. 10. jkt 27. T 0)
  (make-gf-file 'dnpjku36 38. 30. 6. 10. jku 28. T 0)
  (make-gf-file 'dnpjkv36 38. 30. 6. 10. jkv 29. T 0)
  (make-gf-file 'dnpjkw36 38. 30. 6. 10. jkw 30. T 0)
  (make-gf-file 'dnpjkx36 38. 30. 6. 10. jkx 31. T 0)
  (make-gf-file 'dnpjky36 38. 30. 6. 10. jky 32. T 0)
  (make-gf-file 'dnpjkz36 38. 30. 6. 10. jkz 33. T 0)
  (close infobj))

(defun test-make-dnp-gf-tfm()
  (setq bitsize 36.)
  (clear-width-table)
  (setq default-jgf-dir `(as ,'|yaski|))
  (setq default-jtfm-dir `(as ,'|yaski|))
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq infobj (open '|bs:<pattern.dnp>jm01.rf| '(in fixnum)))
  (make-gf-file 'dnpjsy36 38. 30. 6. 10. jsy 1. T 0)
  (close infobj))

(defun test-make-dnp-38-gf-tfm()
  (setq bitsize 38.)
  (clear-width-table)
  (setq default-jgf-dir `(bs ,'|pattern.dnp|))
  (setq default-jtfm-dir `(bs ,'|pattern.dnp|))
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq infobj (open '|bs:<pattern.dnp>jm1.rf38| '(in fixnum)))
  (make-gf-file 'dnpjsy38 38. 32. 6. 10. jsy 1. T 0)
  (close infobj))

(defun add-jsy-etl-gf-tfm()
  (setq bitsize 32.)
  (setq infobj (open '|bs:<pattern>etlfont-jis.dat| '(in fixnum)))
  (clear-width-table)
  (setq default-jgf-dir `(bs ,'|tex.jgf|))
  (setq default-jtfm-dir `(bs ,'|tex.jtfm|))
  (setq jsy '((2 93. 2)(95. 53. 101.)))
  (make-gf-file 'etljsy32 34. 27. 5. 9. jsy 1. T 0))

(defun make-dnp-38-gf-tfm()
  (setq bitsize 38.)
  (clear-width-table)
;  (setq default-jgf-dir `(bs ,'|pattern.dnp|))
;  (setq default-jtfm-dir `(bs ,'|pattern.dnp|))
; non-kanji (from jm1.rf38)
; non existent symbol is omitted by make-gf-file
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq jroma '((189. 90. 33.)))
  (setq jhira '((283. 83. 1)))
  (setq jkata '((377. 86. 1)))
  (setq jgreek '((471. 56. 1)))
  (setq jrussian '((565. 81. 1)))
  (setq jkeisen '((659. 32. 1)))
; daiichi suijun (from jm1.rf38)
  (setq jka `((,(+ 1410. 1.) 256. 0)))
  (setq jkb `((,(+ 1410. 257.) 256. 0)))
  (setq jkc `((,(+ 1410. 513.) 256. 0)))
  (setq jkd `((,(+ 1410. 769.) 256. 0)))
  (setq jke `((,(+ 1410. 1025.) 256. 0)))
  (setq jkf `((,(+ 1410. 1281.) 256. 0)))
  (setq jkg `((,(+ 1410. 1537.) 256. 0)))
  (setq jkh `((,(+ 1410. 1793.) 256. 0)))
  (setq jki `((,(+ 1410. 2049.) 256. 0)))
  (setq jkj `((,(+ 1410. 2305.) 256. 0)))
  (setq jkk `((,(+ 1410. 2561.) 256. 0)))
  (setq jkl `((,(+ 1410. 2817.) 149. 0)))
; daini suijun (from jm2.rf38)
  (setq jkm '((1. 256. 0)))
  (setq jkn '((257. 256. 0)))
  (setq jko '((513. 256. 0)))
  (setq jkp '((769. 256. 0)))
  (setq jkq '((1025. 256. 0)))
  (setq jkr '((1281. 256. 0)))
  (setq jks '((1537. 256. 0)))
  (setq jkt '((1793. 256. 0)))
  (setq jku '((2049. 256. 0)))
  (setq jkv '((2305. 256. 0)))
  (setq jkw '((2561. 256. 0)))
  (setq jkx '((2817. 256. 0)))
  (setq jky '((3073. 256. 0)))
  (setq jkz '((3329. 60. 0)))
  
  (setq infobj (open '|bs:<pattern.dnp>jm1.rf38| '(in fixnum)))
  (make-gf-file 'dnpjsy38 38. 32. 6. 10. jsy 1. T 0)
  (make-gf-file 'dnpjroma38 38. 32. 6. 10. jroma 2. T 0)
  (make-gf-file 'dnpjhira38 38. 32. 6. 10. jhira 3. T 0)
  (make-gf-file 'dnpjkata38 38. 32. 6. 10. jkata 4. T 0)
  (make-gf-file 'dnpjgreek38 38. 32. 6. 10. jgreek 5. T 0)
  (make-gf-file 'dnpjrussian38 38. 32. 6. 10. jrussian 6. T 0)
  (make-gf-file 'dnpjkeisen38 38. 32. 6. 10. jkeisen 7. T 0)
  (make-gf-file 'dnpjka38 38. 32. 6. 10. jka 8. T 0)
  (make-gf-file 'dnpjkb38 38. 32. 6. 10. jkb 9. T 0)
  (make-gf-file 'dnpjkc38 38. 32. 6. 10. jkc 10. T 0)
  (make-gf-file 'dnpjkd38 38. 32. 6. 10. jkd 11. T 0)
  (make-gf-file 'dnpjke38 38. 32. 6. 10. jke 12. T 0)
  (make-gf-file 'dnpjkf38 38. 32. 6. 10. jkf 13. T 0)
  (make-gf-file 'dnpjkg38 38. 32. 6. 10. jkg 14. T 0)
  (make-gf-file 'dnpjkh38 38. 32. 6. 10. jkh 15. T 0)
  (make-gf-file 'dnpjki38 38. 32. 6. 10. jki 16. T 0)
  (make-gf-file 'dnpjkj38 38. 32. 6. 10. jkj 17. T 0)
  (make-gf-file 'dnpjkk38 38. 32. 6. 10. jkk 18. T 0)
  (make-gf-file 'dnpjkl38 38. 32. 6. 10. jkl 19. T 0)
  (close infobj)
  (setq infobj (open '|bs:<pattern.dnp>jm2.rf38| '(in fixnum)))
  (make-gf-file 'dnpjkm38 38. 32. 6. 10. jkm 20. T 0)
  (make-gf-file 'dnpjkn38 38. 32. 6. 10. jkn 21. T 0)
  (make-gf-file 'dnpjko38 38. 32. 6. 10. jko 22. T 0)
  (make-gf-file 'dnpjkp38 38. 32. 6. 10. jkp 23. T 0)
  (make-gf-file 'dnpjkq38 38. 32. 6. 10. jkq 24. T 0)
  (make-gf-file 'dnpjkr38 38. 32. 6. 10. jkr 25. T 0)
  (make-gf-file 'dnpjks38 38. 32. 6. 10. jks 26. T 0)
  (make-gf-file 'dnpjkt38 38. 32. 6. 10. jkt 27. T 0)
  (make-gf-file 'dnpjku38 38. 32. 6. 10. jku 28. T 0)
  (make-gf-file 'dnpjkv38 38. 32. 6. 10. jkv 29. T 0)
  (make-gf-file 'dnpjkw38 38. 32. 6. 10. jkw 30. T 0)
  (make-gf-file 'dnpjkx38 38. 32. 6. 10. jkx 31. T 0)
  (make-gf-file 'dnpjky38 38. 32. 6. 10. jky 32. T 0)
  (make-gf-file 'dnpjkz38 38. 32. 6. 10. jkz 33. T 0)
  (close infobj))

;; Following code is added by yaski on 14-July-87
;; to recreate all Japanese fonts after a slight change in
;; make-tfm-file.  This change adds new header info into TFM file which
;; indicates that it is a japanese font file.

(defun recreate-all-fonts()
  (setq default-jgf-dir `(bs ,`|pattern.jgf|))
  (setq default-jtfm-dir `(bs ,`|pattern.jtfm|))
;To make three kind of gf fonts
  (make-all-gf-tfm-32) ; for g1*32.300gf
  (make-all-gf-tfm-32-2) ; for g2*32.300gf
  (make-all-gf-tfm-32-3) ; for g3*32.300gf
;To make four JIS fonts
  (make-jis-gf-tfm-24)
  (setq byte-count-for-48 (* 6 48.)) ; This is used in read-into-array-48
  (make-jis-gf-tfm-36) ; because JIS 36 dots font is stored in 48 by 48 array
  (make-jis-gf-tfm-48)
  (make-jis-gf-tfm-72)
;To make ETL fonts
  (new-make-etl-gf-tfm)
;To make DNP fonts
  (setq byte-count-for-48 (* 6 36.))
  (make-dnp-gf-tfm)
  (setq byte-count-for-48 (* 6 38.))
  (make-dnp-38-gf-tfm))

(defun recreate-dnp-36()
  (setq default-jgf-dir `(bs ,`|pattern.jgf|))
  (setq default-jtfm-dir `(bs ,`|pattern.jtfm|))
  (setq byte-count-for-48 (* 6 36.))
  (make-dnp-gf-tfm))

;; Following routines are added to make new fonts from DNP
;; on 4:18am  Friday, 24 July 1987

(declare(special i-file-dir))

(defun fname(head tail)
  (cond((> tail 99.)
	(implode(append(reverse(cdr(reverse(explode head))))
		       (explode tail))))
       (t(implode(append(explode head)(explode tail))))))

(defun make-dm-gf-tfm(dotsize extension height depth d-size make-tfm-p mag)
  (setq *nopoint t)
  (setq bitsize dotsize)
  (setq jgf-extention extension)
  (clear-width-table)
; non-kanji (from jm00*.rf)
; non existent symbol is omitted by make-gf-file
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq jroma '((189. 90. 33.)))
  (setq jhira '((283. 83. 1)))
  (setq jkata '((377. 86. 1)))
  (setq jgreek '((471. 56. 1)))
  (setq jrussian '((565. 81. 1)))
  (setq jkeisen '((659. 32. 1)))
; daiichi suijun (from jm10*.rf)
  (setq jka '((1. 256. 0)))
  (setq jkb '((257. 256. 0)))
  (setq jkc '((513. 256. 0)))
  (setq jkd '((769. 256. 0)))
  (setq jke '((1025. 256. 0)))
  (setq jkf '((1281. 256. 0)))
  (setq jkg '((1537. 256. 0)))
  (setq jkh '((1793. 256. 0)))
  (setq jki '((2049. 256. 0)))
  (setq jkj '((2305. 256. 0)))
  (setq jkk '((2561. 256. 0)))
  (setq jkl '((2817. 149. 0)))
; daini suijun (from jm20*.rf)
  (setq jkm '((1. 256. 0)))
  (setq jkn '((257. 256. 0)))
  (setq jko '((513. 256. 0)))
  (setq jkp '((769. 256. 0)))
  (setq jkq '((1025. 256. 0)))
  (setq jkr '((1281. 256. 0)))
  (setq jks '((1537. 256. 0)))
  (setq jkt '((1793. 256. 0)))
  (setq jku '((2049. 256. 0)))
  (setq jkv '((2305. 256. 0)))
  (setq jkw '((2561. 256. 0)))
  (setq jkx '((2817. 256. 0)))
  (setq jky '((3073. 256. 0)))
  (setq jkz '((3329. 60. 0)))
  
  (setq infobj (open (list i-file-dir (fname 'jm00 dotsize) 'RF) '(in fixnum)))
  (make-gf-file (fname 'dmjsy d-size)
		dotsize height depth d-size jsy 1. make-tfm-p mag)
  (make-gf-file (fname 'dmjroma d-size)
		dotsize height depth d-size jroma 2. make-tfm-p mag)
  (make-gf-file (fname 'dmjhira d-size)
		dotsize height depth d-size jhira 3. make-tfm-p mag)
  (make-gf-file (fname 'dmjkata d-size)
		dotsize height depth d-size jkata 4. make-tfm-p mag)
  (make-gf-file (fname 'dmjgreek d-size)
		dotsize height depth d-size jgreek 5. make-tfm-p mag)
  (make-gf-file (fname 'dmjrussian d-size)
		dotsize height depth d-size jrussian 6. make-tfm-p mag)
  (make-gf-file (fname 'dmjkeisen d-size)
		dotsize height depth d-size jkeisen 7. make-tfm-p mag)
  (close infobj)
  (setq infobj (open (list i-file-dir (fname 'jm10 dotsize) 'RF) '(in fixnum)))
  (make-gf-file (fname 'dmjka d-size)
		dotsize height depth d-size jka 8. make-tfm-p mag)
  (make-gf-file (fname 'dmjkb d-size)
		dotsize height depth d-size jkb 9. make-tfm-p mag)
  (make-gf-file (fname 'dmjkc d-size)
		dotsize height depth d-size jkc 10. make-tfm-p mag)
  (make-gf-file (fname 'dmjkd d-size)
		dotsize height depth d-size jkd 11. make-tfm-p mag)
  (make-gf-file (fname 'dmjke d-size)
		dotsize height depth d-size jke 12. make-tfm-p mag)
  (make-gf-file (fname 'dmjkf d-size)
		dotsize height depth d-size jkf 13. make-tfm-p mag)
  (make-gf-file (fname 'dmjkg d-size)
		dotsize height depth d-size jkg 14. make-tfm-p mag)
  (make-gf-file (fname 'dmjkh d-size)
		dotsize height depth d-size jkh 15. make-tfm-p mag)
  (make-gf-file (fname 'dmjki d-size)
		dotsize height depth d-size jki 16. make-tfm-p mag)
  (make-gf-file (fname 'dmjkj d-size)
		dotsize height depth d-size jkj 17. make-tfm-p mag)
  (make-gf-file (fname 'dmjkk d-size)
		dotsize height depth d-size jkk 18. make-tfm-p mag)
  (make-gf-file (fname 'dmjkl d-size)
		dotsize height depth d-size jkl 19. make-tfm-p mag)
  (close infobj)
  (setq infobj (open (list i-file-dir (fname 'jm20 dotsize) 'RF) '(in fixnum)))
  (make-gf-file (fname 'dmjkm d-size)
		dotsize height depth d-size jkm 20. make-tfm-p mag)
  (make-gf-file (fname 'dmjkn d-size)
		dotsize height depth d-size jkn 21. make-tfm-p mag)
  (make-gf-file (fname 'dmjko d-size)
		dotsize height depth d-size jko 22. make-tfm-p mag)
  (make-gf-file (fname 'dmjkp d-size)
		dotsize height depth d-size jkp 23. make-tfm-p mag)
  (make-gf-file (fname 'dmjkq d-size)
		dotsize height depth d-size jkq 24. make-tfm-p mag)
  (make-gf-file (fname 'dmjkr d-size)
		dotsize height depth d-size jkr 25. make-tfm-p mag)
  (make-gf-file (fname 'dmjks d-size)
		dotsize height depth d-size jks 26. make-tfm-p mag)
  (make-gf-file (fname 'dmjkt d-size)
		dotsize height depth d-size jkt 27. make-tfm-p mag)
  (make-gf-file (fname 'dmjku d-size)
		dotsize height depth d-size jku 28. make-tfm-p mag)
  (make-gf-file (fname 'dmjkv d-size)
		dotsize height depth d-size jkv 29. make-tfm-p mag)
  (make-gf-file (fname 'dmjkw d-size)
		dotsize height depth d-size jkw 30. make-tfm-p mag)
  (make-gf-file (fname 'dmjkx d-size)
		dotsize height depth d-size jkx 31. make-tfm-p mag)
  (make-gf-file (fname 'dmjky d-size)
		dotsize height depth d-size jky 32. make-tfm-p mag)
  (make-gf-file (fname 'dmjkz d-size)
		dotsize height depth d-size jkz 33. make-tfm-p mag)
  (close infobj))

(defun make-dg-gf-tfm(dotsize extension height depth d-size make-tfm-p mag)
  (setq *nopoint t)
  (setq bitsize dotsize)
  (setq jgf-extention extension)
  (clear-width-table)
; non-kanji (from jm00*.rf)
; non existent symbol is omitted by make-gf-file
  (setq jsy '((2 93. 2)(95. 94. 101.)))
  (setq jroma '((189. 90. 33.)))
  (setq jhira '((283. 83. 1)))
  (setq jkata '((377. 86. 1)))
  (setq jgreek '((471. 56. 1)))
  (setq jrussian '((565. 81. 1)))
  (setq jkeisen '((659. 32. 1)))
; daiichi suijun (from jm10*.rf)
  (setq jka '((1. 256. 0)))
  (setq jkb '((257. 256. 0)))
  (setq jkc '((513. 256. 0)))
  (setq jkd '((769. 256. 0)))
  (setq jke '((1025. 256. 0)))
  (setq jkf '((1281. 256. 0)))
  (setq jkg '((1537. 256. 0)))
  (setq jkh '((1793. 256. 0)))
  (setq jki '((2049. 256. 0)))
  (setq jkj '((2305. 256. 0)))
  (setq jkk '((2561. 256. 0)))
  (setq jkl '((2817. 149. 0)))
; daini suijun (from jm20*.rf)
  (setq jkm '((1. 256. 0)))
  (setq jkn '((257. 256. 0)))
  (setq jko '((513. 256. 0)))
  (setq jkp '((769. 256. 0)))
  (setq jkq '((1025. 256. 0)))
  (setq jkr '((1281. 256. 0)))
  (setq jks '((1537. 256. 0)))
  (setq jkt '((1793. 256. 0)))
  (setq jku '((2049. 256. 0)))
  (setq jkv '((2305. 256. 0)))
  (setq jkw '((2561. 256. 0)))
  (setq jkx '((2817. 256. 0)))
  (setq jky '((3073. 256. 0)))
  (setq jkz '((3329. 60. 0)))
  
  (setq infobj (open (list i-file-dir (fname 'jg00 dotsize) 'RF) '(in fixnum)))
  (make-gf-file (fname 'dgjsy d-size)
		dotsize height depth d-size jsy 1. make-tfm-p mag)
  (make-gf-file (fname 'dgjroma d-size)
		dotsize height depth d-size jroma 2. make-tfm-p mag)
  (make-gf-file (fname 'dgjhira d-size)
		dotsize height depth d-size jhira 3. make-tfm-p mag)
  (make-gf-file (fname 'dgjkata d-size)
		dotsize height depth d-size jkata 4. make-tfm-p mag)
  (make-gf-file (fname 'dgjgreek d-size)
		dotsize height depth d-size jgreek 5. make-tfm-p mag)
  (make-gf-file (fname 'dgjrussian d-size)
		dotsize height depth d-size jrussian 6. make-tfm-p mag)
  (make-gf-file (fname 'dgjkeisen d-size)
		dotsize height depth d-size jkeisen 7. make-tfm-p mag)
  (close infobj)
;  (setq infobj (open (list i-file-dir (fname 'jg10 dotsize) 'RF) '(in fixnum)))
;  (make-gf-file (fname 'dgjka d-size)
;		dotsize height depth d-size jka 8. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkb d-size)
;		dotsize height depth d-size jkb 9. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkc d-size)
;		dotsize height depth d-size jkc 10. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkd d-size)
;		dotsize height depth d-size jkd 11. make-tfm-p mag)
;  (make-gf-file (fname 'dgjke d-size)
;		dotsize height depth d-size jke 12. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkf d-size)
;		dotsize height depth d-size jkf 13. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkg d-size)
;		dotsize height depth d-size jkg 14. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkh d-size)
;		dotsize height depth d-size jkh 15. make-tfm-p mag)
;  (make-gf-file (fname 'dgjki d-size)
;		dotsize height depth d-size jki 16. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkj d-size)
;		dotsize height depth d-size jkj 17. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkk d-size)
;		dotsize height depth d-size jkk 18. make-tfm-p mag)
;  (make-gf-file (fname 'dgjkl d-size)
;		dotsize height depth d-size jkl 19. make-tfm-p mag)
;  (close infobj)
  (setq infobj (open (list i-file-dir (fname 'jg20 dotsize) 'RF) '(in fixnum)))
  (make-gf-file (fname 'dgjkm d-size)
		dotsize height depth d-size jkm 20. make-tfm-p mag)
  (make-gf-file (fname 'dgjkn d-size)
		dotsize height depth d-size jkn 21. make-tfm-p mag)
  (make-gf-file (fname 'dgjko d-size)
		dotsize height depth d-size jko 22. make-tfm-p mag)
  (make-gf-file (fname 'dgjkp d-size)
		dotsize height depth d-size jkp 23. make-tfm-p mag)
  (make-gf-file (fname 'dgjkq d-size)
		dotsize height depth d-size jkq 24. make-tfm-p mag)
  (make-gf-file (fname 'dgjkr d-size)
		dotsize height depth d-size jkr 25. make-tfm-p mag)
  (make-gf-file (fname 'dgjks d-size)
		dotsize height depth d-size jks 26. make-tfm-p mag)
  (make-gf-file (fname 'dgjkt d-size)
		dotsize height depth d-size jkt 27. make-tfm-p mag)
  (make-gf-file (fname 'dgjku d-size)
		dotsize height depth d-size jku 28. make-tfm-p mag)
  (make-gf-file (fname 'dgjkv d-size)
		dotsize height depth d-size jkv 29. make-tfm-p mag)
  (make-gf-file (fname 'dgjkw d-size)
		dotsize height depth d-size jkw 30. make-tfm-p mag)
  (make-gf-file (fname 'dgjkx d-size)
		dotsize height depth d-size jkx 31. make-tfm-p mag)
  (make-gf-file (fname 'dgjky d-size)
		dotsize height depth d-size jky 32. make-tfm-p mag)
  (make-gf-file (fname 'dgjkz d-size)
		dotsize height depth d-size jkz 33. make-tfm-p mag)
  (close infobj))

(defun make-all-dm() ; make all mincho font files
  (setq dnp-making-mode t)
  (setq default-jgf-dir `(cs ,`|dnp.mincho.gf1|))
  (setq default-jtfm-dir `(cs ,`|dnp.mincho.tfm|))
  (setq i-file-dir '(ds mincho))
;  (make-dm-gf-tfm 27. '300gf 23. 4. 7. t 0) ; also dm*6.360gf, dm*5.432gf
;  (make-dm-gf-tfm 30. '300gf 24. 6. 8. t 0)
;  (make-dm-gf-tfm 34. '300gf 28. 6. 9. t 0)
;  (make-dm-gf-tfm 38. '300gf 32. 6. 10. t 0) ; also dm*7.432gf
;  (make-dm-gf-tfm 46. '300gf 38. 8. 12. t 0) ; also dm*10.360gf, dm*7.518gf
  (make-dm-gf-tfm 66. '300gf 55. 11. 17. t 0)
                              ; also dm*10.518gf, dm*12.432gf, dm*7.746gf
  (setq default-jgf-dir `(cs ,`|dnp.mincho.gf2|))
  (make-dm-gf-tfm 29. '329gf 24. 5. 7. () 0.5)
  (make-dm-gf-tfm 42. '329gf 35. 7. 10. () 0.5)
  (make-dm-gf-tfm 50. '329gf 42. 8. 12. () 0.5)
  (make-dm-gf-tfm 55. '432gf 46. 9. 10. () 2) ; also dm*12.360gf, dm*7.622gf
  (make-dm-gf-tfm 71. '329gf 59. 12. 17. () 0.5) ; also dm*9.622gf
  (make-dm-gf-tfm 79. '622gf 66. 13. 10. () 4) ; also dm*12.518gf
  (make-dm-gf-tfm 95. '746gf 79. 16. 10. () 5) ; also dm*12.622gf
)

(defun make-all-dg() ; make all gothic font files
  (setq dnp-making-mode t)
  (setq default-jgf-dir `(cs ,`|dnp.gothic.gf1|))
  (setq default-jtfm-dir `(cs ,`|dnp.gothic.tfm|))
  (setq i-file-dir '(js gothic))
  (make-dg-gf-tfm 27. '300gf 23. 4. 7. t 0) ; also dg*6.360gf, dg*5.432gf
  (make-dg-gf-tfm 30. '300gf 24. 6. 8. t 0)
  (make-dg-gf-tfm 34. '300gf 28. 6. 9. t 0)
  (make-dg-gf-tfm 38. '300gf 32. 6. 10. t 0) ; also dg*7.432gf
  (make-dg-gf-tfm 46. '300gf 38. 8. 12. t 0) ; also dg*10.360gf, dg*7.518gf
  (make-dg-gf-tfm 66. '300gf 55. 11. 17. t 0)
                              ; also dg*10.300gf, dg*12.432gf, dg*7.746gf
  (setq default-jgf-dir `(cs ,`|dnp.gothic.gf2|))
  (make-dg-gf-tfm 29. '329gf 24. 5. 7. () 0.5)
  (make-dg-gf-tfm 42. '329gf 35. 7. 10. () 0.5)
  (make-dg-gf-tfm 50. '329gf 42. 8. 12. () 0.5)
  (make-dg-gf-tfm 55. '432gf 46. 9. 10. () 2) ; also dg*12.360gf, dg*7.622gf
  (make-dg-gf-tfm 71. '329gf 59. 12. 17. () 0.5) ; also dg*9.622gf
  (make-dg-gf-tfm 79. '622gf 66. 13. 10. () 4) ; also dg*12.518gf
  (make-dg-gf-tfm 95. '746gf 79. 16. 10. () 5) ; also dg*12.622gf
)

(defun test-dg() ; make all mincho font files
  (setq dnp-making-mode t)
  (setq default-jgf-dir `(bs ,`|dnp.gf|))
  (setq default-jtfm-dir `(bs ,`|dnp.tfm|))
  (setq i-file-dir '(js gothic))
  (make-dg-gf-tfm 38. '300gf 32. 6. 10. t 0))

(defun make-all()
  (make-all-dg)
  (make-all-dm))

(defun make-dm-again() ; make all mincho font files
  (setq dnp-making-mode t)
  (setq default-jgf-dir `(cs ,`|dnp.mincho.gf1|))
  (setq default-jtfm-dir `(cs ,`|dnp.mincho.tfm|))
  (setq i-file-dir '(ds mincho))
;  (make-dm-gf-tfm 27. '300gf 23. 4. 7. t 0) ; also dm*6.360gf, dm*5.432gf
;  (make-dm-gf-tfm 30. '300gf 24. 6. 8. t 0)
;  (make-dm-gf-tfm 34. '300gf 28. 6. 9. t 0)
;  (make-dm-gf-tfm 38. '300gf 32. 6. 10. t 0) ; also dm*7.432gf
;  (make-dm-gf-tfm 46. '300gf 38. 8. 12. t 0) ; also dm*10.360gf, dm*7.518gf
;  (make-dm-gf-tfm 66. '300gf 55. 11. 17. t 0)
                              ; also dm*10.518gf, dm*12.432gf, dm*7.746gf
  (setq default-jgf-dir `(cs ,`|dnp.mincho.gf2|))
  (make-dm-gf-tfm 29. '329gf 24. 5. 7. () 0.5)
  (make-dm-gf-tfm 42. '329gf 35. 7. 10. () 0.5)
  (make-dm-gf-tfm 50. '329gf 42. 8. 12. () 0.5)
  (make-dm-gf-tfm 55. '432gf 46. 9. 10. () 2) ; also dm*12.360gf, dm*7.622gf
  (make-dm-gf-tfm 71. '329gf 59. 12. 17. () 0.5) ; also dm*9.622gf
  (make-dm-gf-tfm 79. '622gf 66. 13. 10. () 4) ; also dm*12.518gf
  (make-dm-gf-tfm 95. '746gf 79. 16. 10. () 5) ; also dm*12.622gf
)

(defun make-dg-again() ; make all mincho font files
  (setq dnp-making-mode t)
  (setq default-jgf-dir `(cs ,`|dnp.gothic.gf1|))
  (setq default-jtfm-dir `(cs ,`|dnp.gothic.tfm|))
  (setq i-file-dir '(js gothic))
;  (make-dg-gf-tfm 27. '300gf 23. 4. 7. t 0) ; also dg*6.360gf, dg*5.432gf
;  (make-dg-gf-tfm 30. '300gf 24. 6. 8. t 0)
;  (make-dg-gf-tfm 34. '300gf 28. 6. 9. t 0)
;  (make-dg-gf-tfm 38. '300gf 32. 6. 10. t 0) ; also dg*7.432gf
;  (make-dg-gf-tfm 46. '300gf 38. 8. 12. t 0) ; also dg*10.360gf, dg*7.518gf
;  (make-dg-gf-tfm 66. '300gf 55. 11. 17. t 0)
                              ; also dg*10.300gf, dg*12.432gf, dg*7.746gf
  (setq default-jgf-dir `(cs ,`|dnp.gothic.gf2|))
  (make-dg-gf-tfm 29. '329gf 24. 5. 7. () 0.5)
  (make-dg-gf-tfm 42. '329gf 35. 7. 10. () 0.5)
  (make-dg-gf-tfm 50. '329gf 42. 8. 12. () 0.5)
  (make-dg-gf-tfm 55. '432gf 46. 9. 10. () 2) ; also dg*12.360gf, dg*7.622gf
  (make-dg-gf-tfm 71. '329gf 59. 12. 17. () 0.5) ; also dg*9.622gf
  (make-dg-gf-tfm 79. '622gf 66. 13. 10. () 4) ; also dg*12.518gf
  (make-dg-gf-tfm 95. '746gf 79. 16. 10. () 5) ; also dg*12.622gf
)

(defun make-again()(make-dm-again)(make-dg-again))

; code added on 5:33pm  Monday, 17 August 1987
; to recreate JIS and ETL fonts with proper width (previous version used
; dotsize + 2dots as width, but now they should have exact dotsize as width

(defun recreate-jis-and-etl()
  (setq default-jgf-dir `(bs ,`|tex.jgf|))
  (setq default-jtfm-dir `(bs ,`|tex.jtfm|))
;To make three kind of gf fonts
;  (make-all-gf-tfm-32) ; for g1*32.300gf
;  (make-all-gf-tfm-32-2) ; for g2*32.300gf
;  (make-all-gf-tfm-32-3) ; for g3*32.300gf
;To make four JIS fonts
  (make-jis-gf-tfm-24)
  (setq byte-count-for-48 (* 6 48.)) ; This is used in read-into-array-48
  (make-jis-gf-tfm-36) ; because JIS 36 dots font is stored in 48 by 48 array
  (make-jis-gf-tfm-48)
  (make-jis-gf-tfm-72)
;To make ETL fonts
  (new-make-etl-gf-tfm)
;To make DNP fonts
;  (setq byte-count-for-48 (* 6 36.))
;  (make-dnp-gf-tfm)
;  (setq byte-count-for-48 (* 6 38.))
;  (make-dnp-38-gf-tfm)
)

; Following code added on 9:42am  Saturday, 7 November 1987
; to make additional dnp fonts for 300dpi printers

(defun make-dm(dotsize ptsize mag)
  (let*((fname-ext (cond((equal mag 0)'300gf)
		       ((equal mag 0.5)'329gf)
		       ((equal mag 1)'360gf)
		       ((equal mag 2)'432gf)
		       ((equal mag 3)'518gf)
		       ((equal mag 4)'622gf)
		       ((equal mag 5)'746gf)
		       ((equal mag 6)'896gf)
		       ((equal mag 7)'1075gf)))
       (height (ifix(+$ 0.5 (//$(*$ (float dotsize) 5.0)6.0))))
       (depth (- dotsize height))
       (making-tfm (cond((equal mag 0)t)(t ()))))
    (make-dm-gf-tfm dotsize fname-ext height depth ptsize making-tfm mag)))

(defun make-dg(dotsize ptsize mag)
  (let*((fname-ext (cond((equal mag 0)'300gf)
		       ((equal mag 0.5)'329gf)
		       ((equal mag 1)'360gf)
		       ((equal mag 2)'432gf)
		       ((equal mag 3)'518gf)
		       ((equal mag 4)'622gf)
		       ((equal mag 5)'746gf)
		       ((equal mag 6)'896gf)
		       ((equal mag 7)'1075gf)))
       (height (ifix(+$ 0.5 (//$(*$ (float dotsize) 5.0)6.0))))
       (depth (- dotsize height))
       (making-tfm (cond((equal mag 0)t)(t ()))))
    (make-dg-gf-tfm dotsize fname-ext height depth ptsize making-tfm mag)))

(defun make-additional-dm() ; make additional dnp mincho font files
  (setq dnp-making-mode t)
;  (setq default-jgf-dir `(cs ,`|dnp.mincho.gf1|))
;  (setq default-jtfm-dir `(cs ,`|dnp.mincho.tfm|))
  (setq default-jgf-dir '(ds mincho))
  (setq default-jtfm-dir '(ds mincho))
  (setq i-file-dir '(ds mincho))
;  (make-dm 19. 5. 0)
;  (make-dm 23. 6. 0)
;  (make-dm 32. 7. 1)
;  (make-dm 72. 17. 0.5)
;  (make-dm 76. 20. 0)
  (make-dm 113. 10. 6)
  (make-dm 136. 10. 7)
)

(defun make-additional-dg() ; make additional dnp gothic font files
  (setq dnp-making-mode t)
;  (setq default-jgf-dir `(cs ,`|dnp.gothic.gf1|))
;  (setq default-jtfm-dir `(cs ,`|dnp.gothic.tfm|))
  (setq default-jgf-dir '(bs gothic))
  (setq default-jtfm-dir '(bs gothic))
  (setq i-file-dir '(js gothic))
;  (make-dg 19. 5. 0)
;  (make-dg 23. 6. 0)
;  (make-dg 32. 7. 1)
;  (make-dg 72. 17. 0.5)
;  (make-dg 76. 20. 0)
;  (make-dg 113. 10. 6)
  (make-dg 136. 10. 7)
)

(defun make-additional()
  (make-additional-dm)
  (make-additional-dg))

(defun make-newmag-for-ten()
  (setq dnp-making-mode t)
;  (setq default-jgf-dir '(ds mincho))
;  (setq default-jtfm-dir '(ds mincho))
;  (setq i-file-dir '(ds mincho))
;  (make-dm 46. 10. 1)
;  (make-dm 66. 10. 3)
  (setq default-jgf-dir '(bs gothic))
  (setq default-jtfm-dir '(bs gothic))
  (setq i-file-dir '(js gothic))
  (make-dg 46. 10. 1)
  (make-dg 66. 10. 3))
