/*----------------------------------------------------------------------------
**   Copyright (C) 1987 Thomas Taylor
**
** This file is part of TexDraw
**
** TexDraw is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY.  No author or distributor
** accepts responsibility to anyone for the consequences of using it
** or for whether it serves any particular purpose or works at all,
** unless he says so in writing. 
**
** This software is free of charge.  It may not be re-sold.
** Everyone is granted permission to copy, modify and redistribute
** TexDraw, as long as the original copyright and this notice are
** retained.
**
**
**         Module:  texdriv.c
**
**   Program Name:  texdraw
**    Program Use:  Graphics Program for LaTeX Text Formatter
**
**         Author:  Tom Taylor
**           Date:  August 21, 1987
**
**          Class:  C Sci 498, Senior Project
**        Sponsor:  Hank Levy
**
**
**    This module contains driver functions for the texdraw program.
**
----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
**   Copyright (C) 1988 Hiroto Kagotani
----------------------------------------------------------------------------*/

#include "texdraw.h"

char *programName = NULL;
char *displayName = NULL;



/*----------------------------------------------------------------------------
  local (to this module) variables:
----------------------------------------------------------------------------*/
#define STRINGSIZE 81
#define MAXSTRING  STRINGSIZE-1

#define NAMELEN    20

static int currCmd = NONE;
static int mode = NONE;
static int  quitCount = 0;

extern int snapToGrid;
/***
extern int grid;
***/

char prevch = SP;

/*----------------------------------------------------------------------------
  Main:
    1.  Loop forever, looking for an EnterWindow event.
    2.  Call appropriate function for the entered window.
----------------------------------------------------------------------------*/

main(argc, argv)
int argc;
char **argv;
{
  XEvent event;
  int menuNum;

  account();

  programName = rindex(*argv, '/') ? rindex(*argv, '/') + 1 : *argv;

  while (--argc > 0) {
    if (**++argv == '-') {
      if (!strcmp(*argv, "-display") && argc > 1) {
        displayName = *++argv;
	argc --;
      }
      /***
      else if (!strcmp(*argv, "-grid")) grid = 1;
      ***/
    }
  }
  wind_Initialize();

  XFlush(display);
  while (quitCount < 2) {
    if (quitCount > 0) {
      quitCount = 0;
      wind_UnselectMenu(QUIT);
    }
    XSync(display, 0);
    XNextEvent(display, &event);

    if (event.type == Expose)
      wind_HandleExposure(event);

    else if (event.type == EnterNotify) {
      if (VALID(menuNum = wind_LookupMenuNum(event.xcrossing.window))) {
        driv_MenuSelectCmd(menuNum);
        switch (currCmd) {

          /* These commands execute each time the mouse button */
          /* is pressed while in the menu window.              */
          case ALIGNH:
          case ALIGNV:
          case CLEAR:
          case DELETECURRENT:
#ifdef DUMPDATA
          case DUMPDATA:
#endif
          case FONTS:
          case GRAVITY:
	  case GRID:
	  case GRIDSIZE:
	  case SNAPGRID:
          case QUIT:
          case READ:
          case REDRAW:
          case SHOWCURRENT:
          case THICK:
          case UNDO:
          case WRITE:
            driv_RepeatCommand(currCmd);
            break;

	  }
      }
      else if (event.xcrossing.window == picWind)
        switch (mode) {
          /* These commands use "rubber-banding" to define or modify     */
          /* objects in the picture window.                              */
          case BOX:
          case CIRCLE:
          case DISK:
          case LINE:
          case OVAL:
          case RULE:
          case VECTOR:
          case MOVEORIGIN:
          case MOVECURRENT:
          case MOVEPOINT:
          case COPYCURRENT:
          case SELECTCURRENT:
          case RESIZE:
            driv_RubberBand(mode);
            break;

          /* This command accepts text entry from the keyboard into the  */
          /* picture window                                              */
          case TEXT:
            driv_EnterPicText();
            break;
        }
    }
  }
  XFlush(display);
  wind_Finalize();
}




/*----------------------------------------------------------------------------
  account:
----------------------------------------------------------------------------*/
account()
{
  int oumask;

#ifdef CS_TITECH
  oumask = umask(000);
  system("/bin/date >> /usr/local/misc/texdraw/`/usr/ucb/whoami`");
  umask(oumask);
#endif
}




/*----------------------------------------------------------------------------
  driv_MenuSelectCmd:
    This function is called whenever a menu window is entered.
    Look for a button to be pressed.  If a button is pressed, it means this
    menu has been selected, so set the global variable "currCmd" to "menuNum".
    If no button is pressed, just return.
----------------------------------------------------------------------------*/

void
driv_MenuSelectCmd(menuNum)
int menuNum;
{
  XEvent event;
  bool done = FALSE;

  while(!done) {
    XSync(display, 0);
    XNextEvent(display, &event);
    switch (event.type) {
      case Expose:
        wind_HandleExposure(event);
	break;

      /* If the left mouse button is pressed, set the global variable  */
      /* to the number of the menu in which the cursor is located.     */
      case ButtonPress:
        if (driv_IsLeftButton(event)) {

          switch (menuNum) {
            /* The following cases select the primary mode for mouse    */
            /* activity in the picture window.  Only one of the primary */
            /* modes may be active at a time, and its menu window will  */
            /* have a shaded background.                                */
            case BOX:
            case CIRCLE:
            case COPYCURRENT:
            case DISK:
            case LINE:
            case OVAL:
            case RULE:
            case MOVEORIGIN:
            case MOVECURRENT:
            case MOVEPOINT:
            case RESIZE:
            case QUIT:
            case SELECTCURRENT:
            case TEXT:
            case VECTOR:
              wind_UnselectMenu(mode);
              mode = currCmd = menuNum;
              wind_SelectMenu(mode);
              break;
          }

          /* The following cases do not affect the primary mode.  They  */
          /* are selected or un-selected as soon as the left mouse      */
          /* button is pressed, so this function returns.               */
          switch (menuNum) {
            case ALIGNH:
            case ALIGNV:
            case CLEAR:
#ifdef DUMPDATA
            case DUMPDATA:
#endif
            case DELETECURRENT:
            case FONTS:
            case GRAVITY:
 	    case GRID:
	    case GRIDSIZE:
	    case SNAPGRID:
            case QUIT:
            case READ:
            case REDRAW:
            case SHOWCURRENT:
            case THICK:
            case WRITE:
            case UNDO:
              currCmd = menuNum;
              done = TRUE;
              break;
          }
        }
        break;

      /* All other commands are selected only if the left mouse button    */
      /* has been pressed and the cursor is moved out of the menu window. */
      case LeaveNotify: 
        done = TRUE;
        break;
    } 
  }
}

/*----------------------------------------------------------------------------
  driv_ExecuteCommand:
    Executes a command once only.
----------------------------------------------------------------------------*/
void
driv_ExecuteCommand(cmd)
int cmd;
{
  switch (cmd) {
    case ALIGNH:         objs_ToggleHoriz();     break;
    case ALIGNV:         objs_ToggleVert();      break;
    case CLEAR:          objs_DeleteAll();       break;
    case DELETECURRENT:  objs_DeleteCurrent();   break;
#ifdef DUMPDATA
    case DUMPDATA:       objs_DumpSet();         break;
#endif
    case FONTS:          wind_ChangeFont();      break;
    case GRAVITY:        obj_ToggleGravity();    break;
    case GRID:           obj_ToggleGrid();       break;
    case GRIDSIZE:       obj_ResizeGrid();       break;
    case SNAPGRID:       obj_SnapToGrid();       break;
    case QUIT:           driv_Quit();            break;
    case READ:           objs_ReadFromFile();    break;
    case REDRAW:         objs_Redraw();          break;
    case SHOWCURRENT:    objs_ShowCurrent();     break;
    case THICK:          obj_ToggleThickness();  break;
    case UNDO:           objs_Undo();            break;
    case WRITE:          objs_WriteToFile();     break;
  }
  currCmd = NONE;
}




/*----------------------------------------------------------------------------
   driv_Quit()
----------------------------------------------------------------------------*/
void
driv_Quit()
{
  if (++quitCount < 2)
    wind_Message1("press button again to quit");
}




/*----------------------------------------------------------------------------
  driv_RepeatCommand:
    Executes a command repeatedly.
    This function is for menu commands which execute immediately on 
    selection (before leaving the menu window), and which DO 
    re-execute each time the mouse button is pressed.
----------------------------------------------------------------------------*/
void
driv_RepeatCommand(cmd)
int cmd;
{
  XEvent event;

  currCmd = NONE;
  driv_ExecuteCommand(cmd);
  while (quitCount < 2) {
    XSync(display, 0);
    XNextEvent(display, &event);
    switch (event.type) {
      case Expose:
        wind_HandleExposure(event);
	break;
      case ButtonPress:
        if (driv_IsLeftButton(event)) {
          driv_ExecuteCommand(cmd);
          break;
        }
      case LeaveNotify:
        return;
        break;
    }
  }
}




/*----------------------------------------------------------------------------
  driv_IsLeftButton:
    Returns TRUE iff "event" was a left button event.
----------------------------------------------------------------------------*/
bool
driv_IsLeftButton(event)
XEvent event;
{
  if (event.xbutton.button == Button1)
    return TRUE;
  return FALSE;
}




/*----------------------------------------------------------------------------
  driv_RubberBand:
    This function is called whenever the mouse cursor enters the picture
    window, and the primary "mode" is one of the "rubber-banding" modes.

    1.  Grab the mouse.
    2.  Accept the mouse position for start coordinates when the left button
        is pressed, and begin "rubber-banding".
    3.  Accept the mouse position for end coordinates when when the left
        button is released.
----------------------------------------------------------------------------*/
void
driv_RubberBand(mode)
int mode;
{
  int xDummy, yDummy, x1, y1, x2, y2;
  XEvent event;
  int shape = NONE;
  bool done = FALSE;
  int status, maskDummy;
  Window rootWindDummy, subWind;

  while (!done) {
    XSync(display, 0);
    XNextEvent(display, &event);
    switch (event.type) {
      case Expose:
	wind_HandleExposure(event);
	break;

      case ButtonPress:
        if ((shape == NONE) && driv_IsLeftButton(event)) {
          /* grab the mouse until the "rubber-banding" is complete */
          status = XGrabPointer(display, picWind, False,
                              ButtonPressMask|ButtonReleaseMask|
		              Button1MotionMask|LeaveWindowMask,
		              GrabModeAsync, GrabModeAsync,
		              None, smallCrossCursor, CurrentTime);
          if (status != 0) 
            fprintf(stderr,"driv_RubberBand - can not grab mouse \n");

          /* find out where the mouse is */
	  x2 = event.xbutton.x;
	  y2 = INV(event.xbutton.y);
	  if (snapToGrid) {
	    objs_SnapGrid(&x2, &y2);
	  }
	  else {
	    objs_Gravitate(&x2, &y2);
	    objs_Align(&x2, &y2);
	  }
          /* initialize x1, y1, and shape */
          objs_InitShape(mode, &shape, &x1, &y1, x2, y2);

	  if (VALID(shape)) {
            /* Find the best approximation to the mouse position  */
            /* (necessary for the case where mouse does not move) */
            geom_BestTexObject(shape, x1, y1, &x2, &y2);

            /* draw the rubber band */
            poly_Rubber(shape, x1, y1, x2, y2);
	  }
	  else {
            XUngrabPointer(display, CurrentTime);
	  }

        }
	else if (VALID(shape)) {  /* Cancel by other button */

          /* erase the rubber band */
          poly_Rubber(shape, x1, y1, x2, y2);
          shape = NONE;

          XUngrabPointer(display, CurrentTime);
	}
        break;                                     /* ButtonPress */

      case MotionNotify:
	while (XCheckTypedEvent(display, MotionNotify, &event));
        if (VALID(shape)) {

          /* erase the rubber band */
          poly_Rubber(shape, x1, y1, x2, y2);

          /* find out where the mouse is */
	  x2 = event.xbutton.x;
	  y2 = INV(event.xbutton.y);
	  if (snapToGrid) {
	    objs_SnapGrid(&x2, &y2);
	  }
	  else {
	    objs_Gravitate(&x2, &y2);
	    /*****
	      objs_Align(&x2, &y2);
	    *****/
	  }
          /* Find the best approximation to the mouse position */
          geom_BestTexObject(shape, x1, y1, &x2, &y2);

          /* redraw the rubber band */
          poly_Rubber(shape, x1, y1, x2, y2);

        }
        break;                                           /* MotionNotify */

      case ButtonRelease:
        if (VALID(shape) && driv_IsLeftButton(event)) {

          /* erase the rubber band */
          poly_Rubber(shape, x1, y1, x2, y2);

          /* update the set of objects and the display */
          objs_Update(mode, shape, x1, y1, x2, y2);
          shape = NONE;

          XUngrabPointer(display, CurrentTime);
        }
        break;                                      /* ButtonRelease */

      case LeaveNotify:
        if (shape == NONE)
          done = TRUE;
        break;                                      /* LeaveNotify    */
    }
  }
}




/*----------------------------------------------------------------------------
  PrintEvent:
    For debug purposes.
----------------------------------------------------------------------------*/

PrintEvent(event)
XEvent event;
{
  printf("\n");

  printf("EVENT  type:");
  if      (event.type == EnterNotify)    printf("EnterNotify");
  else if (event.type == LeaveNotify)    printf("LeaveNotify");
  else if (event.type == Expose)         printf("Expose");
  else if (event.type == ButtonPress)    printf("ButtonPress");
  else if (event.type == ButtonRelease)  printf("ButtonRelease");
  else if (event.type == MotionNotify)   printf("MotionNotify");
  printf("\n");

  printf("     window:");
  wind_PrintWindow(event.xany.window);
  printf("\n");
}




/*----------------------------------------------------------------------------
  driv_EnterCommand:
    Gets a text string in "str" at position (x, y) in the text window.
    Returns TRUE if a valid string is entered.
    Returns FALSE if aborted with control-C.
----------------------------------------------------------------------------*/
bool
driv_EnterCommand(x, y, str, max)
int x, y;
char str[];
int max;
{
  XFontStruct *font = textFont;
  XEvent event;
  int n, w;
  int len = 0;
  char xstr[STRINGSIZE], c;
  bool answer;
  bool done = FALSE;
  static char cursor[] = "_";

  XGrabKeyboard(display, textWind, False,
		GrabModeAsync, GrabModeAsync, CurrentTime);

  while (!done) {

    /* Null-terminate the string, and display the cursor. */
    str[len] = '\0';
    w = XTextWidth(font, str, str_Length(str));
    XDrawImageString(display, textWind, wind_LookupTextGC(font),
							x+w, y, cursor, 1);

    XSync(display, 0);
    XNextEvent(display, &event);
    switch (event.type) {
      case Expose:
	wind_HandleExposure(event);
	break;

      case KeyPress:

        /* Find out what key was pressed. Ignore multi-character sequences. */
	n = XLookupString(&event.xkey, xstr, sizeof(xstr), NULL, NULL);
        if (n != 1) break;
        c = xstr[0];

        /* If the keypress was ctrl-C, abort. */
        if (c == CTRL_C) {
          str[0] = '\0';
          answer = FALSE;
          done = TRUE;
        }

        /* If the key pressed was a carriage return, then get outa here. */
        if (c == LF || c == CR) {
          answer = TRUE;
          done = TRUE;
          break;
        }

        /* If the key pressed was backspace, erase the cursor and */
        /* the last character.                                    */
        if (len && (c == DEL || c == BS)) {
          XDrawImageString(display, textWind, wind_LookupTextEraseGC(font),
							x+w, y, cursor, 1);
          len--;
          str[len] = '\0';
          w = XTextWidth(font, str, str_Length(str));
          XDrawImageString(display, textWind, wind_LookupTextGC(font),
							x+w, y, cursor, 1);
          break;
        }
        /* Don't accept other non-printing characters. */
        if (! isprint(xstr[0])) break;

        /* Don't exceed the maximum string length. */
        if (len >= max) break;

        /* Erase the cursor. */
        XDrawImageString(display, textWind, wind_LookupTextEraseGC(font),
							x+w, y, cursor, 1);

        /* Add the character to the string, and display it. */
        str[len++] = c;
        str[len] = '\0';
        XDrawImageString(display, textWind, wind_LookupTextGC(font),
							x+w, y, xstr, 1);
        break;
    }
  }
  XUngrabKeyboard(display, CurrentTime);
  return answer;
}




/*----------------------------------------------------------------------------
  driv_EnterPicText:
----------------------------------------------------------------------------*/
void
driv_EnterPicText()
{
  XEvent event;
  Window rootWindDummy, subWind;
  XFontStruct *font = picFont;
  int xDummy, yDummy, x=0, y=0, w=0, n;
  int len = 0;
  char str[STRINGSIZE], xstr[STRINGSIZE], c;
  bool typing = FALSE;
  bool done   = FALSE;
  static char cursor[] = "_";
  int maskDummy;

  XGrabKeyboard(display, textWind, False,
		GrabModeAsync, GrabModeAsync, CurrentTime);

  while (!done) {
    XSync(display, 0);
    XNextEvent(display, &event);
    switch (event.type) {
      case Expose:
	wind_HandleExposure(event);
	break;

      case ButtonPress:

        /* Add the current string to the set of objects */
        /* and erase the cursor.                        */
        if (typing) {
          objs_AddText(TEXT, x-scrXofs, y-scrYofs, str, len, font);
          XDrawImageString(display, picWind, wind_LookupTextEraseGC(font),
							x+w, INV(y), cursor, 1);
        }

        typing = TRUE;

        /* Start a new string. */
        len = 0;
        str[0] = '\0';
        w = 0;
        prevch = SP;
	x = event.xbutton.x;
	y = INV(event.xbutton.y);
	if (snapToGrid) {
	  objs_SnapGrid(&x, &y);
	}
        x += scrXofs;
        y += scrYofs;

        /* Display an "underline" cursor at the start of the string. */
        XDrawImageString(display, picWind, wind_LookupTextGC(font),
							x, INV(y), cursor, 1);

        break;

      case KeyPress:

        /* Don't start a string until x and y are known. */
        if (!typing) break;

        /* Find out what key was pressed. Ignore multi-character sequences. */
	n = XLookupString(&event.xkey, xstr, sizeof(xstr), NULL, NULL);

        if (n != 1) break;
        c = xstr[0];

        /* If the key pressed was backspace, erase cursor. */
        /* and the last character.                         */
        if (len && (c == DEL || c == BS)) {
          XDrawImageString(display, picWind, wind_LookupTextEraseGC(font),
							x+w, INV(y), cursor, 1);
          len--;
          str[len] = '\0';
          w = XTextWidth(font, str, str_Length(str));
          XDrawImageString(display, picWind, wind_LookupTextGC(font),
							x+w, INV(y), cursor, 1);
          break;
        }

        /* Don't accept other non-printing characters. */
        if (! isprint(xstr[0])) break;

        /* Make sure it is a valid LaTeX string character. */
        if (!read_IsInputStringChar(c)) break;

        /* Don't exceed the maximum string length. */
        if (len >= MAXSTRING) break;

        /* Don't accept leading or multiple spaces */
        /* since LaTeX does not print them.        */
        if (c == SP && prevch == SP) break;

        /* Erase the cursor. */
        XDrawImageString(display, picWind, wind_LookupTextEraseGC(font),
							x+w, INV(y), cursor, 1);

        /* Add the character to the string, and display it. */
        prevch = c;
        str[len++] = c;
        XDrawImageString(display, picWind, wind_LookupTextGC(font),
							x+w, INV(y), xstr, 1);

        /* Null-terminate the string, and display the cursor. */
        str[len] = '\0';
        w = XTextWidth(font, str, str_Length(str));
        XDrawImageString(display, picWind, wind_LookupTextGC(font),
							x+w, INV(y), cursor, 1);

        break;

      case LeaveNotify:

        /* Add the current string to the set of objects */
        /* and erase the cursor.                        */
        if (typing) {
          objs_AddText(TEXT, x-scrXofs, y-scrYofs, str, len, font);
          XDrawImageString(display, picWind, wind_LookupTextEraseGC(font),
							x+w, INV(y), cursor, 1);
        }

        /* Get outa here. */
        done = TRUE;
        break;

    }
  }
  XUngrabKeyboard(display, CurrentTime);
}

