/*----------------------------------------------------------------------------
**   Copyright (C) 1988 Hiroto Kagotani
**
** This file is part of TexDraw
**
** TexDraw is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY.  No author or distributor
** accepts responsibility to anyone for the consequences of using it
** or for whether it serves any particular purpose or works at all,
** unless he says so in writing. 
**
** This software is free of charge.  It may not be re-sold.
** Everyone is granted permission to copy, modify and redistribute
** TexDraw, as long as the original copyright and this notice are
** retained.
**
**
**         Module:  texgc.c
**
**   Program Name:  texdraw
**    Program Use:  Graphics Program for LaTeX Text Formatter
**
**         Author:  Hiroto Kagotani
**
**
**    This module contains GC functions for the texdraw program.
**
----------------------------------------------------------------------------*/


#include "texdraw.h"


/*----------------------------------------------------------------------------
  global (to all modules) variables:
----------------------------------------------------------------------------*/
GC filledGC;
GC filledEraseGC;
GC gridGC;
GC selectGC;
GC thickGC;
GC thickEraseGC;
GC thinGC;
GC thinEraseGC;
GC tiledGC;
GC xorDashedGC;
GC xorDottedGC;


/* --------------------------------------------------------------------------
   gc_MakeFilled:
---------------------------------------------------------------------------*/
void
gc_MakeFilled()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  filledGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeFilledErase:
---------------------------------------------------------------------------*/
void
gc_MakeFilledErase()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = back_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  filledEraseGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeGrid:
---------------------------------------------------------------------------*/
void
gc_MakeGrid()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.line_style = LineOnOffDash;
  gcmask |= GCLineStyle;

  gcv.dashes = 1;
  gcmask |= GCDashList;

  gridGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeSelect:
---------------------------------------------------------------------------*/
void
gc_MakeSelect()  
{
  int w = 16, h = 16, d = DefaultDepth(display, screen);
  static char selectPattern[] = {
   0x88, 0x88, 0x00, 0x00, 0x22, 0x22, 0x00, 0x00,
   0x88, 0x88, 0x00, 0x00, 0x22, 0x22, 0x00, 0x00,
   0x88, 0x88, 0x00, 0x00, 0x22, 0x22, 0x00, 0x00,
   0x88, 0x88, 0x00, 0x00, 0x22, 0x22, 0x00, 0x00};
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.fill_style = FillTiled;
  gcmask |= GCFillStyle;

  if (DefaultDepth(display, screen) > 1) {
    Pixmap srcP = XCreateBitmapFromData(display, rootWind, selectPattern, w, h);
    Pixmap destP = XCreatePixmap(display, rootWind, w, h, d);
    XCopyPlane(display, srcP, destP, DefaultGC(display, screen),
		0, 0, w, h, 0, 0, 1);
    gcv.tile = destP;
  }
  else {
    gcv.tile = XCreateBitmapFromData(display, rootWind, selectPattern, w, h);
  }
  gcmask |= GCTile;

  selectGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeTextErase:
---------------------------------------------------------------------------*/
GC
gc_MakeTextErase(fid)  
Font fid;
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = back_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.font = fid;
  gcmask |= GCFont;

  return XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeTextWrite:
---------------------------------------------------------------------------*/
GC
gc_MakeTextWrite(fid)  
Font fid;
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.font = fid;
  gcmask |= GCFont;

  return XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeThick:
---------------------------------------------------------------------------*/
void
gc_MakeThick()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.line_width = 2;
  gcmask |= GCLineWidth;

  thickGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeThickErase:
---------------------------------------------------------------------------*/
void
gc_MakeThickErase()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = back_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.line_width = 2;
  gcmask |= GCLineWidth;

  thickEraseGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeThin:
---------------------------------------------------------------------------*/
void
gc_MakeThin()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  thinGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeThinErase:
---------------------------------------------------------------------------*/
void
gc_MakeThinErase()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = back_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  thinEraseGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeTiled:
---------------------------------------------------------------------------*/
void
gc_MakeTiled()  
{
  int w = 8, h = 8, d = DefaultDepth(display, screen);
  static char tiledPattern[] = {
   0x77, 0xFF, 0xDD, 0xFF, 0x77, 0xFF, 0xDD, 0xFF};
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXcopy;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.fill_style = FillTiled;
  gcmask |= GCFillStyle;

  if (DefaultDepth(display, screen) > 1) {
    Pixmap srcP = XCreateBitmapFromData(display, rootWind, tiledPattern, w, h);
    Pixmap destP = XCreatePixmap(display, rootWind, w, h, d);
    XCopyPlane(display, srcP, destP, DefaultGC(display, screen),
		0, 0, w, h, 0, 0, 1);
    gcv.tile = destP;
  }
  else {
    gcv.tile = XCreateBitmapFromData(display, rootWind, tiledPattern, w, h);
  }
  gcmask |= GCTile;

  tiledGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeXorDashed:
---------------------------------------------------------------------------*/
void
gc_MakeXorDashed()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXxor;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.line_style = LineOnOffDash;
  gcmask |= GCLineStyle;

  gcv.dashes = 4;
  gcmask |= GCDashList;

  xorDashedGC = XCreateGC(display, rootWind, gcmask, &gcv);
}


/* --------------------------------------------------------------------------
   gc_MakeXorDotted:
---------------------------------------------------------------------------*/
void
gc_MakeXorDotted()  
{
  XGCValues gcv;
  int gcmask = 0;

  gcv.function = GXxor;
  gcmask |= GCFunction;

  gcv.foreground = fore_pixel;
  gcmask |= GCForeground;

  gcv.background = back_pixel;
  gcmask |= GCBackground;

  gcv.line_style = LineOnOffDash;
  gcmask |= GCLineStyle;

  gcv.dashes = 1;
  gcmask |= GCDashList;

  xorDottedGC = XCreateGC(display, rootWind, gcmask, &gcv);
}
