/*----------------------------------------------------------------------------
**   Copyright (C) 1987 Thomas Taylor
**
** This file is part of TexDraw
**
** TexDraw is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY.  No author or distributor
** accepts responsibility to anyone for the consequences of using it
** or for whether it serves any particular purpose or works at all,
** unless he says so in writing. 
**
** This software is free of charge.  It may not be re-sold.
** Everyone is granted permission to copy, modify and redistribute
** TexDraw, as long as the original copyright and this notice are
** retained.
**
**
**         Module:  texpoly.c
**
**   Program Name:  texdraw
**    Program Use:  Graphics Program for LaTeX Text Formatter
**
**         Author:  Tom Taylor
**           Date:  August 21, 1987
**
**          Class:  C Sci 498, Senior Project
**        Sponsor:  Hank Levy
**
**
**    This module contains polygon functions for the texdraw program.
**
----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
**   Copyright (C) 1988 Hiroto Kagotani
----------------------------------------------------------------------------*/


#include "texdraw.h"




/*----------------------------------------------------------------------------
  poly_Rubber:
----------------------------------------------------------------------------*/
void
poly_Rubber(shape, x1, y1, x2, y2)
int shape, x1, y1, x2, y2;
{
  Object obj;

  switch (shape) {
    case BOX:
    case RULE:
    case OVAL:
    case SELECTCURRENT:
      obj_CreateGeom(BOX, x1, y1, x2, y2, &obj);
      poly_Box(obj, xorDottedGC);
      break;
    case CIRCLE:
    case DISK:
      obj_CreateGeom(CIRCLE, x1, y1, x2, y2, &obj);
      poly_Circle(obj, xorDottedGC);
      break;
    case LINE:
    case VECTOR:
      obj_CreateGeom(LINE, x1, y1, x2, y2, &obj);
      poly_Line(obj, xorDottedGC);
      break;
    case COPYCURRENT:
    case MOVECURRENT:
      obj_CreateGeom(MOVECURRENT, x1, y1, x2, y2, &obj);
      poly_Line(obj, xorDottedGC);
      break;
    case RESIZE:
      obj_CreateGeom(BOX, x1, y1, x2, y2, &obj);
      poly_Box(obj, xorDashedGC);
      poly_BoxOnGraph(obj, xorDashedGC);
      break;
  }
}




/*----------------------------------------------------------------------------
  poly_BoxOnGraph:
----------------------------------------------------------------------------*/
void
poly_BoxOnGraph(obj, gc)
Object obj;
GC gc;
{
  Window dummyWind;
  int x;
  int y;

  XTranslateCoordinates(display, picWind, graphWind,
		obj.x, INV(obj.y + obj.kind.box.h), &x, &y, &dummyWind );

  XDrawRectangle(display, graphWind, gc, x, y, obj.kind.box.w, obj.kind.box.h);
}




/*----------------------------------------------------------------------------
  poly_Box:
----------------------------------------------------------------------------*/
void
poly_Box(obj, gc)
Object obj;
GC gc;
{
  XDrawRectangle(display, picWind, gc,
	obj.x, INV(obj.y + obj.kind.box.h), obj.kind.box.w, obj.kind.box.h);
}




/*----------------------------------------------------------------------------
  poly_Circle:
----------------------------------------------------------------------------*/
void
poly_Circle(obj, gc)
Object obj;
GC gc;
{
  register d = obj.kind.circle.dia;
  register x = obj.x - d / 2;                 /*  left bound of this circle */
  register y = INV(obj.y) - d / 2;            /* upper bound of this circle */

  XDrawArc(display, picWind, gc, x, y, d, d, 0, 360*64);
}




/*----------------------------------------------------------------------------
  poly_Disk:
----------------------------------------------------------------------------*/
void
poly_Disk(obj, gc)
Object obj;
GC gc;
{
  register d = obj.kind.disk.dia;
  register x = obj.x - d / 2;                   /*  left bound of this disk */
  register y = INV(obj.y) - d / 2;              /* upper bound of this disk */

  XDrawArc(display, picWind, thickGC, x, y, d, d, 0, 360*64);
  XFillArc(display, picWind, gc, x, y, d, d, 0, 360*64);
}




/*----------------------------------------------------------------------------
  poly_Line:
----------------------------------------------------------------------------*/
void
poly_Line(obj, gc)
Object obj;
GC gc;
{
  register x1 = obj.x;
  register y1 = INV(obj.y);
  register x2 = obj.x + obj.kind.line.dx;
  register y2 = INV(obj.y + obj.kind.line.dy);

  XDrawLine(display, picWind, gc, x1, y1, x2, y2);
}




/*----------------------------------------------------------------------------
  poly_Oval:
----------------------------------------------------------------------------*/
void
poly_Oval(obj, gc)
Object obj;
GC gc;
{
  XSegment oval_seg[4];
  XArc oval_arc[4];
  register l = MIN(obj.x, obj.x + obj.kind.oval.w);
  register r = MAX(obj.x, obj.x + obj.kind.oval.w);
  register t = INV(MAX(obj.y, obj.y + obj.kind.oval.h));
  register b = INV(MIN(obj.y, obj.y + obj.kind.oval.h));
  int rad;
  int dia;

  rad = MIN(r-l, b-t) / 2;
  geom_GraphToTexRad(CIRCLE, &rad);
  dia = 2 * rad;

  oval_seg[0].x1 = l + rad;	/* top line */
  oval_seg[0].y1 = t;
  oval_seg[0].x2 = r - rad;
  oval_seg[0].y2 = t;

  oval_seg[1].x1 = l + rad;	/* bottom line */
  oval_seg[1].y1 = b;
  oval_seg[1].x2 = r - rad;
  oval_seg[1].y2 = b;

  oval_seg[2].x1 = l;		/* left line */
  oval_seg[2].y1 = t + rad;
  oval_seg[2].x2 = l;
  oval_seg[2].y2 = b - rad;

  oval_seg[3].x1 = r;		/* right line */
  oval_seg[3].y1 = t + rad;
  oval_seg[3].x2 = r;
  oval_seg[3].y2 = b - rad;

  oval_arc[0].x = l;		/* left-top corner */
  oval_arc[0].y = t;
  oval_arc[0].width = dia;
  oval_arc[0].height = dia;
  oval_arc[0].angle1 = 90*64;
  oval_arc[0].angle2 = 90*64;

  oval_arc[1].x = r - dia;	/* right-top corner */
  oval_arc[1].y = t;
  oval_arc[1].width = dia;
  oval_arc[1].height = dia;
  oval_arc[1].angle1 = 0*64;
  oval_arc[1].angle2 = 90*64;

  oval_arc[2].x = l;		/* left-bottom corner */
  oval_arc[2].y = b - dia;
  oval_arc[2].width = dia;
  oval_arc[2].height = dia;
  oval_arc[2].angle1 = 180*64;
  oval_arc[2].angle2 = 90*64;

  oval_arc[3].x = r - dia;	/* right-bottom corner */
  oval_arc[3].y = b - dia;
  oval_arc[3].width = dia;
  oval_arc[3].height = dia;
  oval_arc[3].angle1 = 270*64;
  oval_arc[3].angle2 = 90*64;

  XDrawSegments(display, picWind, gc, oval_seg, 4);
  XDrawArcs(display, picWind, gc, oval_arc, 4);
}




/*----------------------------------------------------------------------------
  poly_Rule:
----------------------------------------------------------------------------*/
void
poly_Rule(obj, gc)
Object obj;
GC gc;
{
  XFillRectangle(display, picWind, gc,
	obj.x, INV(obj.y + obj.kind.rule.h), obj.kind.rule.w, obj.kind.rule.h);
}




/*----------------------------------------------------------------------------
  poly_Vector:
----------------------------------------------------------------------------*/
void
poly_Vector(obj, gc)
Object obj;
GC gc;
{
  XPoint arrow[4];
  register x1 = obj.x;
  register y1 = INV(obj.y);
  register dx = obj.kind.vector.dx;
  register dy = -obj.kind.vector.dy;
  register x2 = x1 + dx;
  register y2 = y1 + dy;
  double len = LENGTH(dx, dy);
  double cos = len != 0.0 ? dx / len : 0;
  double sin = len != 0.0 ? dy / len : 0;

  arrow[0].x = x2;
  arrow[0].y = y2;
  arrow[1].x = x2 - ROUND(6*cos + 2*sin);
  arrow[1].y = y2 - ROUND(6*sin - 2*cos);
  arrow[2].x = x2 - ROUND(6*cos - 2*sin);
  arrow[2].y = y2 - ROUND(6*sin + 2*cos);
  arrow[3].x = x2;
  arrow[3].y = y2;

  XDrawLine(display, picWind, gc, x1, y1, x2, y2);
  XDrawLines(display, picWind, gc, arrow, 4, CoordModeOrigin);
  XFillPolygon(display, picWind, tiledGC, arrow, 4, Convex, CoordModeOrigin);
}
